
/*
 * @(#)AbstractTokenProxy.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data abstract client-side Token proxy interface.
 *
 * @version     0.94 3 April 1997
 * @author      Rich Burridge
 */

public interface
AbstractTokenProxy extends AbstractSharedDataProxy {

/**
 * <A NAME="SD_GRAB"></A>
 * <EM>grab</EM>
 *
 * @param <EM>client</EM> the client grabbing/inhibiting this token.
 * @param <EM>exclusive</EM> indicates whether the grab should be exclusive.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN  - token given does not exist.
 *      NO_SUCH_CLIENT - client given does not exist.
 * </PRE>
 *
 * @return the status of the token.
 */

    int
    grab(Client client, boolean exclusive) throws SharedDataException;

/**
 * <A NAME="SD_GIVE"></A>
 * <EM>give</EM>
 *
 * @param <EM>client</EM> the client giving this token.
 * @param <EM>receivingClientName</EM> the name of the client to receive
 * the token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN  - token given does not exist.
 *      NO_SUCH_CLIENT - client given does not exist.
 * </PRE>
 *
 * @return the status of the token.
 */

    int
    give(Client client,
         String receivingClientName) throws SharedDataException;

/**
 * <A NAME="SD_PLEASE"></A>
 * <EM>please</EM>
 *
 * @param <EM>client</EM> the client requesting this token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN - token given does not exist.
 * </PRE>
 *
 * @return the status of the token.
 */

    int
    please(Client client) throws SharedDataException;

/**
 * <A NAME="SD_RELEASE"></A>
 * <EM>release</EM>
 *
 * @param <EM>client</EM> the client releasing this token.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN       - token given does not exist.
 *      NO_SUCH_CLIENT      - client given does not exist.
 *      CLIENT_NOT_GRABBING - this client is not grabbing this token.
 *      CLIENT_NOT_RELEASED - the token was not released successfully.
 * </PRE>
 *
 * @return the status of the token.
 */

    int
    release(Client client) throws SharedDataException;

/**
 * <A NAME="SD_TEST"></A>
 * <EM>test</EM>
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 * <PRE>
 * The following exception types can occur:
 *      NO_SUCH_TOKEN - token given does not exist.
 * </PRE>
 *
 * @return one of the following:
 * <PRE>
 * NOT_IN_USE : not in use
 * GRABBED    : grabbed
 * INHIBITED  : inhibited
 * GIVING     : giving
 * </PRE>
 */

    int
    test() throws SharedDataException;
}
