
/*
 * @(#)ChannelObserver.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data Channel Observer interface.
 *
 * @version     0.82 3 October 1996
 * @author      Rich Burridge
 */

public interface
ChannelObserver extends SharedDataObserver {

/**
 * <A NAME="SDO_CHANNELCHANGED"></A>
 * <EM>channelChanged</EM> is called when a channel change has been observed.
 *
 * There are two types of channel change:
 * - a client has joined the channel.
 * - a client has left the channel.
 *
 * @param <EM>event</EM> the channel changed event containing more information
 * on exactly what has changed.
 *
 * This method could look something like:
 * <pre>
 *
 *    public void
 *    channelChanged(ChannelEvent e) {
 *        Session session    = e.getSession();
 *        String  clientName = e.getClientName();
 *        Channel channel    = e.getChannel();
 *        int     type       = e.getType();
 *
 *        if (type == ChannelEvent.JOIN) {
 *
 *            // deal with channel join...
 *
 *        } else if (type == ChannelEvent.LEAVE) {
 *
 *            // deal with channel leave...
 *
 *        }
 *    }
 *
 * </pre>
 */
 
    public void
    channelChanged(ChannelEvent event);
}
