
/*
 * @(#)Data.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;

/**
 * Shared Data Data class.
 *
 * @version     0.89 29 January 1997
 * @author      Rich Burridge
 */

public
class Data extends SharedDataObject {

    /* The length (in bytes) of the raw data. */
    private int length;

    /** The raw data. */
    private byte[] data;

    /** The priority that this data is being sent/received at. */
    private int priority = Channel.MEDIUM_PRIORITY;

    /** The name of the sender. */
    private String senderName = null;

    /** The channel that the data is being sent over. */
    private Channel channel = null;


/**
 * <A NAME="SD_DATA"></A>
 * <EM>Data</EM> is a constructor for the Data class. An array of bytes is
 * turned into a Data object. The length of the data array is the length of
 * the data object.
 *
 * @param <EM>data</EM> the data (an array of bytes).
 */

    public
    Data(byte[] data) {
        if (Data_Debug) {
            System.out.println("Data: constructor."); 
        }

        this.data = data;
	length = data.length;
    }


/**
 * <A NAME="SD_DATAL"></A>
 * <EM>Data</EM> is a constructor for the Data class. An array of bytes is
 * turned into a Data object. The length of the data object is provided. If
 * this value is invalid, then the length of the data object is the length of
 * the array of bytes.
 *
 * @param <EM>data</EM> the data (an array of bytes).
 * @param <EM>length</EM> the length of the data.
 */

    public
    Data(byte[] data, int length) {
        if (Data_Debug) {
            System.out.println("Data: constructor " +
                               " Data " + data +
                               " Length " + length);
        }

        this.data = data;
	if (length > 0 || length <= data.length) {
            this.length = length;
        } else {
            this.length = data.length;
        }
    }


/**
 * <A NAME="SD_DATAS"></A>
 * <EM>Data</EM> is a constructor for the Data class. An String is turned
 * into a Data object. The length of the data array is the length of
 * the String object.
 *
 * @param <EM>string</EM> the data (a String).
 */
 
    public
    Data(String string) {
        if (Data_Debug) {
            System.out.println("Data: constructor " +
                               " String " + string);
        }
 
        length = string.length();
        data = new byte[length];
        string.getBytes(0, length, data, 0);
    }


/**
 * <A NAME="SD_GETDATA"></A>
 * <EM>getData</EM> get the data associated with this Data object.
 *
 * @return the data (an array of bytes) contained in this Data object.
 */

    public byte[]
    getData() {
        if (Data_Debug) {
            System.out.println("Data: getData: " + data); 
        }

	return(data);
    }


/**
 * <A NAME="SD_GETLENGTH"></A>
 * <EM>getLength</EM> get the length of the data in this Data object.
 *
 * @return the length (in bytes) of the data in this Data object.
 */

    public int
    getLength() {
        if (Data_Debug) {
            System.out.println("Data: getLength: " + length); 
        }

	return(length);
    }


/**
 * <A NAME="SD_GETSENDERNAME"></A>
 * <EM>getSenderName</EM> get the name of the Data sender.
 *
 * @return the name of the sender of this Data.
 */

    public String
    getSenderName() {
        if (Data_Debug) {
            System.out.println("Data: getSenderName: " + senderName); 
        }

	return(senderName);
    }


/**
 * <A NAME="SD_GETPRIORITY"></A>
 * <EM>getPriority</EM> gets the priority that this Data was sent at.
 *
 * @return the priority that this Data was sent at.
 */

    public int
    getPriority() {
        if (Data_Debug) {
            System.out.println("Data: getPriority: " + priority);
        }

	return(priority);
    }


/**
 * <A NAME="SD_SETPRIORITY"></A>
 * <EM>setPriority</EM> set a new priority value for this data.
 *
 * @param <EM>priority</EM> the new priority value.
 */

    public void
    setPriority(int priority) {
        if (Data_Debug) {
            System.out.println("Data: setPriority: " + priority); 
        }

	if (priority < Channel.TOP_PRIORITY) {
	    this.priority = Channel.TOP_PRIORITY;
	} else if (priority > Channel.LOW_PRIORITY) {
	    this.priority = Channel.LOW_PRIORITY;
	} else {
	    this.priority = priority;
	}
    }


/**
 * <A NAME="SD_GETCHANNEL"></A>
 * <EM>getChannel</EM> get the Channel that this data was sent over.
 *
 * @return the Channel that this Data was sent over.
 */

    public Channel
    getChannel() {
        if (Data_Debug) {
            System.out.println("Data: getChannel: " + channel); 
        }

	return(channel);
    }


/* Private/protected/package methods. */

/**
 * <A NAME="SD__SETCHANNEL"></A>
 * <EM>_setChannel</EM> set a new channel value for this data.
 *
 * @param <EM>channel</EM> the new channel value.
 */

    public void
    _setChannel(Channel channel) {
        if (Data_Debug) {
            System.out.println("Data: _setChannel " + channel);
        }

	this.channel = channel;
    }


/**
 * <A NAME="SD__SETSENDERNAME"></A>
 * <EM>_setSenderName</EM> set a new sender name for this data.
 *
 * @param <EM>senderName</EM> the new sender name.
 */

    public void
    _setSenderName(String senderName) {
        if (Data_Debug) {
            System.out.println("Data: _setSenderName " + senderName);
        }

	this.senderName = senderName;
    }
}
