
/*
 * @(#)SessionObserver.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data Session Observer interface.
 *
 * @version     0.82 3 October 1996
 * @author      Rich Burridge
 */

public interface
SessionObserver extends SharedDataObserver {

/**
 * <A NAME="SD_SESSIONCHANGED"></A>
 * <EM>sessionChanged</EM> is called when a session change has been observed.
 *
 * There are two types of session change:
 * - a client has joined a session.
 * - a client has left a session.
 *
 * @param <EM>event</EM> the session changed event containing more information
 * on exactly what has changed.
 *
 * This method could look something like:
 * <pre>
 *
 *    public void
 *    sessionChanged(SessionEvent e) {
 *        String  clientName = e.getClientName();
 *        Session session    = e.getSession();
 *        int     type       = e.getType();
 *
 *        if (type == SessionEvent.JOIN) {
 *
 *            // deal with session join...
 *
 *        } else if (type == SessionEvent.LEAVE) {
 *
 *            // deal with session leave...
 *        }
 *    }
 *
 * </pre>
 */

    public void
    sessionChanged(SessionEvent event);
}
