
/*
 * @(#)SharedDataException.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

/**
 * Shared Data SharedDataException class.
 *
 * @version     0.87 17 December 1996
 * @author      Rich Burridge
 */

public class
SharedDataException extends Exception implements SharedDataDebugFlags {

    /** The various types of Shared Data exceptions that can be thrown. */
    public static final int UNKNOWN             = 1000;
    public static final int NO_SUCH_SESSION     = 1001;
    public static final int NO_SUCH_CHANNEL     = 1002;
    public static final int NO_SUCH_TOKEN       = 1003;
    public static final int NO_SUCH_CLIENT      = 1004;
    public static final int NO_SUCH_CONSUMER    = 1005;
    public static final int NO_SUCH_OBSERVER    = 1006; 
    public static final int CHANNEL_EXISTS      = 1007;
    public static final int TOKEN_EXISTS        = 1008; 
    public static final int MANAGER_EXISTS      = 1009;
    public static final int PERMISSION_DENIED   = 1010;
    public static final int CLIENT_NOT_GRABBING = 1011;
    public static final int CLIENT_NOT_RELEASED = 1012;
    public static final int INVALID_URL         = 1013;
    public static final int NO_REGISTRY         = 1014;
    public static final int ALREADY_BOUND       = 1015;
    public static final int NOT_BOUND           = 1016;
    public static final int NAME_IN_USE         = 1017;
    public static final int NO_SUCH_VALUE       = 1018;
    public static final int INCORRECT_LENGTH    = 1019;

    /** The type of this Shared Data exception. */
    int type;


/**
 * <A NAME="SD_EXCEPTION"></A>
 * <EM>SharedDataException</EM> is throw if there is any kind of error in
 * processing a Shared Data request.
 */

    public
    SharedDataException() {
        super();
 
        if (SharedDataException_Debug) {
            System.out.println("SharedDataException: constructor.");
        }
	type = UNKNOWN;
    }


/**
 * <A NAME="SD_EXCEPTIONBYNAME"></A>
 * <EM>SharedDataException</EM> is throw if there is any kind of error in
 * processing a Shared Data request.
 *
 * @param <EM>s</EM> a string describing the exception thrown.
 */

    public
    SharedDataException(String s) {
        super(s);

	if (SharedDataException_Debug) {
            System.out.println("SharedDataException: constructor: " + s);
        }

	type = UNKNOWN;
    }


/**
 * <A NAME="SD_EXCEPTIONBYTYPE"></A>
 * <EM>SharedDataException</EM> is throw if there is any kind of error in
 * processing a Shared Data request.
 *
 * @param <EM>type</EM> the type of this Shared Data exception.
 */
 
    public
    SharedDataException(int type) {
        super();
 
        if (SharedDataException_Debug) {
            System.out.println("SharedDataException: constructor:" +
			       " type: " + type);
        }

        this.type = type;
    }


/**
 * <A NAME="SD_GETTYPE"></A>
 * <EM>getType</EM> the the type of this Shared Data exception.
 *
 * @return the type of this Shared Data exception.
 */

    public int
    getType() {
        if (SharedDataException_Debug) {
            System.out.println("SharedDataException: getType:" +
                               " type: " + type);
        }

	return(type);
    }


/**
 * <A NAME="SD_TYPETOSTRING"></A>
 * <EM>typeToString</EM> converts an exception type into a string
 *  describing it.
 *
 * @param <EM>type</EM> the exception type.
 *
 * @return a String describing this type of exception.
 */

    public String
    typeToString(int type) {
        if (SharedDataException_Debug) {
            System.out.println("SharedDataException: typeToString:" +
                               " type: " + type);
        }

        switch (type) {
	    case NO_SUCH_SESSION:
		return("no such session");
	    case NO_SUCH_CHANNEL:
		return("no such channel");
	    case NO_SUCH_TOKEN:
		return("no such token");
	    case NO_SUCH_CLIENT:
		return("no such client");
	    case NO_SUCH_OBSERVER:
		return("no such observer");
	    case CHANNEL_EXISTS:
		return("channel exists");
	    case TOKEN_EXISTS:
		return("token exists");
	    case MANAGER_EXISTS:
		return("manager exists");
	    case PERMISSION_DENIED:
		return("permission denied");
	    case CLIENT_NOT_GRABBING:
		return("client not grabbing");
	    case CLIENT_NOT_RELEASED:
		return("client not released");
	    case INVALID_URL:
		return("invalid url");
	    case NO_REGISTRY:
		return("no registry");
	    case ALREADY_BOUND:
		return("already bound");
	    case NOT_BOUND:
		return("not bound");
	    case NAME_IN_USE:
		return("name in use");
        }
        return("unknown");
    }
}
