
/*
 * @(#)URLString.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

import java.util.StringTokenizer;

/**
 * Shared Data URL parsed String class.
 *
 * @version     0.89 29 January 1997
 * @author      Rich Burridge
 */

public
class URLString extends SharedDataObject {

    /** The URL protocol (coll:). */
    private String protocol;

    /** The URL host name. */
    private String host;

    /** The URL port number. */
    private int port;

    /** The connection type for this Session/Client. */
    private String connectType;

    /** The URL object type (Session or Client). */
    private String objectType;

    /** The URL object name. */
    private String objectName ;

    /** The status of the URLString. True if successfully broken down. */
    private boolean status = false;


/**
 * <A NAME="SD_URLSTRING"></A>
 * <EM>URLString</EM> is a constructor for the URLString class. A URL String
 * is divided up into its component parts.
 *
 * @param <EM>url</EM> the URL string to be broken down.
 */

    public
    URLString(String url) {
        StringTokenizer tok = new StringTokenizer(url, "/:");

        if (URLString_Debug) {
            System.out.println("URLString: constructor: " + url); 
        }

        try {
            protocol    = tok.nextToken();                   /* "coll". */
            host        = tok.nextToken();                   /* Server host. */
            port        = Integer.parseInt(tok.nextToken()); /* Port. */
            connectType = tok.nextToken();               /* Connection type. */
	    objectType  = tok.nextToken();                   /* Object type. */
            objectName  = tok.nextToken();                   /* Object name. */
	    status      = true;

            if (URLString_Debug) {
                System.out.println("URLString: constructor: " +
                                   " protocol: "        + protocol +
                                   " host: "            + host +
                                   " port number: "     + port +
                                   " connection type: " + connectType +
				   " object type: "     + objectType +
                                   " object name: "     + objectName);
            }
        } catch (Exception e) {
            status = false;
        }
    }


/**
 * <A NAME="SD_GETHOST"></A>
 * <EM>getHOST</EM> get the host name for the server for this object.
 *
 * @return the host name of the server for this object.
 */

    public String
    getHost() {
        if (URLString_Debug) {
            System.out.println("URLString: getHost: " + host);
        }

        return(host);
    }


/**
 * <A NAME="SD_GETPORT"></A>
 * <EM>getPort</EM> get the port number being used by the server for this
 * object.
 *
 * @return the port number being used by the server for this object.
 */

    public int
    getPort() {
        if (URLString_Debug) {
            System.out.println("URLString: getPort: " + port);
        }

        return(port);
    }


/**
 * <A NAME="SD_GETCONNECTIONTYPE"></A>
 * <EM>getConnectionType</EM> get the connection type of this Session/Client.
 *
 * @return the connection type of this Session/Client.
 */
 
    public String
    getConnectionType() {
        if (URLString_Debug) {
            System.out.println("URLString: getConnectionType: " + connectType);
        }
 
        return(connectType);
    }


/**
 * <A NAME="SD_GETOBJECTTYPE"></A>
 * <EM>getObjectType</EM> get the type of this object (Session or Client).
 *
 * @return the type of this object (Session or client).
 */

    public String
    getObjectType() {
        if (URLString_Debug) {
            System.out.println("URLString: getObjectType: " + objectType);
        }

        return(objectType);
    }


/**
 * <A NAME="SD_GETOBJECTNAME"></A>
 * <EM>getObjectName</EM> get the name of this object.
 *
 * @return the name of this object.
 */

    public String
    getObjectName() {
        if (URLString_Debug) {
            System.out.println("URLString: getObjectName: " + objectName);
        }

        return(objectName);
    }


/**
 * <A NAME="SD_GETSTATUS"></A>
 * <EM>getStatus</EM> get the status of this URLString.
 *
 * @return a status value indicating whether this URLString was successfully
 * unpacked.
 */

    public boolean
    getStatus() {
        if (URLString_Debug) {
            System.out.println("URLString: getStatus: " + status);
        }

        return(status);
    }
}
