
/*
 * @(#)SessionFactory.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData;

import java.net.URL;
import java.util.StringTokenizer;

/**
 * Shared Data SessionFactory class.
 *
 * This is a factory class that is the basis for creating new sessions.
 * These would be of the appropriate type (T.120, RMI, TCP/IP ...).
 * The session would then be published by the naming service, and tied
 * to a specific URL.
 *
 * @version     0.90 12 February 1997
 * @author      Rich Burridge
 */

public
class SessionFactory extends SharedDataObject {

/**
 * <A NAME="SD_CREATESESSION"></A>
 * <EM>createSession</EM> is a class method that will create a new session.
 * If the Session already exists, a handle is returned to that Session.
 * This method will throw a SharedDataException if it can't find a session
 * of the appropriate type.
 *
 * @param <EM>url</EM> the url for this session.
 *
 * @exception SharedDataException If a Shared Data exception has occured. 
 * <PRE>
 * The following exception types can occur:
 *      INVALID_URL     - url string given is invalid.
 *      NO_SUCH_SESSION - a session of this type could not be returned.
 * </PRE>
 *
 * @return an appropriate Session.
 */
 
    public static Session
    createSession(String url) throws SharedDataException {
	Session session = null;

	if (SessionFactory_Debug) {
            System.out.println("SessionFactory: createSession:" +
				" manager: none"); 
        }

	try {
	    session = (Session) Naming.lookup(url);
	} catch (SharedDataException sde) {
	    if (sde.getType() == SharedDataException.NOT_BOUND) {
		URLString urlString   = new URLString(url);
	        String    sessionName = urlString.getObjectName();
	        String    connectType = urlString.getConnectionType();
                String    className   = "SharedData." + connectType +
                                           "." + connectType + "Session";

                try {
		    session = (Session) Class.forName(className).newInstance();
	        } catch (Exception e) {
		    throw new SharedDataException(
				SharedDataException.NO_SUCH_SESSION);
	        }

	        session.setName(sessionName);
                Naming.bind(url, session);

/*  The second call to Naming.lookup() is needed in order to return a
 *  client-side handle to the Session.
 */
		session = (Session) Naming.lookup(url);
	    }
	}
	return(session);
    }


/**
 * <A NAME="SD_CREATESESSIONWITHMANAGER"></A>
 * <EM>createSession</EM> is a class method that will create a new session,
 * and associate a session manager with that session.
 * If the Session already exists, a handle is returned to that Session.
 * This method will throw a SharedDataException if it can't find a session
 * of the appropriate type.
 *
 * @param <EM>urlEM> the url for this session.
 * @param <EM>sessionManager</EM> the session manager to associate with this
 * session.
 *
 * @exception SharedDataException If a Shared Data exception has occured. 
 * <PRE>
 * The following exception types can occur:
 *      INVALID_URL    - url string given is invalid.
 *      NO_SUCH_SESSION - a session of this type could not be returned.
 *      MANAGER_EXISTS - a manager already exists for this session.
 * </PRE>
 *
 * @return an appropriate Session.
 */
 
    public static Session
    createSession(String url, SessionManager sessionManager)
                                                throws SharedDataException {
        Session session = null;

        if (SessionFactory_Debug) {
            System.out.println("SessionFactory: createSession:" +
				" manager: " + sessionManager);
        }

        try {
	    session = (Session) Naming.lookup(url);
        } catch (SharedDataException sde) {
            if (sde.getType() == SharedDataException.NOT_BOUND) {
                URLString urlString   = new URLString(url);
                String    sessionName = urlString.getObjectName(); 
                String    connectType = urlString.getConnectionType(); 
                String    className   = "SharedData." + connectType +        
                                           "." + connectType + "Session";        
                try {
                    session = (Session) Class.forName(className).newInstance();
                } catch (Exception e) {
                    throw new SharedDataException(
				SharedDataException.NO_SUCH_SESSION);
                }

                session.setName(sessionName);
                Naming.bind(url, session);

/*  The second call to Naming.lookup() is needed in order to return a
 *  client-side handle to the Session.
 */
                session = (Session) Naming.lookup(url);
	    }
        }
        session.po.attachSessionManager(sessionManager, session);
        return(session);
    }
}
