
/*
 * @(#)ByteArray.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import java.util.Hashtable;

/**
 * Byte Array class.
 *
 * @version     0.94 4 April 1997
 * @author      Rich Burridge
 */

class
ByteArray implements socketDebugFlags {

    /** The name of this shared byte array object. */
    protected String name;

    /** The shared byte array. */
    protected byte[] value;

    /** Indicates whether this byte array has been created yet. */
    private boolean created = false;

    /** The length of the shared byte array. */
    protected int length;

    /** The observers associated with this shared byte array. */
    protected Hashtable observers = null;


/**
 * <A NAME="SD_BYTEARRAY"></A>
 * <EM>ByteArray</EM> is a constructor for the Byte Array class.
 * An array of bytes is turned into a ByteArray object. The length is
 * set to the full length of the byte array.
 *
 * The Byte Array class also contains the list of observers interested
 * in changes to this shared byte array.
 *
 * @param <EM>name</EM> the name of the shared byte array being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>created</EM> indicates whether this shared byte array should 
 * be placed in a created state. 
 * @param <EM>value</EM> the value of the byte array.
 */

    ByteArray(String name, boolean created, byte[] value) {
	if (ByteArray_Debug) {
            System.out.println("ByteArray: constructor:" +
				" name: "    + name +   
				" created: " + created +
				" value: "   + value);
        }

        this.name    = name;
	this.created = created;
        this.value   = value;
        this.length  = value.length;
    }


/**
 * <A NAME="SD_BYTEARRAY"></A>
 * <EM>ByteArray</EM> is a constructor for the Byte Array class.
 * An array of bytes is turned into a ByteArray object.
 * The length of the data object is provided. If this value is invalid, then
 * the length of the data object is the length of the array of bytes.
 * 
 * The Byte Array class also contains the list of observers interested
 * in changes to this shared byte array.
 *
 * @param <EM>name</EM> the name of the shared byte array being constructed.
 * This name cannot include '\u0001' or newline characters. 
 * @param <EM>created</EM> indicates whether this shared byte array should 
 * be placed in a created state. 
 * @param <EM>value</EM> the byte array.
 * @param <EM>length</EM> the length of the byte array.
 */
 
    ByteArray(String name, boolean created, byte[] value, int length) {
	if (ByteArray_Debug) {
            System.out.println("ByteArray: constructor:" +
				" name: "    + name +   
				" created: " + created +
				" value: "   + value +
				" length: "  + length);  
        }

        this.name    = name;
        this.created = created;
	this.value   = value;
        this.length  = length;

        if (length > 0 || length <= value.length) {
            this.length = length;
        } else {
            this.length = value.length;
        }
    }


/**
 * <A NAME="SD_GETOBSERVERS"></A>
 * <EM>getObservers</EM> get the observers hashtable for this shared byte array.
 *
 * @return the hashtable of observers.
 */

    public Hashtable
    getObservers() {
        if (ByteArray_Debug) {
            System.out.println("ByteArray: getObservers.");
        }

	return(observers);
    }


/**
 * <A NAME="SD_SETVALUE"></A>
 * <EM>setValue</EM> set a new value for this shared byte array.
 *
 * @param <EM>value</EM> the new value for this shared byte array
 */
 
    public void
    setValue(byte[] value) {
	if (ByteArray_Debug) {
            System.out.println("ByteArray: setValue: " + value);
        }
 
        if (this.value.length == value.length) {
	    System.arraycopy(value, 0, this.value, 0, length); 
	} else {
	    System.out.println("ByteArray: setValue: " +
			       " shared byte array: " + name +
			       " has changed length.\n" +
			       " old length: " + this.value.length +
			       " new length: " + value.length);
	}
    }


/**
 * <A NAME="SD_GETVALUE"></A>
 * <EM>getValue</EM> get the byte array associated with this SharedByteArray
 * object.
 *
 * @return the byte array contained in this SharedByteArray object.
 */
 
    public byte[]
    getValue() {
        if (ByteArray_Debug) {
            System.out.println("ByteArray: getValue: " + value);
        }
 
        return(value);
    }


/**
 * <A NAME="SD_GETLENGTH"></A>
 * <EM>getLength</EM> get the length of the shared byte array associated
 * with this SharedByteArray object.
 *
 * @return the length (in bytes) of the byte array.
 */
 
    public int
    getLength() {
        if (ByteArray_Debug) {
            System.out.println("ByteArray: getLength: " + length);
        }
 
        return(length);
    }
}
