
/*
 * @(#)ClientImpl.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;

/**
 * Shared Data Client class.
 *
 * @version     0.89 27 January 1997
 * @author      Rich Burridge
 */

class
ClientImpl extends socketSharedDataObject implements Client {

    /** The name of this client. */
    private String name;


/**
 * <A NAME="SD_CLIENTIMPL"></A>
 * <EM>ClientImpl</EM> the constructor for the ClientImpl class.
 *
 * @param <EM>name</EM> the name of the client being constructed.
 * This name cannot include '\u0001' or newline characters. 
 */

    public
    ClientImpl(String name) {
        if (ClientImpl_Debug) {
            System.out.println("ClientImpl: constructor: " + name);
        }

        this.name = name;
    }    


/**
 * <A NAME="SD_AUTHENTICATE"></A>
 * <EM>authenticate</EM> used to authenticate a client for potentially
 * joining a session. The Session, Channel or Token Manager will be doing
 * this client validation.
 *
 * The manager sends the client a challenge. The client responds.
 * This response is validated by the manager and determines if the
 * client will be allowed to join the session, channel or token.   
 *
 * @param <EM>challenge</EM> the challenge given by the manager to
 * the client.
 *
 * @return the response by the client to the managers challenge.
 */

    public String
    authenticate(String challenge) {
        if (ClientImpl_Debug) {
            System.out.println("ClientImpl: authenticate:" +
                       " this class should not be called directly."  +
                       " You should implement the Client interface.");
        }
	return((String) null);
    }


/**
 * <A NAME="SD_GETNAME"></A>
 * <EM>getName</EM> get the name of this client.
 *
 * @return the name of the client.
 * The client name cannot include '\u0001' or newline characters. 
 */

    public String
    getName() {
        if (ClientImpl_Debug) {
            System.out.println("ClientImpl: getName: " + name);
        }
	return(name);
    }


/**
 * <A NAME="SD_CLIENTCHANGED"></A>
 * <EM>clientChanged</EM> is called when a client change has been observed.
 *
 * It should never be called for this implementation.
 *
 * @param <EM>event</EM> the client changed event containing more information
 * on exactly what has changed.
 *
 * </pre>
 */

    public void
    clientChanged(ClientEvent event) {
        if (ClientImpl_Debug) {
            System.out.println("ClientImpl: clientChanged:" +          
                       " this class should not be called directly."  +
                       " You should implement the Client interface.");   
        }
    }
}
