
/*
 * @(#)ManagerProxyMessage.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.StringTokenizer;

/**
 * Shared Data manager proxy message class.
 *
 * @version     0.94 6 April 1997
 * @author      Rich Burridge
 */

class
ManagerProxyMessage extends SharedDataMessage {

    /** The client-side managable object for this message. */
    private Manageable manageable;

    /** The client-side manager associated with this message. */
    private SharedDataManager manager;


/**
 * <A NAME="SD_MANAGERPROXYMESSAGE"></A>
 * <EM>ManagerProxyMessage</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>manager</EM>
 * @param <EM>manageable</EM>
 * @param <EM>tok</EM>
 */

    public
    ManagerProxyMessage(Session session, SharedDataManager manager,
			Manageable manageable) {
        if (ManagerProxyMessage_Debug) {
            System.out.println("ManagerProxyMessage: constructor:" +
                               " session: "    + session +
                               " manager: "    + manager +
                               " manageable: " + manageable);
        }
 
        this.session    = session;
        this.manager    = manager;
        this.manageable = manageable;
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>tok</EM>
 */

    protected void
    handleMessage(StringTokenizer tok) {
        int    id   = Integer.parseInt(tok.nextToken());
        String type = tok.nextToken();

	if (ManagerProxyMessage_Debug) {
            System.out.println("ManagerProxyMessage: handleMessage:" +
                               " tok: " + tok);
        }

        if (type.equals(T_Manager)) {
            parseManagerMessage(id, tok);
        } else {
            System.out.println("ManagerProxyMessage: handleMessage:" +
                               " unknown type: " + type);
        }
    }


/**
 * <A NAME="SD_PARSEMANAGERMESSAGE"></A>
 * <EM>parseManagerMessage</EM>
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>tok</EM>
 */

    void
    parseManagerMessage(int id, StringTokenizer tok) {
        String objectType = tok.nextToken();
        String action     = tok.nextToken();
        String objectName = tok.nextToken();

        if (ManagerProxyMessage_Debug) {
            System.out.println("ManagerProxyMessage: parseManagerMessage:" +
				" id: "  + id +
				" tok: " + tok);
        }

        if (action.equals(T_Authenticate)) {           /* AUTHENTICATE. */
            String clientName = tok.nextToken();
	    AuthenticateClient client = new AuthenticateClient(id, session,
					 clientName, objectType, objectName,
				         manager, manageable);
	    Thread thread = new Thread(client);

	    thread.start();
	}
    }
}
