
/*
 * @(#)ObserverMessage.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.StringTokenizer;

/**
 * Shared Data observer message thread class.
 *
 * @version     0.89 27 January 1997
 * @author      Rich Burridge
 */

class
ObserverMessage extends SharedDataMessage implements Runnable {

    /** The observer to be notified. */
    private SharedDataObserver observer;

    /** The client name to use in the observer event. */
    private String clientName;

    /** The type of manageable object to raise the event on. */
    private Manageable manageable;

    /** The observer event type. */
    private int type;


/**
 * <A NAME="SD_OBSERVERMESSAGE"></A>
 * <EM>ObserverMessage</EM>
 *
 * @param <EM>observer</EM>
 * @param <EM>session</EM>
 * @param <EM>clientName</EM>
 * @param <EM>manageable</EM>
 * @param <EM>type</EM>
 */

    public
    ObserverMessage(SharedDataObserver observer, Session session,
		    String clientName, Manageable manageable, int type) {
        if (ObserverMessage_Debug) {
            System.out.println("ObserverMessage: constructor:" +
                               " observer name: "   + observer.getName() +
			       " session name: "    + session.getName() +
                               " client name: "     + clientName +
			       " manageable name: " + manageable.getName() +
			       " type: "            + type);
        }
 
        this.observer   = observer;
	this.session    = session;
	this.clientName = clientName;
	this.manageable = manageable;
	this.type       = type;
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>tok</EM>
 */

    protected void
    handleMessage(StringTokenizer tok) {
        if (ObserverMessage_Debug) {
            System.out.println("ObserverMessage: handleMessage.");
        }

        if (manageable instanceof Channel) {
            ((ChannelObserver) observer).channelChanged(new ChannelEvent(
                session, clientName, (Channel) manageable, type));
        } else if (manageable instanceof Session) {
            ((SessionObserver) observer).sessionChanged(new SessionEvent(
                session, clientName, type));
        } else if (manageable instanceof Token) {
            ((TokenObserver) observer).tokenChanged(new TokenEvent(
                session, clientName, (Token) manageable, type));
        }
    }


/**
 * <A NAME="SD_RUN"></A>
 * <EM>run</EM>
 */

    public void
    run() {
        if (ObserverMessage_Debug) {
            System.out.println("ObserverMessage: run.");
        }

        handleMessage(null);
    }
}
