
/*
 * @(#)ReceiveClient.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.Vector;

/**
 * Shared Data proxy receive client class.
 *
 * @version     0.89 27 January 1997
 * @author      Rich Burridge
 */

class
ReceiveClient extends socketSharedDataObject
	      implements Client, ChannelConsumer {

    /** The client-side channel to consume data on. */
    private Channel channel;

    /** Data message queue (from server). */
    private Vector messages = null;

    /** The name of this client. */
    private String name;


/**
 * <A NAME="SD_RECEIVECLIENT"></A>
 * <EM>ReceiveClient</EM> the constructor for the ReceiveClient class.
 *
 * @param <EM>channel</EM>
 * @param <EM>name</EM> the name of the client being constructed.
 * This name cannot include '\u0001' or newline characters. 
 */

    public
    ReceiveClient(Channel channel, String name) {
        if (ReceiveClient_Debug) {
            System.out.println("ReceiveClient: constructor: " +
			       " channel: " + channel +
			       " name: "    + name);
        }

        this.channel = channel;
	this.name    = name;
	messages     = new Vector();
    }    


/**
 * <A NAME="SD_AUTHENTICATE"></A>
 * <EM>authenticate</EM> used to authenticate a client.
 *
 * @param <EM>challenge</EM> the challenge given by the manager to the client.
 *
 * @return the response by the remote client to the managers challenge.
 */

    public String
    authenticate(String challenge) {
        if (ReceiveClient_Debug) {
            System.out.println("ReceiveClient: authenticate:" +
                               " - this should never be called.");
        }
	return((String) null);
    }


/**
 * <A NAME="SD_GETNAME"></A>
 * <EM>getName</EM> get the name of this client.
 *
 * @return the name of the client.
 * The client name cannot include '\u0001' or newline characters. 
 */

    public String
    getName() {
        if (ReceiveClient_Debug) {
            System.out.println("ReceiveClient: getName: " + name);
        }
	return(name);
    }


/**
 * <A NAME="SD_CLIENTCHANGED"></A>
 * <EM>clientChanged</EM> is called when a client change has been observed.
 *
 * It should never be called for the proxy autheentication client.
 *
 * @param <EM>event</EM> the client changed event containing more information
 * on exactly what has changed.
 */

    public void
    clientChanged(ClientEvent event) {
        if (ReceiveClient_Debug) {
            System.out.println("ReceiveClient: clientChanged:" +
			       " - this should never be called.");
        }
    }


/**
 * <A NAME="SD_DATARECEIVED"></A>
 * <EM>dataReceived</EM> is called when data is received for this client on
 * the given channel. This interface is used for normal data and for
 * uniformly sequenced data.
 *
 * The data object received is to a copy of the client data which this
 * consumer can do with as they require.
 *
 * @param <EM>data</EM> the data which can be of unlimited size.
 */


    public void
    dataReceived(Data data) {
        if (ReceiveClient_Debug) {
            System.out.println("ReceiveClient: dataReceived " +
                               " data: " + data);
        }

	try {
	    putMessage(data);
	} catch (Exception e) {
            System.out.println("ReceiveClient: dataReceived: " +
                               "exception " + e);
            if (ReceiveClient_Debug) {
		e.printStackTrace();
	    }
	}
    }


/**
 * <A NAME="SD_DATAVAILABLE"></A>
 * <EM>dataAvailable</EM>
 *
 * @return
 */

    boolean
    dataAvailable() {
        if (ReceiveClient_Debug) {
            System.out.println("ReceiveClient: dataAvailable.");
        }

	synchronized (messages) {
	    return(messages.size() != 0);
	}
    }


/**
 * <A NAME="SD_GETMESSAGE"></A>
 * <EM>getMessage</EM>
 *
 * @exception InterruptedException
 *
 * @return
 */

    synchronized Data
    getMessage() throws InterruptedException {
        Data data = null;

        if (ReceiveClient_Debug) {
            System.out.println("ReceiveClient: getMessage.");
        }

        notifyAll();
	while (messages.size() == 0) {
            wait();
        }
        data = (Data) messages.firstElement();
        messages.removeElement(data);
	return(data);
    }


/**     
 * <A NAME="SD_PUTMESSAGE"></A>
 * <EM>putMessage</EM>
 *
 * @param <EM>data</EM>
 *
 * @exception InterruptedException
 */
 
    synchronized void
    putMessage(Data data) throws InterruptedException {
        if (ReceiveClient_Debug) {
            System.out.println("ReceiveClient: putMessage:" +
			       " data: " + data);
        }

	messages.addElement(data);
	notifyAll();
    }
}
