
/*
 * @(#)ServerByteArray.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.primitive.SharedByteArrayObserver;
import java.util.Hashtable;

/**
 * Server Byte Array class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

class
ServerByteArray extends ByteArray {

/**
 * <A NAME="SD_SERVERBYTEARRAY"></A>
 * <EM>ServerByteArray</EM> is a constructor for the Server byte array class.
 * An array of bytes is turned into a ServerByteArray object. The length is
 * set to the full length of the byte array.
 *
 * The Server byte array class also contains the list of observers interested
 * in changes to this shared byte array.
 *
 * @param <EM>name</EM> the name of the shared byte array being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>created</EM> indicates whether this shared byte array should
 * be placed in a created state.
 * @param <EM>value</EM> the value of the byte array.
 */

    ServerByteArray(String name, boolean created, byte[] value) {
	super(name, created, value);

	if (ServerByteArray_Debug) {
            System.out.println("ServerByteArray: constructor:" +
				" name: "    + name +   
				" created: " + created +
				" value: "   + value);
        }

        observers = new Hashtable();
    }


/**
 * <A NAME="SD_SERVERBYTEARRAY"></A>
 * <EM>ServerByteArray</EM> is a constructor for the Server byte array class.
 *  An array of bytes is turned into a ServerByteArray object.
 * The length of the data object is provided. If this value is invalid, then
 * the length of the data object is the length of the array of bytes.
 * 
 * The Server byte array class also contains the list of observers interested
 * in changes to this shared byte array.
 *
 * @param <EM>name</EM> the name of the shared byte array being constructed.
 * This name cannot include '\u0001' or newline characters. 
 * @param <EM>value</EM> the byte array.
 * @param <EM>length</EM> the length of the byte array.
 */
 
    ServerByteArray(String name, boolean created, byte[] value, int length) {
	super(name, created, value, length);

	if (ServerByteArray_Debug) {
            System.out.println("ServerByteArray: constructor:" +
				" name: "    + name +   
				" created: " + created +
				" value: "   + value +
				" length: "  + length);  
        }

        observers = new Hashtable();
    }


/**
 * <A NAME="SD_ADDOBSERVER"></A>
 * <EM>addObserver</EM> adds an observer to the list that will be informed
 * when the value of this shared byte array changes.
 *
 * @param <EM>thread</EM> the thread connection back to the proxy.
 * @param <EM>observer</EM> the observer being added.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */
 
    public void
    addObserver(SocketThread thread, SharedByteArrayObserver observer) {
        if (ServerByteArray_Debug) {
            System.out.println("ServerByteArray: addObserver: " +
			       " thread: "   + thread +
			       " observer: " + observer);
        }
 
        synchronized (observers) {
            observers.put(observer, thread);
        }
    }


/**
 * <A NAME="SD_DELETEOBSERVER"></A>
 * <EM>deleteObserver</EM> deletes an observer from the list that will be 
 * informed when the value of this shared byte array changes.
 *
 * @param <EM>observer</EM> the observer being deleted.
 *
 * @exception SharedDataException If a Shared Data exception has occured.
 */
 
    public void
    deleteObserver(SharedByteArrayObserver observer) {
        if (ServerByteArray_Debug) {
            System.out.println("ServerByteArray: deleteObserver:" + 
                                " observer: " + observer);
        }
 
        synchronized (observers) {
            observers.remove(observer);
        } 
    }
}
