
/*
 * @(#)SessionProxyMessage.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.StringTokenizer;

/**
 * Shared Data session proxy message class.
 *
 * @version     0.94 6 April 1997
 * @author      Rich Burridge
 */

class
SessionProxyMessage extends SharedDataMessage {

    /** The data (if any) associated with this message. */
    private byte data[];

    /** Handle to the SessionProxy object for this message. */
    private SessionProxy sp;

    /** Session proxy thread associated with this message. */
    private SocketThread thread;


/**
 * <A NAME="SD_SESSIONPROXYMESSAGE"></A>
 * <EM>SessionProxyMessage</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>thread</EM>
 * @param <EM>message</EM>
 * @param <EM>data</EM>
 */

    public
    SessionProxyMessage(Session session, SocketThread thread, byte data[]) {
        if (SessionProxyMessage_Debug) {
            System.out.println("SessionProxyMessage: constructor:" +
                               " session: " + session +
                               " thread: "  + thread +
			       " data: "    + data);
        }
 
        this.session = session;
	this.thread  = thread;
	this.data    = data;

	sessionName  = session.getName();
        sp = (SessionProxy) session.po.getProxy();
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>tok</EM>
 */

    protected void
    handleMessage(StringTokenizer tok) {
        int    id   = Integer.parseInt(tok.nextToken());
	String type = tok.nextToken();

        if (SessionProxyMessage_Debug) {
            System.out.println("SessionProxyMessage: handleMessage:" +
                               " tok: " + tok);
        }
 
               if (type.equals(Session.M_Session)) {
            parseServerSessionMessage(id, tok, thread);
        } else if (type.equals(Channel.M_Channel)) {
            parseServerChannelMessage(id, tok, thread);
        } else if (type.equals(Token.M_Token)) {
            parseServerTokenMessage(id, tok, thread);
        } else if (type.equals(Client.M_Client)) {
	    parseServerClientMessage(id, tok, thread);
        } else {
            System.out.println("SessionProxyMessage: handleMessage:" +
                               " unknown type: " + type);
        }
    }


/**
 * <A NAME="SD_PARSESERVERCLIENTMESSAGE"></A>
 * <EM>parseServerClientMessage</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>tok</EM>
 * @param <EM>thread</EM>
 */

    private void
    parseServerClientMessage(int id, StringTokenizer tok, SocketThread thread) {
	String action = tok.nextToken();

        if (SessionProxyMessage_Debug) {
            System.out.println("SessionProxyMessage:" +
				" parseServerClientMessage:" +
				" id: "     + id +
				" action: " + action);
        }

        if (action.equals(T_Challenge)) {              /* CHALLENGE. */
            String objectType = tok.nextToken();
            String objectName = tok.nextToken();
            String clientName = tok.nextToken();
            String challenge  = tok.nextToken();
	    Client joiningClient = sp.getJoiningClient(objectType);

	    if (clientName.equals(joiningClient.getName())) {
                clientChallenge(id, joiningClient, challenge,
				objectType, objectName);
            } else {
                System.out.println("SessionProxyMessage:" +
		    " parseServerClientMessage:" +
                    " unexpected authentication request for: " + clientName);
            }

        } else {
            String objectName = tok.nextToken();
            String clientName = tok.nextToken();
            Client client     = sp.getClientByName(clientName);
            int    eventType  = 0;

                   if (action.equals(T_Session_Invite)) { /* SESSION INVITE. */
                eventType = ClientEvent.SESSION_INVITE;
            } else if (action.equals(T_Session_Expel)) {  /* SESSION EXPEL. */
                eventType = ClientEvent.SESSION_EXPEL;
            } else if (action.equals(T_Channel_Invite)) { /* CHANNEL INVITE. */
                eventType = ClientEvent.CHANNEL_INVITE;
            } else if (action.equals(T_Channel_Expel)) {  /* CHANNEL EXPEL. */
                eventType = ClientEvent.CHANNEL_EXPEL;
            } else if (action.equals(T_Token_Invite)) {   /* TOKEN INVITE. */
                eventType = ClientEvent.TOKEN_INVITE;
            } else if (action.equals(T_Token_Expel)) {    /* TOKEN EXPEL. */
                eventType = ClientEvent.TOKEN_EXPEL;
            }
 
            if (client != null) {
                ClientMessage cm = new ClientMessage(client, objectName,
                                                     eventType);
		Thread clientThread = new Thread(cm);

		clientThread.start();
            } else {
                System.out.println("SessionProxyMessage:" +
			 	   " parseServerClientMessage:" +
                                   " unknown client: " + clientName);
            }
        }
    }


/**
 * <A NAME="SD_PARSESERVERCHANNELMESSAGE"></A>
 * <EM>parseServerChannelMessage</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>tok</EM>
 * @param <EM>thread</EM>
 */

    private void
    parseServerChannelMessage(int id, StringTokenizer tok, 
			      SocketThread thread) {
        String  action      = tok.nextToken();
        String  channelName = tok.nextToken();
        Channel channel     = sp.getChannelByName(channelName);

        if (SessionProxyMessage_Debug) {
            System.out.println("SessionProxyMessage:" +
				" parseServerChannelMessage: " +
				" id: "     + id +
                                " thread: " + thread +
                                " action: " + action);
        }

        if (channel == null) {
            System.out.println("SessionProxyMessage:" +
                               " parseServerChannelMessage: " +
			       channelName + " doesn't exist.");

        } else {
            if (action.equals(T_InformObserver)) {       /* INFORMOBSERVER. */
		String clientName = tok.nextToken();
                int    type       = Integer.parseInt(tok.nextToken());

                informObservers(clientName, channel, type);

            } else if (action.equals(T_DataReceived)) {  /* DATARECEIVED. */
                ChannelProxy cp     = (ChannelProxy) channel.po.getProxy();
		String senderName   = tok.nextToken();
                String receiverName = tok.nextToken();
                int    priority     = Integer.parseInt(tok.nextToken());
                int    length       = Integer.parseInt(tok.nextToken());

                cp.dataReceived(session, channelName, senderName,
                                receiverName, priority, length, data);
            } else {
                if (SessionProxyMessage_Debug) {
                    System.out.println("SessionProxyMessage:" +
				       " parseServerChannelMessage:" +
                            " action: " + action +
                            " - client should have waited for this reply.");
                }
	    }
        }
    }


/**
 * <A NAME="SD_PARSESERVERSESSIONMESSAGE"></A>
 * <EM>parseServerSessionMessage</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>tok</EM>
 * @param <EM>thread</EM>
 */

    private void
    parseServerSessionMessage(int id, StringTokenizer tok, 
			      SocketThread thread) {
        String action      = tok.nextToken();
        String sessionName = tok.nextToken();

        if (SessionProxyMessage_Debug) {
            System.out.println("SessionProxyMessage:" +
				" parseServerSessionMessage:" +
				" id: "     + id +
				" action: " + action);
        }

        if (action.equals(T_InformObserver)) {       /* INFORMOBSERVER. */
            String clientName = tok.nextToken();
            int    type       = Integer.parseInt(tok.nextToken());

            informObservers(clientName, session, type);

        } else if (action.equals(T_ValueChanged)) {  /* VALUECHANGED. */
	    String observerName  = tok.nextToken();
            String clientName    = tok.nextToken();
	    String byteArrayName = tok.nextToken();
	    int    type          = Integer.parseInt(tok.nextToken());
	    int    length        = Integer.parseInt(tok.nextToken());

            sp.valueChanged(observerName, clientName,
			    byteArrayName, type, length, data);
        } else {
            if (SessionProxyMessage_Debug) {
                System.out.println("SessionProxyMessage:" +
                        " parseServerSessionMessage: " +
                        " action: " + action +
                        " - client should have waited for this reply.");
            }
        }
    }


/**
 * <A NAME="SD_PARSESERVERTOKENMESSAGE"></A>
 * <EM>parseServerTokenMessage</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>tok</EM>
 * @param <EM>thread</EM>
 */

    private void
    parseServerTokenMessage(int id, StringTokenizer tok, SocketThread thread) {
	String action    = tok.nextToken();
        String tokenName = tok.nextToken();
        Token  token     = sp.getTokenByName(tokenName);

        if (SessionProxyMessage_Debug) {
            System.out.println("SessionProxyMessage:" +
				" parseServerTokenMessage:" +
				" id: "        + id +
				" action: "    + action +
				" tokenName: " + tokenName);
        }

        if (token == null) {
            System.out.println("SessionProxyMessage:" +
                               " parseServerTokenMessage:" +
			       tokenName + " doesn't exist.");

        } else {
            if (action.equals(T_InformObserver)) {  /* INFORMOBSERVER. */
                String clientName = tok.nextToken();
                int    type       = Integer.parseInt(tok.nextToken());

                informObservers(clientName, token, type);

            } else {
                if (SessionProxyMessage_Debug) {
                    System.out.println("SessionProxyMessage:" +
			    " parseServerMessage: " +
                            " action: " + action +
                            " - client should have waited for this reply.");
                }
            }
        }
    }
}
