
/*
 * @(#)SessionServerMessage.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.StringTokenizer;

/**
 * Shared Data session server message thread class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */

class
SessionServerMessage extends SharedDataMessage {

    /** The data (if any) associated with this message. */
    private byte data[];

    /** Handle to the SessionServer object for this message. */
    private SessionServer ss;

    /** Session server thread associated with this message. */
    private SocketThread thread;


/**
 * <A NAME="SD_SESSIONSERVERMESSAGE"></A>
 * <EM>SessionServerMessage</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>thread</EM>
 * @param <EM>data</EM>
 */

    public
    SessionServerMessage(Session session, SocketThread thread, byte data[]) {
        if (SessionServerMessage_Debug) {
            System.out.println("SessionServerMessage: constructor:" +
                               " session: " + session +
                               " thread: "  + thread +
			       " data: "    + data);
        }
 
        this.session = session;
	this.thread  = thread;
	this.data    = data;

	ss = (SessionServer) session.so.getServer();
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>tok</EM>
 */

    protected void
    handleMessage(StringTokenizer tok) {
        int    id   = Integer.parseInt(tok.nextToken());
        String type = tok.nextToken();

        if (SessionServerMessage_Debug) {
            System.out.println("SessionServerMessage: handleMessage:" +
				" tok: " + tok);
        }
 
               if (type.equals(Session.M_Session)) {
            parseProxySessionMessage(id, tok, thread);
        } else if (type.equals(Channel.M_Channel)) {
            parseProxyChannelMessage(id, tok, thread);
        } else if (type.equals(Token.M_Token)) {
            parseProxyTokenMessage(id, tok, thread);
        } else if (type.equals(Client.M_Client)) {
            parseProxyClientMessage(id, tok, thread);
        } else {
            System.out.println("SessionServerMessage: handleMessage:" +
                               " unknown type: " + type);
        }
    }


/**
 * <A NAME="SD_PARSEPROXYCLIENTMESSAGE"></A>
 * <EM>parseProxyClientMessage</EM>
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>tok</EM>
 * @param <EM>thread</EM>
 */

    void
    parseProxyClientMessage(int id, StringTokenizer tok, SocketThread thread) {
        String action     = tok.nextToken();
        String objectType = tok.nextToken();
        String reply      = null;

        if (SessionServerMessage_Debug) {
            System.out.println("SessionServerMessage:" +
			       " parseProxyClientMessage:" +
                               " id: "          + id +
                               " action: "      + action +
                               " object type: " + objectType);
        }

        if (action.equals(T_Authenticate)) {            /* AUTHENTICATE. */
            String objectName = tok.nextToken();
            String clientName = tok.nextToken();
            String challenge = tok.nextToken();

            ss.clientChallenge(id, session, clientName,
			       challenge, objectType, objectName);

        } else if (action.equals(T_Challenge)) {        /* CHALLENGE. */
            String objectName = tok.nextToken();
            String clientName = tok.nextToken();
            String response   = tok.nextToken();

            ss.clientResponse(id, session.getName(), clientName,
			      response, objectType, objectName);

        } else if (action.equals(T_Join)) {             /* JOIN. */
            String clientName = tok.nextToken();
            String admitted   = tok.nextToken();
            String message    = addMessageHeader(session.getName(), id,
				    objectType + T_Sep + T_Authenticate +
                                    T_Sep + "0" + T_Sep + admitted);
	    SocketThread joiningThread = null;

            if (ss.joiningThreads != null) {
		joiningThread = (SocketThread)
				ss.joiningThreads.get(clientName);
		joiningThread.writeLine(null, message);
	    }
        } else {
            System.out.println("SessionServerMessage:" +
			       " parseProxyClientMessage:" +
                               " unknown CLIENT action: " + action);
        }
    }


/**
 * <A NAME="SD_PARSEPROXYSESSIONMESSAGE"></A>
 * <EM>parseProxySessionMessage</EM>
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>tok</EM>
 * @param <EM>thread</EM>
 */

    void
    parseProxySessionMessage(int id, StringTokenizer tok, SocketThread thread) {
        String action      = tok.nextToken();
        String sessionName = tok.nextToken();
        String reply       = null;

        if (SessionServerMessage_Debug) {
            System.out.println("SessionServerMessage:" +
			       " parseProxySessionMessage:" +
                               " id: "           + id +
                               " action: "       + action +
			       " session name: " + sessionName);
        }

        if (session == null) {
            reply = addMessageHeader(sessionName, id,
                                Session.M_Session + T_Sep + action + T_Sep +
                                SharedDataException.NO_SUCH_SESSION);

        } else {
            if (action.equals(T_AddByteObserver)) {        // ADDBYTEOBSERVER
                String byteArrayName = tok.nextToken();
		String observerName  = tok.nextToken();
 
                reply = ss.addByteArrayObserver(id, thread, byteArrayName,
                                                        observerName);
 
            } else if (action.equals(T_AddObserver)) {     /* ADDOBSERVER. */
                String observerName = tok.nextToken();

                reply = ss.addObserver(id, thread, session.getName(),
					observerName, Session.M_Session);

            } else if (action.equals(T_Authenticate)) {    /* AUTHENTICATE. */
                String clientName = tok.nextToken();

                reply = ss.authenticateClient(id, thread, session.getName(),
					 clientName, Session.M_Session);

            } else if (action.equals(T_CreateByteArray)) {  // CREATEBYTEARRAY
                String clientName    = tok.nextToken();
		String observerName  = tok.nextToken();
		String byteArrayName = tok.nextToken();

                reply = ss.createByteArray(id, thread, clientName,
					 observerName, byteArrayName, data);

            } else if (action.equals(T_CreateChannel)) {   /* CREATECHANNEL. */
                String  channelName = tok.nextToken();
	        boolean reliable =
			 (new Boolean(tok.nextToken())).booleanValue();

                reply = ss.createChannel(id, channelName, reliable);

            } else if (action.equals(T_CreateToken)) {     /* CREATETOKEN. */
                String tokenName = tok.nextToken();

                reply = ss.createToken(id, tokenName);

							// DELETEBYTEOBSERVER
            } else if (action.equals(T_DeleteByteObserver)) {
                String byteArrayName = tok.nextToken();
                String observerName  = tok.nextToken();

                reply = ss.deleteByteArrayObserver(id, thread, byteArrayName,
                                                        observerName);

            } else if (action.equals(T_DeleteObserver)) {  /* DELETEOBSERVER. */
                String observerName = tok.nextToken();

                reply = ss.deleteObserver(id, session.getName(),
				          observerName, Session.M_Session);

            } else if (action.equals(T_DestroyByteArray)) { // DESTROYBYTEARRAY
                String clientName    = tok.nextToken();
                String byteArrayName = tok.nextToken();

                reply = ss.destroyByteArray(id, thread, clientName, 
					    byteArrayName);

	    } else if (action.equals(T_Expel)) {           /* EXPEL. */
                int count = Integer.parseInt(tok.nextToken());
	        String[] clientNames = new String[count];

	        for (int i = 0; i < count; i++) {
		    clientNames[i] = tok.nextToken();
	        }

                reply = ss.expel(id, session.getName(), clientNames,
			         Session.M_Session, T_Session_Expel);

            } else if (action.equals(T_GetUrl)) {          /* GETURL. */

	        reply = ss.getUrl(id);

            } else if (action.equals(T_Invite)) {          /* INVITE. */
	        System.out.println("SessionServerMessage:" +
                    " parseProxySessionMessage: session invite messages" +
		    " should be sent directly to the client.");

	    } else if (action.equals(T_Join)) {            /* JOIN. */
                String clientName = tok.nextToken();

                reply = ss.join(id, thread, clientName);

            } else if (action.equals(T_Leave)) {           /* LEAVE. */
                String clientName = tok.nextToken();

                reply = ss.leave(id, session, clientName, Session.M_Session);

							  // LISTBYTEARRAYNAMES
            } else if (action.equals(T_ListByteArrayNames)) {
                reply = ss.listNames(id, T_ListByteArrayNames);

            } else if (action.equals(T_ListChannelNames)) { // LISTCHANNELNAMES
                reply = ss.listNames(id, T_ListChannelNames);

            } else if (action.equals(T_ListClientNames)) {  // LISTCLIENTNAMES
                reply = ss.listClientNames(id, session.getName(),
						   Session.M_Session);

            } else if (action.equals(T_ListTokenNames)) {   // LISTTOKENNAMES.
                reply = ss.listNames(id, T_ListTokenNames);

            } else if (action.equals(T_UpdateByteArray)) {  // UPDATEBYTEARRAY
                String clientName    = tok.nextToken();
		String byteArrayName = tok.nextToken();

                reply = ss.updateByteArray(id, clientName,
					   byteArrayName, data);

            } else if (action.equals(T__Manager)) {         /* _MANAGER. */
                String managerName = tok.nextToken();

                reply = ss.attachManager(id, thread, session.getName(),
				         managerName, Session.M_Session);

            } else {
                System.out.println("SessionServerMessage:" +
			           " parseProxySessionMessage:" +
                                   " unknown SESSION action: " + action);
            }
	}

        if (reply != null) {
            thread.writeLine(null, reply);
        }
    }


/**
 * <A NAME="SD_PARSEPROXYCHANNELMESSAGE"></A>
 * <EM>parseProxyChannelMessage</EM>
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>tok</EM>
 * @param <EM>thread</EM>
 */

    void
    parseProxyChannelMessage(int id, StringTokenizer tok, SocketThread thread) {
        String  action      = tok.nextToken();
        String  channelName = tok.nextToken();
        String  reply       = null;
        Channel channel     = ss.getChannelByName(channelName);
	ChannelServer cs    = null;

        if (SessionServerMessage_Debug) {
            System.out.println("SessionServerMessage:" +
				" parseProxyChannelMessage: " +
                                " id: "        + id +
                                " tokenizer: " + tok +
                                " thread: "    + thread);
        }

        if (channel == null) {
            reply = addMessageHeader(session.getName(), id,
			     Channel.M_Channel + T_Sep + action + T_Sep +
			     SharedDataException.NO_SUCH_CHANNEL);

        } else {
            cs = (ChannelServer) channel.so.getServer();

	    if (action.equals(T_AddObserver)) {           /* ADDOBSERVER. */
                String observerName = tok.nextToken();
 
                reply = cs.addObserver(id, thread, session.getName(),
				       observerName, Channel.M_Channel);
 
            } else if (action.equals(T_Authenticate)) {   /* AUTHENTICATE. */
                String clientName = tok.nextToken();
 
                reply = cs.authenticateClient(id, thread, session.getName(),
					      clientName, Channel.M_Channel);
 
            } else if (action.equals(T_DeleteObserver)) { /* DELETEOBSERVER. */
                String observerName = tok.nextToken();
 
                reply = cs.deleteObserver(id, session.getName(),
					  observerName, Channel.M_Channel);
 
            } else if (action.equals(T_Expel)) {          /* EXPEL. */
                int count = Integer.parseInt(tok.nextToken());
                String[] clientNames = new String[count];

                for (int i = 0; i < count; i++) {
                    clientNames[i] = tok.nextToken();
                }

                reply = cs.expel(id, session.getName(), clientNames,
                                 Channel.M_Channel, T_Channel_Expel);

            } else if (action.equals(T_Invite)) {         /* INVITE. */
                int count = Integer.parseInt(tok.nextToken());
                String[] clientNames = new String[count];

                for (int i = 0; i < count; i++) {
                    clientNames[i] = tok.nextToken();
                }
         
                reply = cs.invite(id, session, clientNames,
				  Channel.M_Channel, T_Channel_Invite);
         
            } else if (action.equals(T_Join)) {           /* JOIN. */
                String  clientName = tok.nextToken();
                Client  client     = new ClientImpl(clientName);
                int     mode       = Integer.parseInt(tok.nextToken());
         
                reply = cs.join(id, session, thread, clientName, mode);
         
            } else if (action.equals(T_Leave)) {          /* LEAVE. */
                String clientName = tok.nextToken();
         
                reply = cs.leave(id, session, clientName, Channel.M_Channel);
         
						        /* LISTCLIENTNAMES. */
            } else if (action.equals(T_ListClientNames)) {
         
                reply = cs.listClientNames(id, session.getName(),
					   Channel.M_Channel);
         
            } else if (action.equals(T__Manager)) {       /* _MANAGER. */
                String managerName = tok.nextToken();
         
                reply = cs.attachManager(id, thread, session.getName(),
					 managerName, Channel.M_Channel);

            } else if (action.equals(T_Send)) {           /* SEND. */
                String  senderName   = tok.nextToken();
                String  receiverName = tok.nextToken();
                int     priority     = Integer.parseInt(tok.nextToken());
                int     length       = Integer.parseInt(tok.nextToken());
                boolean uniform      = tok.nextToken().equals("true")
					? true : false;

                reply = cs.send(id, session, channelName, senderName,
                                        receiverName, priority, length,
                                        uniform, data);

            } else if (action.equals(T_SetConsumer)) {    /* SETCONSUMER. */
                String clientName   = tok.nextToken();
                String consumerName = tok.nextToken();
                int    priority     = Integer.parseInt(tok.nextToken());

                reply = cs.setConsumer(id, thread, session.getName(),
				       clientName, consumerName, priority);

            } else {
                System.out.println("SessionServerMessage:" +
			           " parseProxyChannelMessage:" +
                                   " unknown CHANNEL action: " + action);
            }
        }

        if (reply != null) {
            thread.writeLine(null, reply);
        }
    }


/**
 * <A NAME="SD_PARSEPROXYTOKENMESSAGE"></A>
 * <EM>parseProxyTokenMessage</EM>
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>tok</EM>
 * @param <EM>thread</EM>
 */

    void
    parseProxyTokenMessage(int id, StringTokenizer tok, SocketThread thread) {
        String action    = tok.nextToken();
        String tokenName = tok.nextToken();
        String reply     = null;
        Token  token     = ss.getTokenByName(tokenName);
	TokenServer ts   = null;

        if (SessionServerMessage_Debug) {
            System.out.println("SessionServerMessage:" +
			        " parseProxyTokenMessage: " +
                                " id: "        + id +
                                " tokenizer: " + tok +
                                " thread: "    + thread);
        }
 
        if (token == null) {
            reply = addMessageHeader(session.getName(), id,
				     Token.M_Token + T_Sep + action + T_Sep +
				     SharedDataException.NO_SUCH_TOKEN);
 
        } else {
            ts = (TokenServer) token.so.getServer();

	    if (action.equals(T_AddObserver)) {           /* ADDOBSERVER. */
                String observerName = tok.nextToken();

                reply = ts.addObserver(id, thread, session.getName(),
					     observerName, Token.M_Token);

            } else if (action.equals(T_Authenticate)) {   /* AUTHENTICATE. */
                String clientName = tok.nextToken();

                reply = ts.authenticateClient(id, thread,
				session.getName(), clientName, Token.M_Token);

            } else if (action.equals(T_DeleteObserver)) { /* DELETEOBSERVER. */
                String observerName = tok.nextToken();

                reply = ts.deleteObserver(id, session.getName(),
						observerName, Token.M_Token);

            } else if (action.equals(T_Expel)) {          /* EXPEL. */
                int count = Integer.parseInt(tok.nextToken());
                String[] clientNames = new String[count];

                for (int i = 0; i < count; i++) {
                    clientNames[i] = tok.nextToken();
                }

                reply = ts.expel(id, session.getName(), clientNames,
                                         Token.M_Token, T_Token_Expel);

            } else if (action.equals(T_Give)) {           /* GIVE. */
                String givingClientName    = tok.nextToken();
                String receivingClientName = tok.nextToken();

                reply = ts.give(id, givingClientName, receivingClientName);

            } else if (action.equals(T_Grab)) {           /* GRAB. */
                String clientName = tok.nextToken();
                boolean exclusive = tok.nextToken().equals("true")
					? true : false;

                reply = ts.grab(id, session, clientName, exclusive);

            } else if (action.equals(T_Invite)) {         /* INVITE. */
                int count = Integer.parseInt(tok.nextToken());
                String[] clientNames = new String[count];

                for (int i = 0; i < count; i++) {
                    clientNames[i] = tok.nextToken();
                }

                reply = ts.invite(id, session, clientNames,
					Token.M_Token, T_Token_Invite);
  
            } else if (action.equals(T_Join)) {           /* JOIN. */
                String clientName = tok.nextToken();
 
                reply = ts.join(id, session, thread, clientName);
 
            } else if (action.equals(T_Leave)) {          /* LEAVE. */
                String clientName = tok.nextToken();
 
                reply = ts.leave(id, session, clientName, Token.M_Token);
 
							/* LISTCLIENTNAMES. */
            } else if (action.equals(T_ListClientNames)) {
 
                reply = ts.listClientNames(id, session.getName(), 
					   Token.M_Token);
 
            } else if (action.equals(T__Manager)) {       /* _MANAGER. */
                String managerName = tok.nextToken();
        
                reply = ts.attachManager(id, thread, session.getName(),
				               managerName, Token.M_Token);
 
            } else if (action.equals(T_Please)) {            /* PLEASE. */
                String clientName = tok.nextToken();

                reply = ts.please(id, clientName);

            } else if (action.equals(T_Release)) {           /* RELEASE. */
                String clientName = tok.nextToken();

                reply = ts.release(id, session, clientName);

            } else if (action.equals(T_Test)) {              /* TEST. */

                reply = ts.test(id, session);
 
            } else {
                System.out.println("SessionServerMessage:" +
				   " parseProxyTokenMessage:" +
                                   " unknown TOKEN action: " + action);
            }
        }
 
        if (reply != null) {
            thread.writeLine(null, reply);
        }
    }
}
