
/*
 * @(#)TCPSocketThread.java	1.2 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.Socket;
import java.io.*;
import java.util.Vector;

/**
 * Shared Data TCP socket thread class.
 *
 * @version     0.95 15 May 1997
 * @author      Rich Burridge
 */

class
TCPSocketThread extends SocketThread {

    /** The socket for this thread. */
    private Socket socket;

    /** The input stream associated with this socket. */
    private InputStream in;

    /** The output stream associated with this socket. */
    protected OutputStream out;

    /** The data input stream associated with this socket. */
    protected DataInputStream dataIn;


/**
 * <A NAME="SD_TCPSOCKETTHREAD"></A>
 * <EM>TCPSocketThread</EM>
 *
 * @param <EM>socket</EM>
 */

    public
    TCPSocketThread(Socket socket) {
        if (TCPSocketThread_Debug) {
            System.out.println("TCPSocketThread: constructor:" +
                               " socket: " + socket);
        }

        try {
            in     = socket.getInputStream();
            out    = socket.getOutputStream();
            dataIn = new DataInputStream(in);
        } catch (Exception e) {
            System.out.println("TCPSocketThread: constructor: " +
                               "exception " + e);
            if (TCPSocketThread_Debug) {
                e.printStackTrace();
            }
        }

        this.socket = socket;
        messages    = new Vector();
    }


/**
 * <A NAME="SD_TCPSOCKETTHREAD"></A>
 * <EM>TCPSocketThread</EM> creates
 *
 * @param <EM>host</EM>
 * @param <EM>port</EM>
 */

    public
    TCPSocketThread(String host, int port) {
        if (TCPSocketThread_Debug) {
            System.out.println("TCPSocketThread: constructor:" +
                               " host: "    + host +
                               " port: "    + port);
        }

        try {
            this.host     = host;
            this.port     = port;

            socket    = new Socket(host, port);
            in        = socket.getInputStream();
            out       = socket.getOutputStream();
            dataIn    = new DataInputStream(in);
        } catch (Exception e) {
            System.out.println("TCPSocketThread: constructor: " +
                               "exception " + e);
            if (TCPSocketThread_Debug) {
                e.printStackTrace();
            }
        }
 
        messages = new Vector();
    }


/**
 * <A NAME="SD_GETSOCKETMESSAGE"></A>
 * <EM>getSocketMessage</EM>
 *
 * @return
 */

    String
    getSocketMessage() {
        String s = null;

	if (TCPSocketThread_Debug) {
            System.out.println("TCPSocketThread: getSocketMessage.");
        }

	try {
	    s = dataIn.readLine();
	} catch (Exception e) {
            System.out.println("TCPSocketThread: getSocketMessage " +
			       "- exception " + e);
            if (TCPSocketThread_Debug) {
                e.printStackTrace();
            }
        }

	return(s);
    }


/**
 * <A NAME="SD_READBYTES"></A>
 * <EM>readBytes</EM>
 *
 * @param <EM>length</EM>
 *
 * @return
 */

    synchronized byte[]
    readBytes(int length) {
        byte theBytes[] = new byte[length];
             
        if (TCPSocketThread_Debug) { 
            System.out.println("TCPSocketThread: readBytes: " + length);
        }                  
             
        try {
            int remaining = length;
            int off       = 0;

            while (remaining > 0) {
                int bytesRead = dataIn.read(theBytes, off, remaining);
        
                off       += bytesRead;
		remaining -= bytesRead;
            }
        } catch (IOException e) {
            System.out.println("TCPSocketThread: readBytes: " +
                               "exception " + e);
            if (TCPSocketThread_Debug) {
                e.printStackTrace();
            }
        }
        return(theBytes);
    }


/**
 * <A NAME="SD__WRITE"></A>
 * <EM>_write</EM> write a message (possibly with data) over the socket.
 *
 * @param <EM>waitCommand</EM> the reply to wait for (if any).
 * @param <EM>message</EM> the message to write to the socket.
 * @param <EM>dataBytes</EM> the data (or null if there isn't any).
 * @param <EM>length</EM> the length (in bytes) of the data.
 *
 * @return the reply (if we were waiting for it), otherwise null.
 */
 
    synchronized String
    _write(String waitCommand, String message, byte dataBytes[], int length) {
        byte   messageBytes[] = new byte[message.length()];
        String reply          = null;
 
        if (TCPSocketThread_Debug) {
            System.out.println("TCPSocketThread: _write:" +
                                " waitCommand: " + waitCommand +
                                " message: "     + message   +
                                " data: "        + dataBytes +
                                " length: "      + length);
        }
 
        try {
            if (waitCommand != null) {
                while (isWaiting == true) {
                    wait();
                }
                this.waitCommand = waitCommand;
                isWaiting        = true;
            }
 
            message.getBytes(0, message.length(), messageBytes, 0);
            out.write(messageBytes);
            out.write('\n');
            if (dataBytes != null) {
                out.write(dataBytes, 0, length);
            }
            out.flush();
 
            while (waitCommand != null && isWaiting == true) {
                reply = getMessage();
                if (reply.startsWith(this.waitCommand)) {
                    if (SocketThread_Debug) {
                        System.out.println("TCPSocketThread: _write: found: " +
                                           this.waitCommand);
                    }
 
                    isWaiting = false;
                } else {
                    handleMessage(reply);
                }
            }
        } catch (Exception e) {
            System.out.println("TCPSocketThread: _write: exception " + e);
            if (TCPSocketThread_Debug) {
                e.printStackTrace();
            }
        }
        return(reply);
    }
}
