
/*
 * @(#)TokenServer.java	1.2 97/05/28
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.util.*;

/**
 * Shared Data Token Server-side class.
 *
 * @version     0.96 23 May 1997
 * @author      Rich Burridge
 */

public class
TokenServer extends SharedDataServer implements AbstractTokenServer {

    /** The server-side session associated with this token server. */
    private Session session;

    /** The server-side token associated with this token server. */
    private Token token;

    /** The clients who have currently grabbed/inhibited this token. */
    private Hashtable grabbedClients = null;

    /** The client potentially being given this token (if any). */
    private Client receivingClient = null;


/**
 * <A NAME="SD_INITSERVER"></A>
 * <EM>initServer</EM> initialise the "server-side" for this Token.
 *
 * @param <EM>name</EM> the name of the server Token being constructed.
 * This name cannot include '\u0001' or newline characters.
 * @param <EM>session</EM> the server-side session the Token belongs to.
 * @param <EM>object</EM> the server-side Token.
 */

    public void
    initServer(String name, Session session, Object object) {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: initServer:" +
            		       " name: "    + name        +
            		       " session: " + session);
        }

        this.name      = name;
        this.session   = session;
        this.token     = (Token) object;

        clients        = new Hashtable();
        grabbedClients = new Hashtable();
        super.initServer(name, session, object);
    }


    public Object
    getServer() {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: getServer.");
        }
 
        return(this);
    }


/**
 * <A NAME="SD_ADDGRABBEDCLIENT"></A>
 * <EM>addGrabbedClient</EM>
 *
 * @param <EM>client</EM>
 */

    void
    addGrabbedClient(Client client) {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: addGrabbedClient:" +
			       " client: " + client);
        }

	synchronized (grabbedClients) {
	     grabbedClients.put(client.getName(), client);
	}
    }


/**
 * <A NAME="SD_REMOVEGRABBEDCLIENT"></A>
 * <EM>removeGrabbedClient</EM>
 *
 * @param <EM>client</EM>
 *
 * @return
 */

    boolean
    removeGrabbedClient(Client client) {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: removeGrabbedClient:" +
			       " client: " + client);
        }

        if (grabbedClients != null) {
            synchronized (grabbedClients) {
		return(grabbedClients.remove(client.getName()) != null);
	    }
        }
        return(false);
    }    


/**
 * <A NAME="SD_GIVE"></A>
 * <EM>give</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>sessionName</EM>
 * @param <EM>givingClientName</EM>
 * @param <EM>receivedClientName</EM>
 *
 * @return
 */                           

    String
    give(int id, String givingClientName, String receivingClientName) {
        SocketThread thread = getThreadForClient(receivingClientName);
        Client client = getClientByName(receivingClientName);
	int    retval  = 0;
        String message = addMessageHeader(session.getName(), 0,
			     Token.M_Token + T_Sep + T_InformObserver + T_Sep +
			     name + T_Sep + givingClientName + T_Sep +
			     TokenEvent.GIVE);

        if (TokenServer_Debug) {
            System.out.println("TokenServer: give:" +
                           " giving client name: "    + givingClientName +
                           " receiving client name: " + receivingClientName);
        }
 
	if (client == null) {
	    retval = SharedDataException.NO_SUCH_CLIENT;
	} else if (thread != null) {
	    CheckToken ct = new CheckToken(token, getTokenStatus());
	    Thread checkThread = new Thread(ct);

	    checkThread.start();

	    thread.writeLine(null, message);
	    setTokenStatus(Token.GIVING);
	    setReceivingClient(client);
	}

	return(addMessageHeader(session.getName(), id, Token.M_Token + T_Sep +
			T_Give + T_Sep + retval + T_Sep + getTokenStatus()));
    }


/**
 * <A NAME="SD_GRAB"></A>
 * <EM>grab</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>session</EM>
 * @param <EM>clientName</EM>
 * @param <EM>exclusive</EM>
 *
 * @return
 */

    String
    grab(int id, Session session, String clientName, boolean exclusive) {
        int retval    = 0;
        int status    = 0;
	String reply  = null;
        Client client = getClientByName(clientName);

        if (TokenServer_Debug) {
            System.out.println("TokenServer: grab:" +
                               " id: "         + id +
                               " session: "    + session +
                               " clientName: " + clientName +
			       " exclusive: "  + exclusive);
        }
         
        if (client == null ||
	    ((SessionServer) session.so).validClient(client) == false) {
            retval = SharedDataException.NO_SUCH_CLIENT;
        }
 
        if (retval == 0) {
            if (getTokenStatus() == Token.NOT_IN_USE) {
                if (exclusive == true) {
                    setTokenExclusive(true);
                    setTokenStatus(Token.GRABBED);
                } else {
                    setTokenStatus(Token.INHIBITED);
                }
                addGrabbedClient(client);
		status = getTokenStatus();
            } else if (getTokenStatus() == Token.INHIBITED) {
                if (grabbedClientsContains(client) == false) {
		    addGrabbedClient(client);
                }
            } else if (getTokenStatus() == Token.GRABBED) {
                status = Token.ALREADY_GRABBED;
	    } else if (getTokenStatus() == Token.GIVING) {
		if (client.equals(getReceivingClient()) == true) {
		    if (exclusive == true) {
			setTokenExclusive(true);
                        setTokenStatus(Token.GRABBED);
                    } else {
                        setTokenStatus(Token.INHIBITED);
                    }
                    addGrabbedClient(client);
		}
		status = getTokenStatus();
	    }
	}
 
        return(addMessageHeader(session.getName(), id, Token.M_Token + T_Sep +
				T_Grab + T_Sep + retval + T_Sep + status));
    }


/**
 * <A NAME="SD_JOIN"></A>
 * <EM>join</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>session</EM>
 * @param <EM>thread</EM>
 * @param <EM>clientName</EM>
 *
 * @return
 */

    String
    join(int id, Session session, SocketThread thread, String clientName) {
        int retval    = 0;
        Client client = new ClientImpl(clientName);

        if (TokenServer_Debug) {
            System.out.println("TokenServer: join:" +
                               " id: "         + id +
                               " thread: "     + thread +
                               " session: "    + session +
            		       " clientName: " + clientName);
        }

        if (((SessionServer) session.so).validClient(client) == false) {
            retval = SharedDataException.NO_SUCH_CLIENT;
        } else {
            if (retval == 0) {
                synchronized (clients) {
		    if (clients.containsKey(clientName)) {
			retval = SharedDataException.NAME_IN_USE;
		    } else {
		        clients.put(clientName, client);
		    }
		}

                if (retval == 0) {
		    informObservers(session.getName(), clientName,
				    SharedDataEvent.JOIN, Token.M_Token);
                    addClientThreadConnection(clientName, thread);
		}
	    }
        }

        return(addMessageHeader(session.getName(), id, Token.M_Token + T_Sep +
				T_Join + T_Sep + retval));
    }

     
/**
 * <A NAME="SD_PLEASE"></A>
 * <EM>please</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>sessionName</EM>
 * @param <EM>clientName</EM>
 *
 * @return
 */

    String
    please(int id, String clientName) {
        int    retval  = 0;
        String message = addMessageHeader(session.getName(), 0,
			     Token.M_Token + T_Sep + T_InformObserver + T_Sep +
			     name + T_Sep + clientName + T_Sep +
			     TokenEvent.PLEASE);

        if (TokenServer_Debug) {
            System.out.println("TokenServer: please:"  +
                               " id: "          + id +
                               " client name: " + clientName);
        }
        if (grabbedClients != null) {
	    synchronized (grabbedClients) {
		for (Enumeration e = grabbedClients.keys();
				 e.hasMoreElements();) {
                    SocketThread thread =
			 getThreadForClient((String) e.nextElement());

	            if (thread != null) {
                        thread.writeLine(null, message);
        	    }
		}
	    }
	}
        return(addMessageHeader(session.getName(), id, Token.M_Token + T_Sep +
				T_Please + T_Sep + retval + T_Sep +
				getTokenStatus()));
    }


/**
 * <A NAME="SD_RELEASE"></A>
 * <EM>release</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>session</EM>
 * @param <EM>clientName</EM>
 *
 * @return
 */

    String
    release(int id, Session session, String clientName) {
        int    retval = 0;
	Client client = null;

        if (TokenServer_Debug) {
            System.out.println("TokenServer: release:"   +
                               " id: "         + id +
                               " clientName: " + clientName);
        }

        if ((client = getGrabbedClientByName(clientName)) == null) {
            retval = SharedDataException.CLIENT_NOT_GRABBING;
        }

        if (((SessionServer) session.so).validClient(client) == false) {
            retval = SharedDataException.NO_SUCH_CLIENT;
        }

        if (retval == 0) {
            if (getTokenExclusive() == true) {
                if (removeGrabbedClient(client) == true) {
                    setTokenStatus(Token.NOT_IN_USE);
                } else {
                    retval = SharedDataException.CLIENT_NOT_RELEASED;
                }
            } else {
                if (removeGrabbedClient(client) == true) {
                    if (grabbedClientsSize() == 0) {
                        setTokenStatus(Token.NOT_IN_USE);
                    } else {
                        setTokenStatus(Token.INHIBITED);
                    }
                } else {
                    retval = SharedDataException.CLIENT_NOT_RELEASED;
                }
            }
        }

        return(addMessageHeader(session.getName(), id, Token.M_Token + T_Sep +
				T_Release + T_Sep + retval + T_Sep +
				getTokenStatus()));
    }


/**
 * <A NAME="SD_TEST"></A>
 * <EM>test</EM>
 *
 * @param <EM>id</EM> the id to use for client/server messages.
 * @param <EM>session</EM>
 *
 * @return
 */

    String
    test(int id, Session session) {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: test:" +
                                " id: "      + id +
                                " session: " + session);
        }

        return(addMessageHeader(session.getName(), id, Token.M_Token + T_Sep +
				T_Test + T_Sep + "0" + T_Sep +
				getTokenStatus()));
    }


/**
 * <A NAME="SD_GETGRABBEDCLIENTBYNAME"></A>
 * <EM>getGrabbedClientByName</EM>
 *
 * @param <EM>clientName</EM>
 *
 * @return
 */

    private Client
    getGrabbedClientByName(String clientName) {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: getGrabbedClient:" +
			       " client name: " + clientName);
        }

        if (grabbedClients != null) {
	    synchronized (grabbedClients) {
                return((Client) grabbedClients.get(clientName));
	    }
        }
        return(null);
    }


/**
 * <A NAME="SD_GETTOKENSTATUS"></A>
 * <EM>getTokenStatus</EM>
 *
 * @return
 */
 
    synchronized int
    getTokenStatus() {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: getTokenStatus:" +
                               " token name: " + token.getName());
        }
 
        return(token.status.intValue());
    }


/**
 * <A NAME="SD_SETTOKENSTATUS"></A>
 * <EM>setTokenStatus</EM>
 *
 * @param <EM>value</EM>
 */

    synchronized void
    setTokenStatus(int value) {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: setTokenStatus:" +
                               " token name: " + token.getName() +
			       " new status: " + value);
        }

	token.status = new Integer(value);
    }


/**
 * <A NAME="SD_GETRECEIVINGCLIENT"></A>
 * <EM>getReceivingClient</EM>
 *
 * @return
 */
 
    private synchronized Client
    getReceivingClient() {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: getReceivingClient.");
        }

        return(receivingClient);
    }


/**
 * <A NAME="SD_SETRECEIVINGCLIENT"></A>
 * <EM>setReceivingClient</EM>
 *
 * @param <EM>receivingClient</EM>
 */

    private synchronized void
    setReceivingClient(Client receivingClient) {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: setReceivingClient:" +
                               " receiving client: " + receivingClient);
        }

        this.receivingClient = receivingClient;
    }


/**
 * <A NAME="SD_GETTOKENEXCLUSIVE"></A>
 * <EM>getTokenExclusive</EM>
 *
 * @return
 */
 
    private synchronized boolean
    getTokenExclusive() {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: getTokenExclusive:" +
                               " token name: " + token.getName());
        }
 
        return(token.isExclusive);
    }
 
 
/**
 * <A NAME="SD_SETTOKENEXCLUSIVE"></A>
 * <EM>setTokenExclusive</EM>
 *
 * @param <EM>value</EM>
 */
 
    private synchronized void
    setTokenExclusive(boolean value) {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: setTokenExclusive:" +
                               " token name: " + token.getName() +
                               " new value: "  + value);
        }
 
        token.isExclusive = value;
    }


/**
 * <A NAME="SD_GRABBEDCLIENTSSIZE"></A>
 * <EM>grabbedClientsSize</EM>
 *
 * @return
 */

    private int
    grabbedClientsSize() {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: grabbedClientsSize.");
        }

        if (grabbedClients != null) {
	    synchronized (grabbedClients) {
		return(grabbedClients.size());
	    }
	}
	return(0);
    }


/**
 * <A NAME="SD_GRABBEDCLIENTSCONTAINS"></A>
 * <EM>grabbedClientsContains</EM>
 *
 * @param <EM>client</EM>
 *
 * @return
 */

    private boolean
    grabbedClientsContains(Client client) {
        if (TokenServer_Debug) {
            System.out.println("TokenServer: grabbedClientsContains: " +
			       " client name: " + client.getName());
        }

        if (grabbedClients != null) {
            synchronized (grabbedClients) {
                return(grabbedClients.contains(client));
            }
        }
        return(false);
    }
}
