
/*
 * @(#)UDPChannelServerMessage.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.InetAddress;
import java.util.StringTokenizer;

/**
 * Shared Data UDP Channel Server Message class.
 *
 * @version     0.94 7 April 1997
 * @author      Rich Burridge
 */

class
UDPChannelServerMessage extends SharedDataMessage {

    /** Channel server thread associated with this message. */
    private UDPSocketThread thread;

    /** The channel that this message was sent over. */
    private Channel channel;

    /** The internet address of the sender of the message. */
    private InetAddress address;

    /** The port number used by the sender. */
    private int port;

    /** The channel data bytes associated with this message. */
    private byte data[];


/**
 * <A NAME="SD_UDPCHANNELSERVERMESSAGE"></A>
 * <EM>UDPChannelServerMessage</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>channel</EM>
 * @param <EM>address</EM>
 * @param <EM>port</EM>
 * @param <EM>message</EM>
 * @param <EM>data</EM>
 */

    public
    UDPChannelServerMessage(UDPChannelServerThread thread,
			    Session session, Channel channel,
			    InetAddress address, int port, byte data[]) {
        if (UDPChannelServerMessage_Debug) {
            System.out.println("UDPChannelServerMessage: constructor:" +
                               " thread: "  + thread +
			       " session: " + session +
                               " channel: " + channel +
			       " address: " + address +
			       " port: "    + port +
                               " data: "    + data);
        }

        this.thread  = thread;
	this.session = session;
        this.channel = channel;
	this.address = address;
	this.port    = port;
        this.data    = data;
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>tok</EM>
 */
 
    protected void
    handleMessage(StringTokenizer tok) {
        int    id   = Integer.parseInt(tok.nextToken());
        String type = tok.nextToken();
 
        if (UDPChannelServerMessage_Debug) {
            System.out.println("UDPChannelServerMessage: handleMessage:" +
                               " tok: " + tok);
        }

        if (type.equals(Channel.M_Channel)) {
            parseProxyChannelMessage(id, tok);
        } else {
            System.out.println("UDPChannelServerMessage: handleMessage:" +
                               " unknown type: " + type);
        }
    }


/**
 * <A NAME="SD_PARSEPROXYCHANNELMESSAGE"></A>
 * <EM>parseProxyChannelMessage</EM>
 *
 * @param <EM>id</EM> the id to use in client/server messages.
 * @param <EM>tok</EM>
 */

    void
    parseProxyChannelMessage(int id, StringTokenizer tok) {
        String  action      = tok.nextToken();
        String  channelName = tok.nextToken();
        String  reply       = null;
        SessionServer ss    = (SessionServer) session.so.getServer();
	Channel channel     = ss.getChannelByName(channelName);

        if (UDPChannelServerMessage_Debug) {
            System.out.println("UDPChannelServerMessage:" +
                               " parseProxyChannelMessage: " +
                               " action: "      + action +
                               " channelName: " + channelName +
                               " channel: "     + channel);
        }

        if (channel == null) {
            reply = addMessageHeader(session.getName(), id,
                                 Channel.M_Channel + T_Sep + action + T_Sep +
                                 SharedDataException.NO_SUCH_CHANNEL);
 
        } else {
	    ChannelServer cs = (ChannelServer) channel.so.getServer();

            if (action.equals(T_Send)) {                     /* SEND. */
                String  senderName   = tok.nextToken();
                String  receiverName = tok.nextToken();
                int     priority     = Integer.parseInt(tok.nextToken());
                int     length       = Integer.parseInt(tok.nextToken());
                boolean uniform      = tok.nextToken().equals("true")
                                        ? true : false;

                reply = cs.send(id, session, channelName, senderName,
                                receiverName, priority, length, uniform, data);
            } else if (action.equals(T_SetConsumer)) {    /* SETCONSUMER. */
                String clientName   = tok.nextToken();
                String consumerName = tok.nextToken();
                int    priority     = Integer.parseInt(tok.nextToken());

                reply = cs.setConsumer(id, thread, session.getName(),
                                       clientName, consumerName, priority);
            } else {
                System.out.println("UDPChannelServerMessage:" +
                                   " parseProxyChannelMessage:" +
                                   " unknown CHANNEL action: " + action);
            }
	}

        if (reply != null) {
            thread.writeLine(null, reply);
        }
    }
}
