
/*
 * @(#)UDPChannelServerThread.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.DatagramPacket;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.util.StringTokenizer;

/**
 * Shared Data UDP Channel Server Thread class.
 *
 * @version     0.89 27 January 1997
 * @author      Rich Burridge
 */

class
UDPChannelServerThread extends UDPSocketThread {

    /** The session that this UDP channel belongs too. */
    private Session session;

    /** The UDP channel that this packet came over. */
    private Channel channel;

    /** The datagram packet being processed. */
    private static DatagramPacket packet = null;

    /** The unpacked datagram packet. */
    private UDPPacketParser pp = null;


/**
 * <A NAME="SD_UDPCHANNELSERVERTHREAD"></A>
 * <EM>UDPChannelServerThread</EM>
 *
 * @param <EM>session</EM>
 * @param <EM>channel</EM>
 * @param <EM>packet</EM>
 */

    public
    UDPChannelServerThread(Session session, Channel channel,
			   DatagramPacket packet) {
        super(packet.getAddress().getHostName(), packet.getPort());

        if (UDPChannelServerThread_Debug) {
            System.out.println("UDPChannelServerThread: constructor:" +
                               " session: " + session +
			       " channel: " + channel +
			       " packet: "  + packet);
        }

        this.session = session;
	this.channel = channel;
	this.packet  = packet;
    }


/**
 * <A NAME="SD_HANDLEMESSAGE"></A>
 * <EM>handleMessage</EM>
 *
 * @param <EM>message</EM>
 */

    protected void
    handleMessage(String message) {
        StringTokenizer tok = new StringTokenizer(message, T_Sep, false);
	String sessionName = null;

        if (UDPChannelServerThread_Debug) {
            System.out.println("UDPChannelServerThread: handleMessage:" +
                               " message: " + message);
        }

        if ((sessionName = validMessageHeader(tok)) != null) {
	    UDPChannelServerMessage csm = new
			 UDPChannelServerMessage(this, session, channel,
                                        packet.getAddress(), packet.getPort(),
                                        pp.getData());
            csm.handleMessage(tok);
	}
    }


/**
 * <A NAME="SD__WRITE"></A>
 * <EM>_write</EM> write a message (possibly with data) over the UDP channel.
 *
 * @param <EM>waitCommand</EM> the reply to wait for (if any).
 * @param <EM>message</EM> the message to write to the socket.
 * @param <EM>dataBytes</EM> the data (or null if there isn't any).
 * @param <EM>length</EM> the length (in bytes) of the data.
 *
 * @return the reply (if we were waiting for it), otherwise null.
 */

    synchronized String
    _write(String waitCommand, String message, byte dataBytes[], int length) {
        ByteArrayOutputStream ba  = new ByteArrayOutputStream();
        DataOutputStream      dos = new DataOutputStream(ba);
        DatagramPacket        dp  = null;
        String s              = null;

        if (UDPChannelServerThread_Debug) {
            System.out.println("UDPChannelServerThread: _write:" +
                                " waitCommand: " + waitCommand +
                                " message: "     + message   +
                                " data: "        + dataBytes +
                                " length: "      + length);
        }

        try {
            if (waitCommand != null) {
                while (isWaiting == true) {
                    wait();
                }
                this.waitCommand = waitCommand;
                isWaiting        = true;
            }

            dos.writeInt(length);
            dos.writeUTF(message);
            if (length != 0) {
                dos.write(dataBytes, 0, length);
            }  
            dp = new DatagramPacket(ba.toByteArray(), ba.size(),
                                    address, port);
            dsock.send(dp);

            while (waitCommand != null && isWaiting == true) {
                s = getMessage();

                if (UDPChannelServerThread_Debug) {
                    System.out.println("UDPChannelServerThread: _write:" +
                                       " about to handle:" + s);
                }

                if (s.startsWith(this.waitCommand)) {
                    System.out.println("UDPChannelServerThread: _write:" +
				       " found: " + this.waitCommand);
                    isWaiting = false;
                } else {
                    StringTokenizer tok = new StringTokenizer(s, T_Sep, false);
                    String action, type;

                    if (validMessageHeader(tok) != null) {
                        type   = tok.nextToken();
                        action = tok.nextToken();

                        if (type.equals(Channel.M_Channel) &&
			    action.equals(T_Send)) {
                            handleMessageAndData(s, getData());
                        } else {
                            handleMessage(s);
                        }
                    }    
                }
            }    
        } catch (Exception e) {
            System.out.println("UDPChannelServerThread: _write: " +
			       "exception " + e);
            if (UDPChannelProxyThread_Debug) {
                e.printStackTrace();
            }
        }
        return(s);
    }


/**
 * <A NAME="SD_RUN"></A>
 * <EM>run</EM>
 */

    public void
    run() {
        if (UDPChannelServerThread_Debug) {
            System.out.println("UDPChannelServerThread: run.");
        }

	pp = new UDPPacketParser(packet.getData(), packet.getLength());
	handleMessage(pp.getMessage());
    }
}
