
/*
 * @(#)UDPPacketParser.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.io.ByteArrayInputStream;
import java.io.DataInputStream;

/**
 * Shared Data UDP Packet Parser class.
 *
 * @version     0.89 27 January 1997
 * @author      Rich Burridge
 */

class
UDPPacketParser extends socketSharedDataObject {

    /** The packet data. */
    private byte[] packetData;

    /** The packet length. */
    private int packetLength;

    /** The message portion of the datagram packet. */
    private String message;

    /** The data portion of the datagram packet. */
    private byte[] data = null;

/**
 * <A NAME="SD_UDPPACKETPARSER"></A>
 * <EM>UDPPacketParser</EM>
 *
 * @param <EM>packetData</EM>
 * @param <EM>packetLength</EM>
 */

    UDPPacketParser(byte[] packetData, int packetLength) {
        ByteArrayInputStream ba     = new ByteArrayInputStream(packetData);
        DataInputStream      dis    = new DataInputStream(ba);
	int                  length = 0;

	if (UDPPacketParser_Debug) {
            System.out.println("UDPPacketParser: constructor:" +
                               " packet data "    + packetData +
                               " packet length: " + packetLength);
        }

        this.packetData   = packetData;
        this.packetLength = packetLength;

	try {
	    length = dis.readInt();
	    message = dis.readUTF();
	    if (length != 0) {
                data = new byte[length];
	        dis.read(data, 0, length);
	    }
	} catch (Exception e) {
            System.out.println("UDPPacketParser: constructor - exception " + e);
            if (UDPPacketParser_Debug) {
                e.printStackTrace();
            }
	}
    }


/**
 * <A NAME="SD_GETDATA"></A>
 * <EM>getData</EM>
 *
 * @return the data portion of the datagram packet sent over the unreliable
 * channel.
 */

    byte[]
    getData() {
        if (UDPPacketParser_Debug) {
            System.out.println("UDPPacketParser: getData.");
        }

	return(data);
    }


/**
 * <A NAME="SD_GETMESSAGE"></A>
 * <EM>getMessage</EM>
 *
 * @return the message portion of the datagram packet sent over the
 * unreliable channel.
 */

    String
    getMessage() {
        if (UDPPacketParser_Debug) {
            System.out.println("UDPPacketParser: getMessage.");
        }

	return(message);
    }
}
