
/*
 * @(#)UDPSocketThread.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;
import java.net.*;
import java.util.Vector;

/** 
 * Shared Data UDP socket thread class.
 *
 * @version     0.89 27 January 1997
 * @author      Rich Burridge
 */
 
class
UDPSocketThread extends SocketThread { 

    /** The datagram socket used to send and receive packets. */
    protected DatagramSocket dsock = null;

    /** The Internet address of the server. */
    protected InetAddress address = null;

    /** The channel data portion of the current packet. */
    protected byte data[] = null;


/**
 * <A NAME="SD_UDPSOCKETTHREAD"></A>
 * <EM>UDPSocketThread</EM>
 *
 * @param <EM>host</EM>
 * @param <EM>port</EM>
 */


    public
    UDPSocketThread(String host, int port) {
        if (UDPSocketThread_Debug) {
            System.out.println("UDPSocketThread: constructor:" +
                               " host: " + host +
			       " port: " + port);
        }

        try {
            dsock   = new DatagramSocket();
            address = InetAddress.getByName(host);
        } catch (Exception e) {
            System.out.println("UDPSocketThread: exception " + e);
            if (UDPSocketThread_Debug) {
                e.printStackTrace();
            }
        }

	this.host = host;
        this.port = port;
        messages  = new Vector();
    }    


/**
 * <A NAME="SD_GETSOCKETMESSAGE"></A>
 * <EM>getSocketMessage</EM>
 *
 * @return
 */

    String
    getSocketMessage() {
        String s = null;

        if (UDPSocketThread_Debug) {
            System.out.println("UDPSocketThread: getSocketMessage.");
        }

        try {
            int             size   = socketSharedDataObject.UDP_BUF_SIZE;
	    DatagramPacket  packet = new DatagramPacket(new byte[size], size);
            UDPPacketParser pp     = null;

	    dsock.receive(packet);

/* XXX: need to be able to handle data packets > UDP_BUF_SIZE. */

            pp   = new UDPPacketParser(packet.getData(), packet.getLength());
	    s    = pp.getMessage();
	    data = pp.getData();

        } catch (Exception e) {
            System.out.println("UDPSocketThread: getSocketMessage " +
                               "- exception " + e);
            if (UDPSocketThread_Debug) {
                e.printStackTrace();
            }
        }

        return(s);
    }
}
