
/*
 * @(#)socketClient.java	1.1 97/05/15
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package SharedData.socket;

import SharedData.*;

/**
 * Shared Data Socket Client class.
 *
 * @version     0.93 6 March 1997
 * @author      Rich Burridge
 */

public class
socketClient extends socketSharedDataObject implements Client {

    /** The name of this client. */
    private String name;

    /** Handle to server-side object for this client. */
    ClientServer cs;
  
    /** Handle to client-side proxy for this client. */
    ClientProxy cc;


    public
    socketClient() {
        if (socketClient_Debug) {
            System.out.println("socketClient: default constructor.");
        }
    }


/**
 * <A NAME="SD_SOCKETCLIENT"></A>
 * <EM>socketClient</EM> the constructor for the socketClient class.
 *
 * @param <EM>name</EM> the name of the socket client being constructed.
 * The client name cannot include '\u0001' or newline characters.
 */

    public
    socketClient(String name) {
        if (socketClient_Debug) {
            System.out.println("socketClient: constructor:" +
                               " name: " + name);
        }

        this.name = name;
    }


/**
 * <A NAME="SD_AUTHENTICATE"></A>
 * <EM>authenticate</EM> used to authenticate a client for potentially
 * joining a session. The Session, Channel or Token Manager will be doing
 * this client validation.
 *
 * The manager sends the client a challenge. The client responds.
 * This response is validated by the manager and determines if the
 * client will be allowed to join the session, channel or token.
 *
 * @param <EM>challenge</EM> the challenge given by the manager to
 * the client.
 *
 * @return the response by the client to the managers challenge.
 */

    public String
    authenticate(String challenge) {
        if (socketClient_Debug) {
            System.out.println("socketClient: authenticate:" +
			" this class should not be called directly."  +
			" You should subclass the socketClient class.");
        }

        return(null);
    }


/**
 * <A NAME="SD_GETNAME"></A>
 * <EM>getName</EM> get the name of this client.
 *
 * @return the name of the client.
 * The client name cannot include '\u0001' or newline characters. 
 */
 
    public String
    getName() {
        if (socketClient_Debug) {
            System.out.println("socketClient: getName: " + name);
        }

        return(name);
    }    
 
 
/**
 * <A NAME="SD_CLIENTCHANGED"></A>
 * <EM>clientChanged</EM> is called when a client change has been observed.
 *
 * It should never be called for this implementation.
 *
 * @param <EM>event</EM> the client changed event containing more information
 * on exactly what has changed.
 *
 * </pre>
 */
 
    public void
    clientChanged(ClientEvent event) {
        if (socketClient_Debug) {
            System.out.println("socketClient: clientChanged:" +
			" this class should not be called directly."  +
			" You should subclass the socketClient class.");
        }
    }


/* Private/protected/package methods. */

/**
 * <A NAME="SD__CREATEPROXY"></A>
 * <EM>_createProxy</EM> create a proxy-side connection for this socket
 * Client.
 *
 * @param <EM>name</EM> the name of the socket client being constructed.
 * The client name cannot include '\u0001' or newline characters.
 * @param <EM>connectionType</EM> the type of this connection.
 * @param <EM>host</EM> the host of the server-side connection.
 * @param <EM>port</EM> the port number to use for the connection.
 */

    synchronized void
    _createProxy(String name, String connectionType, String host, int port) {
        if (socketClient_Debug) {
            System.out.println("socketClient: _createProxy:"         +
                               " object name: "     + name           +
                               " connection type: " + connectionType +
                               " host: "            + host           +
                               " port: "            + port);
        }

        this.name        = name;
	this.connectType = connectionType;
	cc = new ClientProxy(this, name, host, port);
    System.out.println("New Client Proxy object with host"+host);
    }


/**
 * <A NAME="SD__CREATESERVER"></A>
 * <EM>_createServer</EM> create a server-side connection for this socket
 * Client.
 *
 * @param <EM>name</EM> the name of the socket client being constructed.
 * The client name cannot include '\u0001' or newline characters.
 * @param <EM>connectionType</EM> the type of this connection.
 * @param <EM>url</EM> the url associated with this socket Client.
 * @param <EM>port</EM> the port number to use for the connection.
 */
 
    synchronized void
    _createServer(String name, String connectionType, String url, int port) {
        if (socketClient_Debug) {
            System.out.println("socketClient: _createServer:"        +
                               " object name: "     + name           +
                               " connection type: " + connectionType +          
			       " url: "             + url            +
                               " port: "            + port);
        }

        this.name        = name;
        this.connectType = connectType;
        cs = new ClientServer(this, name, port);
    }    
}
