// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ActionGroup.java
// Classes: ActionGroup
// Original Author: jrobbins@ics.uci.edu
// $Id: ActionGroup.java,v 1.9 1997/06/10 23:42:12 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.util.*;

/** Action to group all the DiagramElement's selected in the current
 *  editor into a single FigList. Needs-More-Work: should use
 *  composite DiagramElement, not composite Fig.  Needs-More-Work:
 *  DiagramElement's other than Figs are ignored.
 *  <A HREF="../features.html#basic_shapes_group">
 *  <TT>FEATURE: basic_shapes_group</TT></A>
 *
 * @see FigList
 * @see ActionUngroup  */

public class ActionGroup extends Action {

  public ActionGroup() { }

  public String name() {
    return "Group selected DiagramElement's into a FigList";
  }

  public void doIt(Event e) {
    Editor ce = Globals.curEditor();
    Vector selectedDEs = ce.selectedDEs();
    if (selectedDEs.size() < 2 && !e.shiftDown()) return;
    FigList newItem = new FigList();
    Enumeration eachDE = selectedDEs.elements();
    while (eachDE.hasMoreElements()) {
      Object o = eachDE.nextElement();
      if (o instanceof Fig) {
	Fig f = (Fig) o;
	newItem.addFig(f);
      }
    }
    eachDE = selectedDEs.elements();
    while (eachDE.hasMoreElements()) {
      Object o = eachDE.nextElement();
      if (o instanceof Fig) {
	Fig f = (Fig) o;
	ce.remove(f);
      }
    }
    ce.add(newItem);
    ce.deselect();
    ce.selectItem(newItem);
  }

  public void undoIt() { System.out.println("not done yet"); }

} /* end class ActionGroup */

