// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ActionRemoveVertex.java
// Classes: ActionRemoveVertex
// Original Author: jrobbins@ics.uci.edu
// $Id: ActionRemoveVertex.java,v 1.6 1997/06/10 23:42:18 jrobbins Exp $

package uci.graphedit;

import java.awt.Event;

/** Action to remove the selected (last manipulated) vertex from a FigPoly.
 *
 * @see FigPoly
 */

public class ActionRemoveVertex extends Action {

  public ActionRemoveVertex() { }

  public String name() { return "Remove selected vertex from polygon"; }

  /** Ask the current editor's selected DiagramElement to remove its vertex. */
  public void doIt(Event e) {
    DiagramElement de = null;
    SelectionSingle sel = null;
    Editor ce = Globals.curEditor();
    SelectionMultiple sels = ce.selection();
    if (sels.locked()) {
      Globals.showStatus("Cannot Modify Locked Objects");
      return;
    }

    if (sels != null && !sels.contents().isEmpty())
      sel = (SelectionSingle) sels.contents().firstElement();
    if (sel != null) de = (DiagramElement) sel.content();
    if (de != null && de instanceof FigPoly) {
        FigPoly p = (FigPoly) de;
        p.startTrans();
        p.removeSelectedVertex();
        p.endTrans();
    }
    if (de != null && de instanceof ArcPerzRectiline) {
        ArcPerzRectiline p = (ArcPerzRectiline) de;
        p.startTrans();
        p.removeSelectedVertex();
        p.endTrans();
    }
  }

  public void undoIt() {
    System.out.println("this operation currently cannot be undone");
  }

} /* end class ActionRemoveVertex */
