// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ActionReorder.java
// Classes: ActionReorder
// Original Author: jrobbins@ics.uci.edu
// $Id: ActionReorder.java,v 1.7 1997/06/10 23:42:19 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** Action to change the back-to-front ordering of DiagramElement's.
 *  <A HREF="../features.html#reorder_objects">
 *  <TT>FEATURE: reorder_objects</TT></A>
 *
 * @see LayerDiagram#reorder */

public class ActionReorder extends Action {
  ////////////////////////////////////////////////////////////////
  // constants
  public static final int SEND_TO_BACK = 1;
  public static final int BRING_TO_FRONT = 2;
  public static final int SEND_BACKWARD = 3;
  public static final int BRING_FORWARD = 4;

  ////////////////////////////////////////////////////////////////
  // instance variables
  private int function;

  ////////////////////////////////////////////////////////////////
  // constructor

  /** Construct a new ActionReorder with the given reordering
    constrant (see above) */
  public ActionReorder(int f) { function = f; }

  ////////////////////////////////////////////////////////////////
  // Action API

  public String name() { return "Reorder Objects"; }

  public void doIt(java.awt.Event e) {
    Editor ce = Globals.curEditor();
    Layer v = ce.view();
    ce.selection().startTrans();
    ce.selection().reorder(function, v);
    ce.selection().endTrans();
    ce.repairDamage();
  }

  public void undoIt() {
    System.out.println("Connot undo ActionReorder, yet");
  }

} /* end class ActionReorder */

