// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ActionSetMode.java
// Classes: ActionSetMode
// Original Author: jrobbins@ics.uci.edu
// $Id: ActionSetMode.java,v 1.9 1997/06/10 23:42:25 jrobbins Exp $

package uci.graphedit;

import java.util.*;

/** Action that sets the next global editor mode. The global editor
 *  mode effects the next editor that you move the mouse into. For
 *  example, used in palette * buttons to set the next global mode to
 *  ModeCreateFigLine.
 *
 * @see PaletteFig */

public class ActionSetMode extends Action {
  /** The class of the desired mode. */
  private Class _desiredModeClass;

  /** Arguments to pass to the new mode instance after creation. */
  private Properties _modeArgs;

  /** Override Action constructor so that setModeClass is allways called. */
  public ActionSetMode(Properties args) {
    super(args);
    setModeClass();
  }

  /** Set the next global mode to the named mode. */
  public ActionSetMode(String modeName) {
    setArg("desiredModeName", modeName);
    setModeClass();
  }

  /** Set the next global mode to the named mode, and maybe make it sticky. */
  public ActionSetMode(String modeName, boolean sticky) {
    setArg("desiredModeName", modeName);
    setArg("shouldBeSticky", sticky ? "True" : "False");
    setModeClass();
  }

  /** Set the next global mode to the named mode, and maybe make it sticky. */
  public ActionSetMode(String modeName, Properties modeArgs) {
    setArg("desiredModeName", modeName);
    _modeArgs = modeArgs;
    setModeClass();
  }

  /** Set the next global mode to the named mode, and maybe make it sticky. */
  public ActionSetMode(String modeName, String modeArgKey, String modeArgVal) {
    setArg("desiredModeName", modeName);
    _modeArgs = new Properties();
    _modeArgs.put(modeArgKey, modeArgVal);
    setModeClass();
  }

  public String name() { return "Set Editor Mode"; }

  /** Find the class of the next global mode from the given name. */
  private void setModeClass() {
    String desiredModeName = (String) getArg("desiredModeName", "ModeSelect");
    try { _desiredModeClass = Class.forName(desiredModeName);}
    catch (java.lang.ClassNotFoundException ignore) { }
  }

  public void doIt(java.awt.Event e) {
    Mode mode;
    // needs-more-work: if mode is not defined, prompt the user
    try { mode = (Mode) _desiredModeClass.newInstance(); }
    catch (java.lang.InstantiationException ignore) {  return; }
    catch (java.lang.IllegalAccessException ignore) { return; }
    mode.args(_modeArgs);
    String shouldBeSticky = (String) getArg("shouldBeSticky");
    if (shouldBeSticky == null) Globals.mode(mode);
    else Globals.mode(mode, shouldBeSticky.equals("True"));
  }

  public void undoIt() {
    System.out.println("undo does not make sense for setting modes");
  }

} /* end class ActionSetMode */
