// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ActiveComponent.java
// Classes: ActiveComponent
// Original Author: jrobbins@ics.uci.edu
// $Id: ActiveComponent.java,v 1.6 1997/06/10 23:42:30 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** Abstract class for objects that combine AWT widgets with editor
 *  Actions. Needs-More-Work: This may be deleted in a future release
 *  because people can do just as well using a GUI builder. */

public abstract class ActiveComponent {

  /** True for palette buttons that can be made to stick down to
   *  allow rapid construction of multiple figures.  <p>
   *  needs-more-work: should probably be in ActionButton. */
  protected boolean _canStick = false;

  public ActiveComponent() { }

  /** Return the AWT widget that the user can manipulate */
  public abstract Component component();
  public abstract Action action();

  /** Carry out an editor Action appropriate for some user interface
   * action.
   * @param e The Event that manipulated the AWT widget.
   */
  public void doIt(Event e) {
    Editor ce = Globals.curEditor();
    if (ce != null) ce.executeAction(action(), e);
  }

  /** Reverse the effect of an earlier action */
  public void undoIt() {
    System.out.println("Not implemented yet");
  }

  /** Set whether a widget can be stuck down to repeatedly perform its
    action */
  public void canStick(boolean b) { _canStick = b; }

  /** Return whether a widget can be stuck down */
  public boolean canStick() { return _canStick; }

  /** Make the widget appear normal, not stuck down. */
  public void unhighlight() {
    component().setBackground(component().getParent().getBackground());
    component().setForeground(component().getParent().getForeground());
  }

  /** Make the widget appear stuck down by inverting foreground and
   * background colors. <p>
   * needs-more-work: This does not work on Windows95. */
  public void highlight() {
    if (_canStick) {
      component().setBackground(component().getParent().getForeground());
      component().setForeground(component().getParent().getBackground());
    }
  }

} /* end class ActiveComponent */
