// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ActiveList.java
// Classes: ActiveList
// Original Author: jrobbins@ics.uci.edu
// $Id: ActiveList.java,v 1.4 1997/06/10 23:42:30 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** An ActiveComponent that holds an AWT List and
 *  executes one of a set of uci.graphedit.Action's based on which one
 *  was chosen. Client Frames must still process events and find when
 *  Active component should handle a given event. <p>
 *
 *  Needs-More-Work: I should define an ActiveChoice that is especially
 *  for setting graphical attributes (see DiagramElement).
 *
 *  Needs-More-Work: This code might get deleted in a future release
 *  if I delete all Palettes and ActiveComponents. */

public class ActiveList extends ActiveComponent {

  /** The AWT List object. */
  private java.awt.List _list;

  /** The Action's to be preformed when a one item is chosen. */
  private Hashtable _actions;

  /** Construct a new ActiveList. */
  public ActiveList() {
    _list = new java.awt.List();
    _actions = new Hashtable();
  }

  /** Construct a new ActiveList. */
  public ActiveList(int numElements) {
    _list = new java.awt.List(numElements, false);
    _actions = new Hashtable();
  }

  /** Return the AWT Choice object. */
  public Component component() { return _list; }
  public Action action() { return null; }
  public Action action(String label) { return (Action) _actions.get(label); }

  /** Based on the event that selects an item, find the proper Action
   *  and execute it.
   *
   * @param e The event that selected an item from my Choice object. */
  public void doIt(java.awt.Event e) {
    Action act = (Action) _actions.get(e.arg);
    if (act != null) { act.doIt(e); }
  }

  public void undoIt() {
    /* needs-more-work: I need the event, or the name of the choice... */
  }

  /** This should be called repeatedly to build up a multiple choice
   *   pop-up menu.
   *
   * @param label  A String describing this item.
   * @param act    An Action to be executed when that item is chosen. */
  public void addItem(String label, Action act) {
    _list.addItem(label);
    _actions.put(label, act);
  }

} /* end class ActiveList */

