// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: FigCircle.java
// Classes: FigCircle
// Original Author: ics125b spring 1996
// $Id: FigCircle.java,v 1.14 1997/06/10 23:42:40 jrobbins Exp $

package uci.graphedit;

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.util.*;

/** Primitive Fig for displaying circles and ovals.
 *
 *  <A HREF="../features.html#basic_shapes_circle">
 *  <TT>FEATURE: basic_shapes_circle</TT></A>.
 */

public class FigCircle extends Fig {

  ////////////////////////////////////////////////////////////////
  // constants

  /** Used as a percentage tolerance for making it easier for the user
   *  to select a hollow circle with the mouse. Needs-More-Work: This
   *  is bad design that needs to be changed. Should use just
   *  GRIP_FACTOR. */
  public static final double CIRCLE_ADJUST_RADIUS = 0.1;

  ////////////////////////////////////////////////////////////////
  // constructors

  /** Construct a new FigCircle with the given position, size, and
   *  line color. */
  public FigCircle(int x, int y, int width, int height, Color c_color) {
    super(x, y, width, height, c_color, null, 4);
  }

  /** Construct a new FigCircle with the given position, size, and
   *  attributes. */
  public FigCircle(int x, int y, int width, int height, Hashtable gAttrs) {
    super(x, y, width, height, Color.black, null, 4);
    put(gAttrs);
  }

  /** Construct a new FigCircle with the given position, size, line
   *  color, and fill color */

  public FigCircle(int x, int y, int width, int height,
		   Color c_color, Color f_color) {
    super(x, y, width, height, c_color, f_color, 4);
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  ////////////////////////////////////////////////////////////////
  // drawing methods

  /** Draw this FigCircle. */
  public void draw(Graphics g) {
    int rx = position().x;
    int ry = position().y;
    int new_x, new_y, new_w, new_h;

    if (objectWidth > 0) { new_x = rx; new_w = objectWidth; }
    else { new_x = rx + objectWidth; new_w = -objectWidth; }

    if (objectHeight > 0) { new_y = ry; new_h = objectHeight; }
    else { new_y = ry + objectHeight; new_h = -objectHeight; }

    /* See if this is a filled object */
    if (filled) {
      g.setColor(objectFillColor);
      g.fillOval(new_x, new_y, new_w, new_h);
    }
    if (lineWidth > 0) {
      g.setColor(objectLineColor);
      g.drawOval(new_x, new_y, new_w-lineWidth, new_h-lineWidth);
    }
  }

  /** Position and draw the handles for when this item is selected. */
  public void drawSelected(Graphics g) {
    int rx = position().x;
    int ry = position().y;
    int new_x, new_y, new_w, new_h;

    if (objectWidth > 0) { new_x = rx; new_w = objectWidth; }
    else { new_x = rx + objectWidth; new_w = -objectWidth; }

    if (objectHeight > 0) { new_y = ry; new_h = objectHeight; }
    else { new_y = ry + objectHeight; new_h = -objectHeight; }

    _handleRects[0].move(new_x + (objectWidth/2) - HAND_SIZE/2,
			 new_y - HAND_SIZE/2);
    _handleRects[1].move(new_x + objectWidth - HAND_SIZE/2,
			 new_y + (objectHeight/2) - HAND_SIZE/2);
    _handleRects[2].move(new_x + (objectWidth/2) - HAND_SIZE/2,
			 new_y + objectHeight - HAND_SIZE/2);
    _handleRects[3].move(new_x - HAND_SIZE/2,
			 new_y + (objectHeight/2) - HAND_SIZE/2);
    drawHandles(g);
  }

  /** When this FigCircle is selected, the Editor should use a
   *  SelectionHandles to record that fact and process events. */
  public Selection selectionObject() { return new SelectionHandles(this); }

  private static Rectangle _CaptureRect = new Rectangle(0, 0, 0, 0);

  /** Reply true if the given mouse coordinates are "near" the
   * circle. Needs-More-Work: there should be separate inside() and
   * near() functions. */
  public boolean inside(int x, int y) {
    boolean ret;
    Rectangle capture;
    double dbl_pntx, dbl_pnty, dbl_width, dbl_height;
    double tmpx, tmpy, centerx, centery;
    double pnt_dist;
    double arc_dist_in, arc_dist_out;
    Point p = position();
    synchronized (_CaptureRect) {
      _CaptureRect.reshape(p.x - GRIP_MARGIN,
			   p.y - GRIP_MARGIN,
			   objectWidth + GRIP_MARGIN * 2,
			   objectHeight + GRIP_MARGIN * 2);
      /* First see if its within the bounding rectangle */
      ret = false;
      if (_CaptureRect.inside(x, y)) {
	centerx = (double)objectWidth/(double)2 + (double)p.x;
	centery = (double)objectHeight/(double)2 + (double)p.y;

	dbl_pntx = (double)(centerx - x) / (objectWidth / 2);
	dbl_pnty = (double)(centery - y) / (objectHeight / 2);

	pnt_dist = Math.sqrt(dbl_pntx * dbl_pntx + dbl_pnty * dbl_pnty);
	dbl_width = (double)objectWidth;
	dbl_height = objectHeight;
	arc_dist_out = 1 + CIRCLE_ADJUST_RADIUS;

	if (!filled) {
	  arc_dist_in = arc_dist_out - CIRCLE_ADJUST_RADIUS;
	} else
	  arc_dist_in = -1;
	ret = ( (pnt_dist < arc_dist_out) && (pnt_dist > arc_dist_in) );
      }
    }
    return ret;
  }

  /** When the user drags one of my handles, move or resize accordingly. */
  public void dragHandle(int mouse_x, int mouse_y,
                         int anchor_x, int anchor_y,
                         Handle h) {
    Point p = position();
    int old_left = p.x;
    int old_top = p.y;
    int old_right = p.x + objectWidth;
    int old_bottom = p.y + objectHeight;

    switch (h.index) {
    case 0:   /* The top middle hitpoint */
      p.y = Math.min(mouse_y, old_bottom - MIN_SIZE);
      objectHeight = old_bottom - p.y;
      break;
    case 1:   /* The right middle hitpoint */
      objectWidth = Math.max(mouse_x - old_left, MIN_SIZE);
      break;
    case 2:   /* The bottom middle hitpoint */
      objectHeight = Math.max(mouse_y - old_top, MIN_SIZE);
      break;
    case 3:    /* The left middle hitpoint */
      p.x = Math.min(mouse_x, old_right - 3);
      objectWidth = old_right - p.x;
      break;
    }
    if (objectWidth < MIN_SIZE) {
      if (h.index == 1) p.x = old_right - MIN_SIZE;
      objectWidth = MIN_SIZE;
    }
    if (objectHeight < MIN_SIZE) {
      if (h.index == 2) p.y = old_bottom - MIN_SIZE;
      objectHeight = MIN_SIZE;
    }
  }

} /* end class FigCircle */

