// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: FigList.java
// Classes: FigList
// Original Author: ics125b spring 1996
// $Id: FigList.java,v 1.10 1997/06/10 23:42:44 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;
import java.io.*;

/** This class implements a basic composite figure that holds a list
 *  of Figs.
 *  <A HREF="../features.html#basic_shapes_group">
 *  <TT>FEATURE: basic_shapes_group</TT></A>
 */

public class FigList extends Fig {

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** The Fig's contained in this FigList */
  private Vector list;

  ////////////////////////////////////////////////////////////////
  // constructors

  /** Construct a new FigList that holds no Fig's. */
  public FigList() {
    super();
    list = new Vector();
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  /** Reply an Enumeration of the Fig's contained in this FigList. */
  public Enumeration elements() { return list.elements(); }

  /** Add a Fig to the list. */
  public void addFig(Fig p) { list.addElement(p); }

  ////////////////////////////////////////////////////////////////
  // DiagramElement API

  /** Retrieve the top-most Fig containing the given point, of null.
   *  Needs-More-Work: just do a linear search.  Later, optimize this
   *  routine using Quad Trees (or other) techniques. */
  public final Fig pick(Point pnt) { return pick(pnt.x, pnt.y); }

  public Fig pick(int x, int y) {
    Enumeration fs = list.elements();
    Fig ret_fig = null;
    while (fs.hasMoreElements()) {
      Fig cur_fig = (Fig) fs.nextElement();
      if (cur_fig.inside(x, y)) ret_fig = cur_fig;
    }
    return ret_fig;
  }

  public boolean inside(int x, int y) { return pick(x, y) != null; }

  /** Translate all the Fig in the list by the given offset. */
  public void translate(int dx, int dy) {
    Enumeration fs = list.elements();
    while (fs.hasMoreElements()) {
      Fig cur_fig = (Fig) fs.nextElement();
      cur_fig.translate(dx, dy);
    }
  }

  /** Delete a Fig from the list */
  public void deleteFig(Fig p) { list.removeElement(p); }

  /** Delete all Fig's from the list. */
  public void removeAll() { list.removeAllElements(); }

  ////////////////////////////////////////////////////////////////
  // drawing methods

  /** Draw all the Fig's in this list. */
  public void draw(Graphics g) {
    Enumeration fs = list.elements();
    while (fs.hasMoreElements()) {
      Fig cur_fig = (Fig) fs.nextElement();
      cur_fig.draw(g);
    }
  }

  /** Needs-More-Work: remove this method, its logic has been moved to
   *  SelectionBox. */
  public void drawSelected(Graphics g) { }

  /** When the Editor selects this FigList, that fact should be
   *  recorded in a SelectionBox object */
  public Selection selectionObject() { return new SelectionBox(this); }

  /** Reply a list of Fig's that have a NetPort as their owner.
   *  Needs-More-Work: should this be in Perspective? */
  public Vector getPortFigs() {
    Vector ret_vect = new Vector();
    Fig cur_fig;
    Enumeration fs = list.elements();
    while (fs.hasMoreElements()) {
      cur_fig = (Fig) fs.nextElement();
      if ((cur_fig.owner() != null) && (cur_fig.owner() instanceof NetPort))
	ret_vect.addElement(cur_fig);
    }
    return ret_vect;
  }

  /** Reply the list of Fig's. */
  public Vector getFigList() { return list; }

  /** Accumulate a bounding box for all the Fig's in the list. */
  public Rectangle getBoundingBox() {
    Rectangle bbox;
    Enumeration fs = list.elements();
    if (fs.hasMoreElements()) bbox = ((Fig)fs.nextElement()).getBoundingBox();
    else return new Rectangle(0, 0, 0, 0);
    while (fs.hasMoreElements()) {
      Fig cur_fig = (Fig) fs.nextElement();
      bbox.add(cur_fig.getBoundingBox());
    }
    return bbox;
  }

} /* end class FigList */

