// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ForwardingPanel.java
// Classes: ForwardingPanel
// Original Author: jrobbins@ics.uci.edu
// $Id: ForwardingPanel.java,v 1.6 1997/06/10 23:42:49 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import uci.util.*;

/** A panel that delgates all of its event handling and repaint method
 *  calls to its EventHandler. Needs-More-Work: This will probably not
 *  be needed under the Java 1.1 event model. */

public class ForwardingPanel extends Panel implements ForwardingComponent {
  ////////////////////////////////////////////////////////////////
  // instance variables

  private EventHandler _eh;

  public Dimension _preferredSize = new Dimension(400, 300);

  ////////////////////////////////////////////////////////////////
  // constructors

  public ForwardingPanel(Dimension d) { this(null, d); }

  public ForwardingPanel(EventHandler eh, Dimension d) {
    _preferredSize = d;
    if (eh != null) setEventHandler(eh);
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  public Dimension preferredSize() { return _preferredSize; }

  public Dimension minimumSize() { return _preferredSize; }

  public void setEventHandler(EventHandler eh) { _eh = eh; }

  public EventHandler getEventHandler() { return _eh; }

  ////////////////////////////////////////////////////////////////
  // drawing methods

  public void paint(Graphics g) { _eh.paint(g); }

  ////////////////////////////////////////////////////////////////
  // event handlers

  public boolean handleEvent(Event e) {
    return _eh.handleEvent(e) || super.handleEvent(e);
  }

  public boolean keyDown(Event e, int key) {
    return _eh.keyDown(e, key);
  }

  //  public boolean keyUp(Event e, int key) {
  //    return _eh.keyUp(e, key);
  //  }
  //
  //  public boolean action(Event e, Object what) {
  //    return _eh.action(e, what);
  //  }

  public boolean mouseExit(Event e, int x, int y) {
    return _eh.mouseExit(e, x, y);
  }

  public boolean mouseEnter(Event e, int x, int y) {
    return _eh.mouseEnter(e, x, y);
  }

  public boolean mouseUp(Event e, int x, int y) {
    return _eh.mouseUp(e, x, y);
  }

  public boolean mouseDown(Event e, int x, int y) {
    return _eh.mouseDown(e, x, y);
  }

  public boolean mouseMove(Event e, int x, int y) {
    return _eh.mouseMove(e, x, y);
  }

  public boolean mouseDrag(Event e, int x, int y) {
    return _eh.mouseDrag(e, x, y);
  }

} /* end class ForwardingPanel */
