// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ModeCreateFigInk.java
// Classes: ModeCreateFigInk
// Original Author: ics125b spring 1996
// $Id: ModeCreateFigInk.java,v 1.4 1997/06/10 23:43:06 jrobbins Exp $

package uci.graphedit;

import java.awt.Event;

/** A Mode to interpert user input while creating a FigInk.
 *  <A HREF="../features.html#basic_shapes_ink">
 *  <TT>FEATURE: basic_shapes_ink</TT></A>
 */

public class ModeCreateFigInk extends ModeCreate {

  ////////////////////////////////////////////////////////////////
  // instance variables

  public static final int MIN_DELTA = 4;

  protected int _lastX, _lastY;

  ////////////////////////////////////////////////////////////////
  // Mode API

  public String instructions() {
    return "Drag to draw a stream of ink";
  }

  ////////////////////////////////////////////////////////////////
  // ModeCreate API

  /** Create a new FigRect instance based on the given mouse down
   *  event and the state of the parent Editor. */
  public DiagramElement createNewItem(Event e, int snapX, int snapY) {
    FigInk p = new FigInk(snapX, snapY, parent.graphAttrs());
    _lastX = snapX; _lastY = snapY;
    return p;
  }

  ////////////////////////////////////////////////////////////////
  // Event handlers

  /** Mouse up means that the user is done. */
  public boolean mouseUp(Event e, int x, int y) {
    parent.damaged(newItem);
    // do not call creationDrag()
    parent.add(newItem);
    parent.selectItem(newItem);
    newItem = null;
    done();
    return true;
  }

  /** Dragging adds points to the ink. */
  public boolean mouseDrag(Event e, int x, int y) {
    FigInk ink = (FigInk)newItem;
    if (!nearLast(x, y)) {
      parent.damaged(newItem); // startTrans?
      ink.addPoint(x, y);
      parent.damaged(newItem); // endTrans?
      _lastX = x; _lastY = y;
    }
    return true;
  }

  /** Internal function to test if the current point is so close to
   *  the last point that it should not be added to the ink. */
  protected boolean nearLast(int x, int y) {
    return x > _lastX - MIN_DELTA &&
      x < _lastX + MIN_DELTA &&
      y > _lastY - MIN_DELTA &&
      y < _lastY + MIN_DELTA;
  }

} /* end class ModeCreateFigInk */

