// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ModeExampleKeys.java
// Classes: ModeExampleKeys
// Original Author: jrobbins@ics.uci.edu
// $Id: ModeExampleKeys.java,v 1.5 1997/06/10 23:43:11 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** Mode to detect many common keystroke commands and execute the
 *  appropriate Action. For example, arrow keys cause the selected
 *  DiagramElements to be nudged, Control-arrows scroll the view, 'g'
 *  groups objects, 'u' ungroups objects, etc.
 *
 *  @see Editor
 *  @see ActionNudge
 *  @see AcionAlign
 *  @see ActionScroll
 *  @see ActionReorder
 *  @see ActionGroup
 *  @see ActionUngroup */

public class ModeExampleKeys extends Mode {

  ////////////////////////////////////////////////////////////////
  // constructor

  public ModeExampleKeys(Editor par) { super(par); }

  ////////////////////////////////////////////////////////////////
  // accessors

  public boolean canExit() { return false; }

  ////////////////////////////////////////////////////////////////
  // key bindings

  public void bindKeys() {
    bindKey(9, new ActionSelectNext()); // should this be here or ModeSelect
    bindKey(9, Event.SHIFT_MASK, new ActionSelectNext());
    bindKey('g', new ActionGroup());
    bindKey('u', new ActionUngroup());

    bindKey('f', new ActionReorder(ActionReorder.BRING_FORWARD));
    bindKey('b', new ActionReorder(ActionReorder.SEND_BACKWARD));
    bindKey('F', new ActionReorder(ActionReorder.BRING_TO_FRONT));
    bindKey('B', new ActionReorder(ActionReorder.SEND_TO_BACK));

    //bindKey(8, new ActionDispose());
    //bindKey(8, Event.SHIFT_MASK, new ActionDispose());
    bindKey(127, new ActionDispose());
    bindKey(127, Event.SHIFT_MASK, new ActionDispose());

    bindKey('i', new ActionAlign(ActionAlign.ALIGN_TOPS));
    bindKey('j', new ActionAlign(ActionAlign.ALIGN_LEFTS));
    bindKey('k', new ActionAlign(ActionAlign.ALIGN_BOTTOMS));
    bindKey('l', new ActionAlign(ActionAlign.ALIGN_RIGHTS));

    bindKey(Event.LEFT, new ActionNudge(ActionNudge.LEFT));
    bindKey(Event.RIGHT, new ActionNudge(ActionNudge.RIGHT));
    bindKey(Event.UP, new ActionNudge(ActionNudge.UP));
    bindKey(Event.DOWN, new ActionNudge(ActionNudge.DOWN));

    bindKey(Event.LEFT, Event.SHIFT_MASK, new ActionNudge(ActionNudge.LEFT));
    bindKey(Event.RIGHT, Event.SHIFT_MASK, new ActionNudge(ActionNudge.RIGHT));
    bindKey(Event.UP, Event.SHIFT_MASK, new ActionNudge(ActionNudge.UP));
    bindKey(Event.DOWN, Event.SHIFT_MASK, new ActionNudge(ActionNudge.DOWN));

    bindCtrlKey(Event.LEFT, new ActionScroll(ActionScroll.LEFT));
    bindCtrlKey(Event.RIGHT, new ActionScroll(ActionScroll.RIGHT));
    bindCtrlKey(Event.UP, new ActionScroll(ActionScroll.UP));
    bindCtrlKey(Event.DOWN, new ActionScroll(ActionScroll.DOWN));
  }

} /* end class ModeExampleKeys */
