// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ModePlace.java
// Classes: ModePlace
// Original Author: jrobbins@ics.uci.edu
// $Id: ModePlace.java,v 1.13 1997/06/10 23:43:12 jrobbins Exp $

package uci.graphedit;

import java.awt.*;

/** Mode to place new a Perspective on a NetNode in a diagram.
 *  Normally invoked via ActionCreateNode.
 *
 * @see ActionCreateNode
 * @see Perspective
 */

public class ModePlace extends Mode {

  /** The (new) node being placed. It might be an existing node that
   *  is adding a new perspective. */
  private NetNode _node;

  /** The (new) perspective being placed. It might be an existing
   *  perspective on an existing node being place in another diagram. */
  private Perspective _pers;

  /** Construct a new instance of ModePlace and store the given NetNode. */
  ModePlace(NetNode n) {
    _node = n;
    _pers = null;
  }

  /** A string to be shown in the status bar of the Editor when this
   * mode is on top of the ModeStack. */
  public String instructions() {
    if (_node == null) return "";
    else return "Click to place " + _node.toString();
  }

  /** Move the perpective along with the mouse. */
  public boolean mouseEnter(Event evt, int x, int y) {
    start();
    _pers = _node.perspectiveFor(parent.view().activeLayer());
    return true;
  }

  /** Move the perpective along with the mouse. */
  public boolean mouseExit(Event evt, int x, int y) {
    parent.damaged(_pers);
    _pers = null;
    return true;
  }

  /** Move the perpective along with the mouse. */
  public boolean mouseMove(Event evt, int x, int y) {
    if (_pers == null) {System.out.println("null pers"); return true; }
    parent.damaged(_pers);
    Point snapPt = new Point(x, y);
    parent.snap(snapPt);
    _pers.position(snapPt.x, snapPt.y);
    parent.damaged(_pers); /* needed? */
    return true;
  }

  /** Eat this event and do nothing */
  public boolean mouseDown(Event evt, int x, int y) {
    return true;
  }

  /* Eactly the same as mouse move */
  public boolean mouseDrag(Event evt, int x, int y) {
    return mouseMove(evt, x, y);
  }

  /** Actually add the Perpective to the diagram.
   *  And give the NetNode a chance to do post processing.
   *
   * @see NetNode#postPlacement */
  public boolean mouseUp(Event evt, int x, int y) {
    parent.add(_pers);
    parent.selectItem(_pers);
    _node.postPlacement(parent);
    done();
    return true;
  }

  /** Draw the perspective being dragged around. */
  public void draw(Graphics g) { if (_pers != null) _pers.draw(g); }

} /* end class ModePlace */
