// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: Palette.java
// Classes: Palette
// Original Author: ics125b spring 1996
// $Id: Palette.java,v 1.9 1997/06/10 23:43:20 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.util.*;

/** Abstract class with shared behavior for windows that provide a
 *  palette of diagram tools and materials. Palettes may be displayed
 *  in their own Frame, or they may simply define a Panel that is used
 *  as part of a PaletteCompound.
 *  <A HREF="../features.html#palettes">
 *  <TT>FEATURE: palettes</TT></A>
 */

public abstract class Palette extends Panel  {

  /** A collection of ActiveComponents displayed in this window */
  private Vector _activeComps;

  /** The size of the grid of of _activeComps */
  private int _rows, _cols;

  /** Construct a new Palette */
  public Palette() { }

  /** Construct a new Palette with its components arranged in r rows
   * and c columns. */
  public Palette(int r, int c) { initialize(r, c); }

  /** Same as previous constructor, but set the title of the palette */
  public Palette(int r, int c, String title) {
    initialize(r, c);
  }

  /** Set up the data structures used by Palette's */
  protected void initialize(int r, int c) {
    _rows = r;
    _cols = c;
    setFont(new Font("Helvetica", Font.PLAIN, 10));
  }

  /** Abstract method to define the components to be shown in a
   * Palette. This should probably be called defineActiveComponents,
   * because Palette's can have more than just buttons.
   *
   * @see PaletteFig#defineButtons
   * @see PaletteAttr#defineButtons */
  public abstract Vector defineButtons();

  /** Layout the Palette components in a grid */
  public void definePanel() {
    _rows = Math.max(_rows, activeComps().size() / _cols);
    setLayout(new GridLayout(_rows,_cols, 0, 0));
    Enumeration cur = activeComps().elements();
    while (cur.hasMoreElements()) {
      Object obj =  cur.nextElement();
      ActiveComponent ac = (ActiveComponent) obj;
      add(ac.component());
    }
    setFont(new Font("Helvetica", Font.PLAIN, 10));
  }

  /** Get and set methods */
  protected Vector activeComps() {
    if (_activeComps == null) _activeComps = defineButtons();
    return _activeComps;
  }

  public void rows(int r) { _rows = r; }
  public void cols(int c) { _cols = c; }
  public int rows() { return _rows; }
  public int cols() { return _cols; }

} /* end class Palette */

