// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: CompoundPalette.java
// Classes: CompoundPalette
// Original Author: jrobbins@ics.uci.edu
// $Id: PaletteCompound.java,v 1.9 1997/06/10 23:43:21 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** This class implements a Palette that contains other, nested
 *  Palette's.  */

public class PaletteCompound extends Palette {

  /** A collection of Palette's contained in this PaletteCompound */
  private Vector _subpalettes;

  /** Construct a new PaletteCompound with the given subpalettes and title */
  public PaletteCompound(Vector subs, String title) {
    _subpalettes = subs;
    initialize(subs.size(), 1);
  }

  /** Same as above constructor, but use r rows and c columns */
  public PaletteCompound(Vector subs, int r, int c, String title) {
    _subpalettes = subs;
    initialize(r, c);
  }

  /** Construct a PaletteCompound from a colection of subpalettes */
  public PaletteCompound(Vector subs) {
    _subpalettes = subs;
    initialize(subs.size(), 1);
  }

  /** Defining the components of a PaletteCompound is accomplished by
   * accumulating all the components of its subpalettes. */
  public Vector defineButtons() {
    Vector buts = new Vector();
    Enumeration cur = _subpalettes.elements();
    while (cur.hasMoreElements()) {
      Vector palButs = ((Palette)cur.nextElement()).activeComps();
      if (palButs == null) System.out.println(this.toString());
      Enumeration curBut = palButs.elements();
      while (curBut.hasMoreElements()) {
	buts.addElement(curBut.nextElement());
      }
    }
    return buts;
  }

  /** The panel of a PaletteCompound is made up of a FlowLayout of all
   * the panels of its subpalettes */
  public void definePanel() {

    rows(Math.max(rows(), activeComps().size() / cols()));
    setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
    Enumeration cur = _subpalettes.elements();
    while (cur.hasMoreElements()) {
      Palette pal = (Palette) cur.nextElement();
      pal.definePanel();
      add(pal);
    }
  }

} /* end class PaletteCompound */
