// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: PaletteSticky.java
// Classes: PaletteSticky
// Original Author: jrobbins@ics.uci.edu
// $Id: PaletteSticky.java,v 1.8 1997/06/10 23:43:23 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.util.*;

/** class to provide "sticky" checkbox on palettes.
 *  <A HREF="../features.html#palettes_sticky_buttons">
 *  <TT>FEATURE: palettes_sticky_buttons</TT></A>
 */

public class PaletteSticky extends PaletteDecorator {

  /** A checkbox that can make some modes sticky */
  private Checkbox _stickyCheckbox;

  public PaletteSticky(Palette subpal) { super(subpal); }

  /** the most recent button pressed */
  private ActiveComponent _lastClicked;

  /** Make the currently stuck button appear to be stuck down. This
   * gives the user the impression that the button is still depressed
   * and that future mouse clicks on the drawing area will make new
   * instances of the same type.
   *  <A HREF="../bugs.html#win_no_button_background">
   *  <FONT COLOR=660000><B>BUG: win_no_button_background</B></FONT></A>
   */
  protected void highlightStuckButton() {
    Enumeration cur = activeComps().elements();
    while (cur.hasMoreElements()) {
      ActiveComponent ac = (ActiveComponent) cur.nextElement();
      ac.unhighlight();
    }
    if (_stickyCheckbox.getState() && _lastClicked != null)
      _lastClicked.highlight();
  }


  /** Handle action events by finding which component was activated
   *  and executnig it's associated action. <p>
   *
   *  Needs-More-Work: When using a sticky tool with the sticky
   *  checkbox set, each Editor that the mouse enters will go into
   *  that Mode. But pushing the 'select' button on the palette will
   *  only make the _current_ Editor go back to select mode, any
   *  other open Editor's that were touched will stay in the stuck
   *  mode.  */
  public boolean action(Event e, Object what) {
    boolean ret = false;
    Editor ce = Globals.curEditor();

    if (e.target == _stickyCheckbox) {
      _lastClicked = null;
      if (!_stickyCheckbox.getState()) Globals.mode(null);
    }

    Enumeration cur = activeComps().elements();
    while (cur.hasMoreElements()) {
      ActiveComponent ac = (ActiveComponent) cur.nextElement();
      if (e.target == ac.component()) {
	if (!ac.canStick()) _stickyCheckbox.setState(false);
	Globals.setSticky(_stickyCheckbox.getState());
	ac.doIt(e);
	if (_stickyCheckbox.getState()) _lastClicked = ac;
	ret = true;
      }
    }
    highlightStuckButton();
    if (!_stickyCheckbox.getState()) ce.finishMode();
    return ret || super.action(e, what);
  }

  public void definePanel() {
    super.definePanel();
    _stickyCheckbox = new Checkbox("Sticky");
    _stickyCheckbox.setState(false);
    add("South", _stickyCheckbox);
    /* maybe add a status pane at the bottom... no, use PaletteMessage...*/
  }

} /* end class PaletteSticky */

