// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: PaletteTop.java
// Classes: PaletteTop
// Original Author: jrobbins@ics.uci.edu
// $Id: PaletteTop.java,v 1.5 1997/06/10 23:43:24 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** */
public class PaletteTop extends PaletteDecorator  {

  protected Frame _frame = null;

  /** Construct a new Palette */
  public PaletteTop(Palette subpal) { super(subpal); }

  public void frame(Frame f) { _frame = f; }

  /** Handle action events by finding which component was activated
   * and executnig it's associated action.*/
  public boolean action(Event e, Object what) {
    boolean ret = false;

    Enumeration cur = activeComps().elements();
    while (cur.hasMoreElements()) {
      ActiveComponent ac = (ActiveComponent) cur.nextElement();
      if (e.target == ac.component()) {
	ac.doIt(e);
	// needs-more-work: should go through a bottle-neck method
	ret = true;
      }
    }
    return ret || super.action(e, what);
  }

  /** Handle events, I just need to catch window destroy events, the rest
     go through action(). */
  public boolean handleEvent(Event e) {
    switch (e.id) {
    case Event.WINDOW_DESTROY: close(); return true;
    }

    return super.handleEvent(e);
  }

  /** First hide the window, then dispose of its data structures. */
  public void close() {
    if (_frame != null) {
      _frame.hide();
      _frame.dispose();
    }
  }

} /* end class PaletteTop */

