// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: PrefsEditor.java
// Classes: PrefsEditor
// Original Author: jrobbins@ics.uci.edu
// $Id: PrefsEditor.java,v 1.12 1997/06/10 23:43:27 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;

/** A dialog box to let the user edit various preferences that effect
 *  the application.  Needs-More-Work: right now the only preferences
 *  that can be edited relate to redrawing. I should add prefs for
 *  handle colors, grid snap(?), and some
 *  behaviors... Needs-More-Work: This code could be deleted if I used
 *  a PropSheet to edit preferences instead.
 *
 * @see RedrawManager */

public class PrefsEditor extends Frame {

  /** A check box for using flicker-free graphics */
  private Checkbox _tryOffScreenCB;

  /** A check box for using straight or rectilinear arcs */
  private Checkbox _printGridCB;
  private Checkbox _printBackgroundCB;

  /** The "More Redraws" button.
   * @see RedrawManager#moreRepairs */
  private Button _moreRedraws;

  /** The "Fewer Redraws" button.
   * @see RedrawManager#fewerRepairs*/
  private Button _fewerRedraws;

  /** The close button */
  private Button _close;

  /** Construct a new PrefsEditor and get ready to open it. This can
   * be invoked via ActionOpenWindow.
   *
   * @see ActionOpenWindow */
  public PrefsEditor() {
    setTitle("Prefs");
    setBackground(Color.lightGray);
    _tryOffScreenCB = new Checkbox("Flicker-free redraw");
    _moreRedraws = new Button("More Redraws");
    _fewerRedraws = new Button("Fewer Redraws");
    _printGridCB = new Checkbox("Print Grid");
    _printBackgroundCB = new Checkbox("Print Background");
    Panel p = new Panel();
    p.setLayout(new GridLayout(6,1));
    _tryOffScreenCB.setState(Globals.prefs().tryOffScreen());
    _printGridCB.setState(Globals.prefs().printGrid());
    _printBackgroundCB.setState(Globals.prefs().printBackground());
    p.add(_tryOffScreenCB);
    p.add(_moreRedraws);
    p.add(_fewerRedraws);
    p.add(_printGridCB);
    p.add(_printBackgroundCB);
    add("Center", p);
    _close = new Button("Close");
    Panel bottomPanel = new Panel();
    bottomPanel.add("Center",_close);
    add("South", bottomPanel);
    pack();
  }

  /** Close the PrefEditor window */
  public void close() { dispose(); }

  /** If the user wants to close the window, do it. */
  public boolean handleEvent(Event e) {
    switch(e.id) {
    case Event.WINDOW_DESTROY:
      if (e.target == this) {
	close();
	return true;
      }
      break;
    }
    return super.handleEvent(e);
  }

  /** If the use pushes one of the buttons, update
   *  Globals.prefs(). Needs-More-Work: It would be nice to have a more
   *  compact way to code this. I expect that every preference that can
   *  be edited will affect Globals.prefs() in a similiar way. */
  public boolean action(Event e, Object what) {
    Prefs p = Globals.prefs();
    if (e.target == _tryOffScreenCB) {
      p.tryOffScreen(!p.tryOffScreen());
      _tryOffScreenCB.setState(p.tryOffScreen());
      return true;
    }
    if (e.target == _moreRedraws) {
      RedrawManager.moreRepairs();
      return true;
    }
    if (e.target == _fewerRedraws) {
      RedrawManager.fewerRepairs();
      return true;
    }
    else if (e.target == _close) {
      close();
      return true;
    }
    else if (e.target == _printGridCB) {
      p.printGrid(!p.printGrid());
      _printGridCB.setState(p.printGrid());
      return true;
    }
    else if (e.target == _printBackgroundCB) {
      p.printBackground(!p.printBackground());
      _printBackgroundCB.setState(p.printBackground());
      return true;
    }
    return false;
  }

} /* end class PrefsEditor */
