// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ArcData.java
// Classes: ArcData
// Original Author: jrobbins@ics.uci.edu
// $Id: ArcData.java,v 1.4 1997/06/10 23:40:26 jrobbins Exp $

package uci.graphedit.demo;

import uci.graphedit.*;
import java.util.*;
import uci.util.*;

/** A sample NetArc subclass for use in the Example application.  There
 *  are no real details here yet.  If I was to expand this Example more
 *  the arc could have more attributes, e.g. bandwidth... and it
 *  could have its own subclasses of ArcPerspective to make it look a
 *  certain way. */

public class ArcData extends NetArc {
  ////////////////////////////////////////////////////////////////
  // constants

  public static final String pCABLE_TYPE = "Cable Type";
  public static final String pBITS_PER_SECOND = "Bits/Second";

  public static final String CABLE_SCSI = "SCSI";
  public static final String CABLE_ETHERNET = "Ethernet";
  public static final String CABLE_FIREWIRE = "FireWire";
  public static final String CABLE_FDDI = "FDDI";

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** Speed of this data line */
  protected int _bitsPerSecond = 100000;

  /** Type of cable, choosen from the constants above: SCSI, FireWire,
   * etc. */
  protected String _cableType = CABLE_SCSI;


  ////////////////////////////////////////////////////////////////
  // constructors

  public ArcData() { } /* needs-more-work */

  ////////////////////////////////////////////////////////////////
  // accessors

  public String getCableType() { return _cableType; }
  public void setCableType(String ct) {
    if (_PossibleCableTypes.contains(ct))
      _cableType = ct;
  }

  public int getBitsPerSecond() { return _bitsPerSecond; }
  public void setBitsPerSecond(int bps) { _bitsPerSecond = bps; }

  public Object get(String key) {
    if (key.equals(pCABLE_TYPE))
      return getCableType();
    else if (key.equals(pBITS_PER_SECOND))
      return new Integer(getBitsPerSecond());
    else return super.get(key);
  }

  public boolean put(String key, Object value) {
    if (key.equals(pCABLE_TYPE))
      setCableType((String)value);
    else if (key.equals(pBITS_PER_SECOND))
      setBitsPerSecond(((Integer)value).intValue());
    else return super.put(key, value);
    return true;
  }

  public static Vector _ModelPropNames = null;

  public Enumeration keysIn(String category) {
    if (_ModelPropNames == null) {
      _ModelPropNames = new Vector();
      _ModelPropNames.addElement(pCABLE_TYPE);
      _ModelPropNames.addElement(pBITS_PER_SECOND);
    }
    Enumeration modelKeys = _ModelPropNames.elements();
    if (category.equals("Model"))
      return new EnumerationComposite(super.keysIn(category), modelKeys);
    else return super.keysIn(category);
  }

  public boolean canPut(String key) {
    return key.equals(pCABLE_TYPE);
    // note that pBITS_PER_SECONDis not editable
  }

  public static Vector _PossibleCableTypes = null;

  static {
    _PossibleCableTypes = new Vector();
    _PossibleCableTypes.addElement(CABLE_SCSI);
    _PossibleCableTypes.addElement(CABLE_ETHERNET);
    _PossibleCableTypes.addElement(CABLE_FIREWIRE);
    _PossibleCableTypes.addElement(CABLE_FDDI);
    uci.ui.PropSheetCategory.addEnumProp(pCABLE_TYPE, _PossibleCableTypes);
  }

  ////////////////////////////////////////////////////////////////
  // perspectives

  public ArcPerspective makePerspective(Layer lay) {
    return new ArcPerzRectiline();
  }

} /* end class ArcData */
