// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: EquipmentApplet.java
// Classes: EquipmentApplet
// Original Author: ics125b spring 1996
// $Id: EquipmentApplet.java,v 1.3 1997/06/10 23:40:28 jrobbins Exp $

package uci.graphedit.demo;

import java.applet.*;
import java.awt.*;
import java.util.*;
import uci.util.*;
import uci.graphedit.*;

/** An example application to show off <B>some</B> of the capabilities
 *  of the UCI Graph Editing Framework. This example is about
 *  connecting computers to printers with cable, and plugging those
 *  machines into the wall power.  It mainly shows off using Images,
 *  and defining a simple model with properties that can be edited in
 *  the property sheet.
 *
 * @see NodeCPU
 * @see NodePrinter
 * @see NodeWall
 * @see ArcPower
 * @see ArcData
 */

public class EquipmentApplet extends Applet {

  ////////////////////////////////////////////////////////////////
  // instance variables

  protected static boolean _spawnFrame = true;
  protected static int _drawAreaWidth = 400;
  protected static int _drawAreaHeight = 300;

  /** The Editor window */
  private Editor ed;

  /** The net-level model to edit */
  private NetList net;

  /** The example palette, shows Nodes for computer equipment */
  Palette palette;

  /** The palette of shapes and selection tool */
  Palette shapePalette;

  /** The overall palette window */
  Palette masterPalette;

  /** The overall palette window */
  PaletteTop topPalette;

  ////////////////////////////////////////////////////////////////
  // constructors

  /** Construct a new EquipmentApplet */
  public EquipmentApplet() {
    System.out.println("making an example");
    net = new NetList();
    net.name("Sample Network");

    palette = new EquipmentPalette();
    shapePalette = new PaletteFig();
  }

  ////////////////////////////////////////////////////////////////
  // main

  /** If this is run as an application, basically do the same thing as
   * the applet */
  public static void main(String[] args) {
    EquipmentApplet demo = new EquipmentApplet();
    demo.init();
  }

  ////////////////////////////////////////////////////////////////
  // argument and parameter methods

  public static void parseParams(Applet applet) {
//     String vendor = System.getProperty("java.vendor");
//     System.out.println("vendor='" + vendor + "'");
//     System.out.println("parsing params");
//     if ("true".equals(applet.getParameter("ShouldPreLoad")) &&
// 	!("Netscape Communications Corporation". equals(vendor) ||
// 	   "Sun Microsystems Inc.". equals(vendor))) {
//       Vector packs = new Vector();
//       packs.addElement("uci.graphedit.PackageGraphEdit");
//       System.out.println("preloading!!!!");
//       Preloader.loadPackages(packs);
//     }
    _drawAreaWidth = 400;
    _drawAreaHeight = 300;
  }

  public void setupWindows() {
    ForwardingPanel awt_comp;
    ed = new Editor(net);
    Globals.setStatusBar(ed);
    ed.show();
    //
    Vector pals = new Vector();
    pals.addElement(palette);
    pals.addElement(shapePalette);
    masterPalette = new PaletteSticky(new PaletteCompound(pals));
    topPalette = new PaletteTop(masterPalette);
    topPalette.definePanel();
    Frame palFrame = new Frame();
    palFrame.setLayout(new BorderLayout());
    palFrame.add("Center", topPalette);
    palFrame.move(400, 10);
    palFrame.pack();
    palFrame.show();
  }

  ////////////////////////////////////////////////////////////////
  // applet related methods

  /** Part of the required Applet API. Does nothing. */
  public void start() {  }

  /** This is called when the Applet/Application starts up. It does
   * preloading if the 'ShouldPreLoad' property is set to true. It
   * also registers some well known Action's as an example. */
  public void init() {
    Globals.setApplet(this);
    parseParams(this);
    setupWindows();
  }

  /** Part of the required Applet API. Does nothing. */
  public void stop() { }

  /** When the user closes this window try to free up as many objects
   * as possible. */
  public void destroy() {
    if (null != topPalette) topPalette.close();
    if (null != ed) ed.close();
    masterPalette = null;
    topPalette = null;
    ed = null;
    net = null;
    palette = null;
    shapePalette = null;
  }

  /** reply a breif string that describes this applet in the "About"
   * box. */
  public String getAppletInfo() {
    return "GEF (the Graph Editing Framework) example editor applet. \n" +
      "EquipmentApplet a very simple demonstration of how GEF can \n" +
      "be used. " + "\n\n" +
      "Author: Jason Robbins\n" +
      "Copyright (c) 1996-1998 Regents of the University of California.\n"+
      "All rights reserved.\n\n";
  }

} /* end class EquipmentApplet */
