// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: FlexibleApplet.java
// Classes: FlexibleApplet
// Original Author: ics125b spring 1996
// $Id: FlexibleApplet.java,v 1.18 1997/06/10 23:40:29 jrobbins Exp $

package uci.graphedit.demo;

import java.applet.*;
import java.awt.*;
import java.util.*;
import uci.util.*;
import uci.graphedit.*;

/** An example application to show off <B>some</B> of the capabilities
 *  of the UCI Graph Editing Framework. This example is a basic drawing
 *  editor that allows the user to place shapes in a drawing area and
 *  minipulate them. Also it uses LayerGrid. It has an Arrange menu
 *  that allows the user to so some manipulations, and an Attributes
 *  menu that allows the user to set fonts and colors. It registers
 *  some actions for execution from the ExecuteActionWindow. <p>
 *
 *  For another example see Argo. Argo (in its current form) is a
 *  Design Environment for C2 style software architectures. An Argo
 *  demo can be found
 *  <a href="http://www.ics.uci.edu/pub/arch/argo/">here</a>.<p>
 *
 *  Needs-More-Work: I need additional examples to show off the power
 *  of the framework and motivate its design. For example: a KISS
 *  viewer(?), a computer network editor, a class hierarchy editor (UML
 *  or OMT), etc.
 *
 * @see Editor
 * @see Palette
 * @see Action
 * @see Mode
 * @see Layer
 * @see DiagramElement
 * @see NetPrimitive
 * @see jargo.softarch.klax.Main
 */

public class FlexibleApplet extends Applet {

  ////////////////////////////////////////////////////////////////
  // instance variables

  protected static boolean _spawnFrame = true;
  protected static boolean _spawnPalette = true;
  protected static int _drawAreaWidth = 400;
  protected static int _drawAreaHeight = 300;
  protected static String _loadDocument;

  /** The Editor window */
  private Editor ed;

  /** The net-level model to edit */
  private NetList net;

  /** The example palette, shows SampleNode */
  Palette palette;

  /** The palette of shapes and selection tool */
  Palette shapePalette;

  /** The palette of attributes for shapes. Needed because applets
  /** cannot have menus when embedded in the browser page. */
  Palette attrPalette;

  /** The overall palette window */
  Palette masterPalette;

  /** The overall palette window */
  PaletteTop topPalette;

  ////////////////////////////////////////////////////////////////
  // constructor

  /** Construct a new FlexibleApplet */
  public FlexibleApplet() {
    System.out.println("making an example");
    net = new NetList();
    net.name("Sample Network");
    palette = new SamplePalette();
    shapePalette = new PaletteFig();
    attrPalette = new PaletteAttr();
  }


  ////////////////////////////////////////////////////////////////
  // main

  /** If this is run as an application, basically do the same thing as
   *  the applet */
  public static void main(String[] args) {
    FlexibleApplet demo = new FlexibleApplet();
    demo.init();
  }

  ////////////////////////////////////////////////////////////////
  // argument and parameter methods

  public static void parseParams(Applet applet) {
//     String vendor = System.getProperty("java.vendor");
//     System.out.println("vendor='" + vendor + "'");
//     System.out.println("parsing params");
//     if ("true".equals(applet.getParameter("ShouldPreLoad")) &&
// 	!("Netscape Communications Corporation". equals(vendor) ||
// 	   "Sun Microsystems Inc.". equals(vendor))) {
//       Vector packs = new Vector();
//       packs.addElement("uci.graphedit.PackageGraphEdit");
//       System.out.println("preloading!!!!");
//       Preloader.loadPackages(packs);
//     }
    _drawAreaWidth = 400;
    _drawAreaHeight = 300;
    _spawnFrame = "true".equals(applet.getParameter("SpawnFrame"));
    _spawnPalette = "true".equals(applet.getParameter("SpawnPalette"));
    String widthStr = applet.getParameter("DrawAreaWidth");
    if (widthStr != null) _drawAreaWidth = Integer.parseInt(widthStr);
    String heightStr = applet.getParameter("DrawAreaHeight");
    if (heightStr != null) _drawAreaHeight = Integer.parseInt(heightStr);
    _loadDocument = applet.getParameter("LoadDocument");
    System.out.println("SpawnFrame= " + _spawnFrame);
    System.out.println("SpawnPalette= " + _spawnPalette);
    System.out.println("DrawArea= " + _drawAreaWidth + " by " +
		       _drawAreaHeight);
    System.out.println("LoadDocument= " + _loadDocument);
  }

  public void setupWindows() {
    Component awt_comp;
    Dimension drawAreaSize = new Dimension(_drawAreaWidth, _drawAreaHeight);
    awt_comp = new ForwardingPanel(drawAreaSize);
    ed = new Editor(net, awt_comp);
    ((ForwardingComponent)awt_comp).setEventHandler(ed);
    if (_spawnFrame) {
      System.out.println("spawning frame");
      Frame drawingFrame = new ForwardingFrame(ed, drawAreaSize);
      ed.frame(drawingFrame);
      drawingFrame.add("Center", awt_comp);
      drawingFrame.pack();
      drawingFrame.move(10, 10);
      drawingFrame.show();
    }
    else add("Center", awt_comp);
    Vector pals = new Vector();
    pals.addElement(palette);
    pals.addElement(shapePalette);
    if (!_spawnPalette) pals.addElement(attrPalette);
    masterPalette = new PaletteSticky(new PaletteCompound(pals));
    topPalette = new PaletteTop(masterPalette);
    topPalette.definePanel();
    if (_spawnPalette) {
      System.out.println("spawning palette");
      Frame paletteFrame = new Frame();
      paletteFrame.setTitle("Palette");
      topPalette.frame(paletteFrame);
      paletteFrame.add("Center", topPalette);
      paletteFrame.pack();
      paletteFrame.move(10 + drawAreaSize.width, 10);
      paletteFrame.show();
    }
    else add("East", topPalette);
  }

  ////////////////////////////////////////////////////////////////
  // applet related methods

  /** Part of the required Applet API. Does nothing. */
  public void start() {  }

  /** This is called when the Applet/Application starts up. It does
   *  preloading if the 'ShouldPreLoad' property is set to true. It
   *  also registers some well known Action's as an example. */
  public void init() {
    Globals.setApplet(this);
    parseParams(this);
    setupWindows();
    Action.register(new ActionSave());
    Action.register(new ActionLoad());
    Action.register(new ActionDispose());
    Action.register(new ActionDelete());
    Action.register(new ActionSpawn());
    Action.register(new ActionAdjustGrid());
    Action.register(new ActionEditNode());
    Action.register(new ActionCreateNode("uci.graphedit.demo.SampleNode",
					 null, false));
    Action.register(new ActionQuit());
  }

  /** Part of the required Applet API. Does nothing. */
  public void stop() { }

  /** When the user closes this window try to free up as many objects
   *  as possible. */
  public void destroy() {
    if (null != topPalette) topPalette.close();
    if (null != ed) ed.close();
    masterPalette = null;
    topPalette = null;
    ed = null;
    net = null;
    palette = null;
    shapePalette = null;
    attrPalette = null;
  }

  /** Reply a breif string that describes this applet in the "About"
   *  box. */
  public String getAppletInfo() {
    return "GEF (the Graph Editing Framework) example editor applet. \n" +
      "FlexibleApplet a very simple demonstration of how GEF can \n" +
      "be used. " + "\n\n" +
      "Author: Jason Robbins\n" +
      "Copyright (c) 1996-1998 Regents of the University of California.\n"+
      "All rights reserved.\n\n";
  }

  /** Reply an array of strings that describe the HTML PARAM's that
   *  can be passed to this applet. */
  public String[][] getParameterInfo() {
    String[][] info = new String[5][3];
    //
    info[0][0] = "LoadDocument";
    info[0][1] = "java.lang.String\n";
    info[0][2] = "If supplied, automatically load the given URL.\n\n";
    //
    info[1][0] = "SpawnFrame";
    info[1][1] = "boolean\n";
    info[1][2] = "Open a new Frame for the drawing area.\n\n";
    //
    info[2][0] = "SpawnPalette";
    info[2][1] = "boolean\n";
    info[2][2] = "Open a new Frame for the palette.\n\n";
    //
    info[3][0] = "DrawAreaWidth";
    info[3][1] = "int\n";
    info[3][2] = "Width of the drawing area in pixels.\n\n";
    //
    info[4][0] = "DrawAreaHeight";
    info[4][1] = "int\n";
    info[4][2] = "Height of the drawing area in pixels.\n\n";
    //
    return info;
  }

} /* end class FlexibleApplet */
