// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: SampleNode.java
// Classes: SampleNode
// Original Author: ics125b spring 1996
// $Id: SampleNode.java,v 1.13 1997/06/10 23:40:34 jrobbins Exp $

package uci.graphedit.demo;

import java.awt.*;
import java.io.*;
import uci.graphedit.*;

/** An example subclass of NetNode for use in the Example application.
 *
 * @see Example */
public class SampleNode extends NetNode {

   /** Initialize a new SampleNode from the given default node and
    *  application specific model. <p>
    *
    *  Needs-More-Work: for now we construct the Perspective
    *  programatically, but eventually we will store it in a class
    *  variable and just refer to it, or copy it(?). That way the user
    *  can edit the Perspective(s) stored in the class variable and
    *  have those changes shown for all existing nodes, or for all
    *  future nodes. Maybe I should think about doing virtual copies?<p>
    */

  public void initialize(NetNode deft, Object model) {
      portList = new NetPort[4];
      portList[0] = new SamplePort(this);
      portList[1] = new SamplePort(this);
      portList[2] = new SamplePort2(this);
      portList[3] = new SamplePort2(this);
   }

  public Perspective makePerspective(Layer lay) {
    Fig obj1 = new FigRect(-25, -25, 50, 50, Color.black, Color.white);
    Fig obj2 = new FigCircle(-20, -20, 40, 40, Color.red);
    Fig obj3 = new FigCircle( -5, -30, 10, 10, Color.black, Color.blue);
    Fig obj4 = new FigCircle( -5,  20, 10, 10, Color.black, Color.blue);
    Fig obj5 = new FigRect(-30,  -5, 10, 10, Color.black, Color.green);
    Fig obj6 = new FigRect( 20,  -5, 10, 10, Color.black, Color.green);
    FigList temp_list = new FigList();
    temp_list.addFig(obj1);
    temp_list.addFig(obj2);
    temp_list.addFig(obj3);
    temp_list.addFig(obj4);
    temp_list.addFig(obj5);
    temp_list.addFig(obj6);
    Perspective pers1 = new Perspective(this, temp_list);
    pers1.addPort(portList[0], obj3);
    pers1.addPort(portList[1], obj4);
    pers1.addPort(portList[2], obj5);
    pers1.addPort(portList[3], obj6);
    return pers1;
  }

  /** Sample event handler: prints a message to the console. */
  public boolean mouseEnter(Event e, int x, int y) {
    //    System.out.println("sample node got mouseEnter");
    return super.mouseEnter(e, x, y);
  }

  /** Sample event handler: prints a message to the console. */
  public boolean mouseExit(Event e, int x, int y) {
    //    System.out.println("sample node got mouseExit");
    return super.mouseExit(e, x, y);
  }

  /** Sample event handler: prints a message to the console. */
  public boolean mouseUp(Event e, int x, int y) {
    //    System.out.println("sample node got mouseUp");
    return super.mouseUp(e, x, y);
  }

  /** Sample event handler: prints a message to the console. */
  public boolean mouseDown(Event e, int x, int y) {
    //    System.out.println("sample node got mouseDown");
    return super.mouseDown(e, x, y);
  }

  /** Sample event handler: prints a message to the console. */
  public boolean mouseDrag(Event e, int x, int y) {
    //    System.out.println("sample node got mouseDrag");
    return super.mouseDrag(e, x, y);
  }

  /** Sample event handler: prints a message to the console. */
  public boolean mouseMove(Event e, int x, int y) {
    //    System.out.println("sample node got mouseMove");
    return super.mouseMove(e, x, y);
  }

  /** Sample event handler: prints a message to the console. */
  public boolean keyUp(Event e, int key) {
    //    System.out.println("sample node got keyUp");
    return super.keyUp(e, key);
  }

  /** Sample event handler: prints a message to the console. */
  public boolean keyDown(Event e, int key) {
    //    System.out.println("sample node got keyDown");
    return super.keyDown(e, key);
  }

  /** Sample event handler: prints a message to the console. */
  public boolean handleMenuEvent(Event e) {
    //    System.out.println("sample node got handleMenuEvent");
    return super.handleMenuEvent(e);
  }

} /* end class SampleNode */
