// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.


package uci.graphedit.demo;
//WR changed

import java.applet.*;
import java.awt.*;
import java.util.*;
import java.net.*;

import uci.util.*;
import uci.graphedit.*;
import uci.graphedit.demo.SamplePalette;


public class ScrollingApplet extends Applet {

  ////////////////////////////////////////////////////////////////
  // instance variables

  protected static boolean _spawnFrame = true;
  protected static boolean _spawnPalette = true;
  protected static int _drawAreaWidth = 500;
  protected static int _drawAreaHeight = 500;
  protected static String _loadDocument;


  /** The Editor window */
  private Editor ed;

  /** The net-level model to edit */
  private NetList net;

  /** The example palette, shows SampleNode */
  Palette palette;

  /** The palette of shapes and selection tool */
  Palette shapePalette;

  /** The palette of attributes for shapes. Needed because applets
  /** cannot have menus when embedded in the browser page. */
  Palette attrPalette;

  /** The overall palette window */
  Palette masterPalette;

  /** The overall palette window */
  PaletteTop topPalette;

  ////////////////////////////////////////////////////////////////
  // constructor

  /** Construct a new ScrollingApplet */
  public ScrollingApplet() {
    System.out.println("Starting ScrollingApplet Version 1.1 ");
    net = new NetList();
    net.name("Sample Network");
    palette = new SamplePalette();

    shapePalette = new PaletteFig();
    attrPalette = new PaletteAttr();
  }


  ////////////////////////////////////////////////////////////////
  // main

  /** If this is run as an application, basically do the same thing as
   *  the applet */
  public static void main(String[] args) {

    ScrollingApplet demo = new ScrollingApplet();
    // WR  ---
    demo.init();
    demo.start();
  }

  ////////////////////////////////////////////////////////////////
  // argument and parameter methods

  public static void parseParams(Applet applet) {
    _drawAreaWidth = 500;
    _drawAreaHeight = 500;
  }


  public void setupWindows() {
    Component awt_comp;
    ScrollPane _scPane;

    // WR: added Scrollbars to Editor !!!
    Dimension drawAreaSize = new Dimension(_drawAreaWidth, _drawAreaHeight);

    // use the new class ForwardingCanvas
    awt_comp = new ForwardingPanel(new Dimension(800,1000) );
    awt_comp.setSize(800,1000);

    _scPane = new ScrollPane(ScrollPane.SCROLLBARS_ALWAYS);
    _scPane.add(awt_comp);
    _scPane.setSize(400,400);

    ed = new Editor(net, awt_comp);
    ((ForwardingComponent)awt_comp).setEventHandler(ed);

    if (_spawnFrame) {
      System.out.println("spawning editor frame");

      Frame drawingFrame = new ForwardingFrame(ed, new Dimension(400,400));

      drawingFrame.setLayout(new BorderLayout());
      ed.frame(drawingFrame);
      drawingFrame.add("Center", _scPane);

      drawingFrame.pack();
      drawingFrame.move(10, 10);
      drawingFrame.show();
      drawingFrame.setSize(400,400);
      drawingFrame.repaint();

    }
    else add("Center", awt_comp);

    Vector pals = new Vector();
    pals.addElement(palette);
    pals.addElement(shapePalette);
    if (!_spawnPalette) pals.addElement(attrPalette);
    masterPalette = new PaletteSticky(new PaletteCompound(pals));
    topPalette = new PaletteTop(masterPalette);
    topPalette.definePanel();
    if (_spawnPalette) {
      Frame paletteFrame = new Frame();
      paletteFrame.setTitle("Palette");
      topPalette.frame(paletteFrame);
      paletteFrame.add("Center", topPalette);
      paletteFrame.pack();
      paletteFrame.move(10 + drawAreaSize.width, 10);
      paletteFrame.show();
    }
    else add("East", topPalette);

  }

  ////////////////////////////////////////////////////////////////
  // applet related methods

  /** Part of the required Applet API. Does nothing. */
  public void start() {  }

  /** This is called when the Applet/Application starts up. It does
   *  preloading if the 'ShouldPreLoad' property is set to true. It
   *  also registers some well known Action's as an example. */
  public void init() {

    // add new parameter to determine betwen applet and application WR
    Globals.setApplet(this);

    parseParams(this);
    setupWindows();

    Action.register(new ActionSave());
    Action.register(new ActionLoad());
    Action.register(new ActionDispose());
    Action.register(new ActionDelete());

    //Action.register(new ActionSpawn());
    //Action.register(new ActionAdjustGrid());
    Action.register(new ActionEditNode());
    Action.register(new ActionCreateNode("uci.graphedit.demo.SampleNode",
                                         null, false));
    Action.register(new ActionQuit());
  }

  /** Part of the required Applet API. Does nothing. */
  public void stop() { }

  /** When the user closes this window try to free up as many objects
   *  as possible. */
  public void destroy() {
    if (null != topPalette) topPalette.close();
    if (null != ed) ed.close();
    masterPalette = null;
    topPalette = null;
    ed = null;
    net = null;
    palette = null;
    shapePalette = null;
    attrPalette = null;
  }

  /** Reply a breif string that describes this applet in the "About"
   *  box. */
  public String getAppletInfo() {
    return "ScrollingApplet by using the GEF Framework. \n" +
      "Author of ScrollingApplet: Walter Rott\n" +
      "Copyright (c) 1998, ECT \n"+
      "Author of GEF: Jason Robbins\n" +
      "Copyright (c) 1996-1998 Regents of the University of California.\n"+
      "All rights reserved.\n\n";
  }

  /** Reply an array of strings that describe the HTML PARAM's that
   *  can be passed to this applet. */
  public String[][] getParameterInfo() {
    String[][] info = new String[5][3];
    //
    info[0][0] = "LoadDocument";
    info[0][1] = "java.lang.String\n";
    info[0][2] = "If supplied, automatically load the given URL.\n\n";
    //
    info[1][0] = "SpawnFrame";
    info[1][1] = "boolean\n";
    info[1][2] = "Open a new Frame for the drawing area.\n\n";
    //
    info[2][0] = "SpawnPalette";
    info[2][1] = "boolean\n";
    info[2][2] = "Open a new Frame for the palette.\n\n";
    //
    info[3][0] = "DrawAreaWidth";
    info[3][1] = "int\n";
    info[3][2] = "Width of the drawing area in pixels.\n\n";
    //
    info[4][0] = "DrawAreaHeight";
    info[4][1] = "int\n";
    info[4][2] = "Height of the drawing area in pixels.\n\n";
    //
    return info;
  }

} /* end class ScrollingApplet */
