// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: ColorPickerGrid.java
// Classes: ColorPickerGrid
// Original Author: jrobbins@ics.uci.edu
// $Id: ColorPickerGrid.java,v 1.4 1997/06/10 23:38:45 jrobbins Exp $

package uci.ui;

import java.awt.*;

/** A small window that allows the user to choose a color. This window
 *  is slave to a PropEditorColor (a small colored tile that appears in
 *  a PropSheet). Whenever the user choose a color, this object
 *  notifies its master PropEditorColor. */

public class ColorPickerGrid extends Frame {

  ////////////////////////////////////////////////////////////////
  // instance variables

  /** An array printf tiny colored squares. */
  protected ColorTilePanel _tiles = new ColorTilePanel(36);
  /** The PropEditorColor that is master to this slave window. */
  protected PropEditorColor _peColor = null;
  /** PATTERN: singleton */
  protected static ColorPickerGrid _theInstance = null;
  /** Text to be displayed in the status bar area of this window. */
  protected Label _statusLabel = new Label("No color selected");

  ////////////////////////////////////////////////////////////////
  // constructors

  public ColorPickerGrid(Color orig) {
    super();
    setLayout(new BorderLayout());
    _statusLabel.setFont(new Font("Courier", Font.PLAIN, 10));
    setBackground(new Color(12632256));
    add("Center", _tiles);
    add("South", _statusLabel);
    _tiles.setColor(orig);
    pack();
    setTitle("Color Picker");
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  public void setPEColor(PropEditorColor pec) {
    _peColor = pec;
    if (_peColor != null) {
      _tiles.allowSelection(true);
      setColor(_peColor.getColor());
    }
    else {
      _tiles.allowSelection(false);
      updateStatusLabel();
    }
  }

  public void setColor(Color c) {
    _tiles.setColor(c);
    updateStatusLabel();
  }

  ////////////////////////////////////////////////////////////////
  // event handlers

  public boolean handleEvent(Event event) {
    if (event.id == Event.WINDOW_DESTROY) {
      hide();         // hide the Frame
      return true;
    }
    return super.handleEvent(event);
  }

  public boolean action(Event e, Object what) {
    updateStatusLabel();
    if (_peColor != null) _peColor.colorPicked(_tiles.getColor());
    return true;
  }

  public void updateStatusLabel() {
    if (_peColor == null) {
      _statusLabel.setText("No color selected");
      return;
    }
    Color c = _tiles.getColor();
    String r = Integer.toString(c.getRed(), 16).toUpperCase();
    String b = Integer.toString(c.getBlue(), 16).toUpperCase();
    String g = Integer.toString(c.getGreen(), 16).toUpperCase();
    if (c.getRed() == 0) r = "00";
    if (c.getBlue() == 0) b = "00";
    if (c.getGreen() == 0) g = "00";
    _statusLabel.setText("Red:" + r + " Blue:" + b + " Green:" + g);
  }

  /** Open the singleton instance a a slave to the given
   *  PropEditorColor, at the screen coordinates given.
   *  <A HREF="../bugs.html#in_color_picker_buried">
   *  <FONT COLOR=660000><B>BUG: in_color_picker_buried</B></FONT></A>
   */
  public static void edit(PropEditorColor pec, int x, int y) {
    if (_theInstance == null) {
      _theInstance = new ColorPickerGrid(Color.white);
      _theInstance.move(x, y);
      _theInstance.show();
    }
    else {
      _theInstance.show();
      _theInstance.toFront();
    }
    _theInstance.setPEColor(pec);
  }

  /** The user has done something to hide my master PropEditorColor,
   * so don't allow any more changes. */
  public static void stopEditing() {
    if (_theInstance == null) return;
    _theInstance.setPEColor(null);
  }

  public static void stopIfEditing(PropEditorColor pec) {
    if (_theInstance == null) return;
    if (_theInstance._peColor != pec) return;
    _theInstance.setPEColor(null);
  }

  /** If the window is shown and is slave to the given
   * PropEditorColor, set its selected color to that of the given
   * PropEditorColor. */
  public static void updateIfEditing(PropEditorColor pec) {
    if (_theInstance == null) return;
    if (_theInstance._peColor != pec) return;
    _theInstance.setColor(pec.getColor());
  }

} /* end class ColorPickerGrid */
