// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

package uci.ui;

import java.awt.*;
import java.util.*;
import uci.graphedit.DiagramElement;

/** <A HREF="../features.html#property_sheet_editors">
 *  <TT>FEATURE: property_sheet_editors</TT></A>
 */

public class PropEditor extends Panel implements Observer {
    ////////////////////////////////////////////////////////////////
    // instance variables

    public IProps _sel = null;
    public Hashtable _pendingStores = new Hashtable();
    public boolean _autoApply = true;
    public String _tabName = "Misc";

    ////////////////////////////////////////////////////////////////
    // constructors

    public PropEditor() { }

    ////////////////////////////////////////////////////////////////
    // accessors

    public void setSelection(IProps s) {
      if (_sel == s) return;
      if (_sel instanceof Observable)
          ((Observable)_sel).deleteObserver(this);
      _sel = s;
      if (_sel instanceof Observable)
          ((Observable)_sel).addObserver(this);
      updateComponents();
    }

    public void setAutoApply(boolean aa) {
      _autoApply = aa;
      if (_autoApply) apply();
    }

    public String getTabName() { return _tabName; }
    public void setTabName(String tn) { _tabName = tn; }

    public boolean canEdit(IProps item) { return true; }

    ////////////////////////////////////////////////////////////////
    // storing properties

    public void store(String key, Object value) {
      if (key == null || value == null) return;
      _pendingStores.put(key, value);
      if (_autoApply) apply();
    }

    public void apply() {
      if (_sel instanceof DiagramElement)
        ((DiagramElement)_sel).startTrans();
      if (_sel != null) _sel.put(_pendingStores);
      if (_sel instanceof DiagramElement)
        ((DiagramElement)_sel).endTrans();
      _pendingStores.clear();
    }

    public void revert() {
      _pendingStores.clear();
      updateComponents();
    }

    ////////////////////////////////////////////////////////////////
    // notifications and updates

    public void updateComponents() { }

    public void updateComponent(String key) { updateComponents(); }

    public void update(Observable obs, Object arg) {
        if (obs == _sel && arg instanceof String)
          updateComponent((String)arg);
    }

} /* end class PropEditor */
