// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: PropEditorColor.java
// Interfaces: PropEditorColor
// Original Author: jrobbins@ics.uci.edu
// $Id: PropEditorColor.java,v 1.2 1997/06/10 23:39:02 jrobbins Exp $

package uci.ui;

import java.awt.*;

/** <A HREF="../features.html#color_picker">
 *  <TT>FEATURE: color_picker</TT></A>
 */

public class PropEditorColor extends Panel {
  ////////////////////////////////////////////////////////////////
  // instance variables
  public Color _color = null;

  public PropEditorColor(Color c) {
    //{{INIT_CONTROLS
    setLayout(null);
    addNotify();
    resize(insets().left + insets().right + 20,
	   insets().top + insets().bottom + 20);
    //}}
    _color = c;
    setForeground(Color.lightGray);
  }

  public PropEditorColor() { this(Color.white); }

  public void addNotify() {
    if (_color != null) super.addNotify();
  }

  ////////////////////////////////////////////////////////////////
  // accessors

  public void setColor(Color c) {
    if (c == null) return;
    _color = c;
    repaint();
    ColorPickerGrid.updateIfEditing(this);
  }

  public Color getColor() { return _color; }


  ////////////////////////////////////////////////////////////////
  // drawing methods
  public void paint(Graphics g) {
    Dimension size = size();
    if (isEnabled()) {
      g.setColor(getForeground());
      g.draw3DRect(0, 0, size.width - 1, size.height - 1, true);
      g.setColor(_color);
    }
    else g.setColor(getBackground());
    g.fill3DRect(1, 1, size.width - 2, size.height - 2, true);
  }

  ////////////////////////////////////////////////////////////////
  // event handlers

  public boolean handleEvent(Event event) {
    if (event.id == Event.MOUSE_DOWN) {
      int gx = event.x;
      int gy = event.y;
      Component c = this;
      while (c instanceof Component) {
	Point loc = c.location();
	gx += loc.x;
	gy += loc.y;
	c = c.getParent();
      }
      ColorPickerGrid.edit(this, gx, gy);
      return true;
    }
    return super.handleEvent(event);
  }

  public void colorPicked(Color newColor) {
    if (newColor == null) return;
    setColor(newColor);
    postEvent(new Event(this, Event.ACTION_EVENT, newColor));
  }

  public Frame getFrame() {
    Component c = this;
    while (c != null && !(c instanceof Frame)) {
      c = c.getParent();
    }
    return (Frame) c;
  }

  public void hide() {
    super.hide();
    ColorPickerGrid.stopIfEditing(this);
  }
  
  //{{DECLARE_CONTROLS
  //}}
} /* end class PropEditorColor */

