// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: PropEditorRect.java
// Interfaces: PropEditorRect
// Original Author: jrobbins@ics.uci.edu
// $Id: PropEditorRect.java,v 1.3 1997/06/10 23:39:07 jrobbins Exp $

package uci.ui;

import java.awt.*;

/** A widget that allows the user to edit rectangles in the property
 *  sheet window. Needs-More-Work: Currently only displays them without
 *  allowing editing.
 *  <A HREF="../features.html#rectangle_editor">
 *  <TT>FEATURE: rectangle_editor</TT></A>
 */

public class PropEditorRect extends Panel {
  ////////////////////////////////////////////////////////////////
  // instance variables
  public Rectangle _rect = null;

  public PropEditorRect(Rectangle r) {
    //{{INIT_CONTROLS
    setLayout(new GridLayout(1, 4));
    addNotify();
    resize(insets().left + insets().right + 20,
	   insets().top + insets().bottom + 20);
    xField = new TextField("0", 4);
    yField = new TextField("0", 4);
    hField = new TextField("0", 4);
    wField = new TextField("0", 4);
    add(xField);
    add(yField);
    add(hField);
    add(wField);
    //}}
    _rect = r;
  }

  public PropEditorRect() { this(new Rectangle(0, 0, 100, 100)); }

  public void addNotify() {
    if (_rect != null) super.addNotify();
  }

  public void enable() {
    xField.setEditable(true);
    yField.setEditable(true);
    wField.setEditable(true);
    hField.setEditable(true);
  }
    
  public void disable() {
    xField.setEditable(false);
    yField.setEditable(false);
    wField.setEditable(false);
    hField.setEditable(false);
  }
  
  ////////////////////////////////////////////////////////////////
  // accessors

  public void setRectangle(Rectangle r) {
    if (r == null) return;
    _rect = r;
    setFields();
  }

  public Rectangle getRectangle() { return _rect; }

  protected void setFields() {
    xField.setText(Integer.toString(_rect.x));
    yField.setText(Integer.toString(_rect.y));
    wField.setText(Integer.toString(_rect.width));
    hField.setText(Integer.toString(_rect.height));
  }

  protected void readFields() {
    try {
      _rect.x = Integer.parseInt(xField.getText());
      _rect.y = Integer.parseInt(yField.getText());
      _rect.width = Integer.parseInt(wField.getText());
      _rect.height = Integer.parseInt(hField.getText());
    }
    catch (java.lang.NumberFormatException ignore) {
      setFields();
    }
  }


  ////////////////////////////////////////////////////////////////
  // event handlers

  public boolean handleEvent(Event event) {
    if (event.target != this && event.id == Event.ACTION_EVENT) {
      readFields();
      postEvent(new Event(this, Event.ACTION_EVENT, _rect));
      return true;
    }
    return super.handleEvent(event);
  }


  //{{DECLARE_CONTROLS
  TextField xField;
  TextField yField;
  TextField wField;
  TextField hField;
  //}}
} /* end class PropEditorRect */

