// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: PropSheet.java
// Interfaces: PropSheet
// Original Author: jrobbins@ics.uci.edu
// $Id: PropSheet.java,v 1.6 1997/06/10 23:39:10 jrobbins Exp $

package uci.ui;

import java.awt.*;
import java.util.*;
import uci.graphedit.DiagramElement;

/** Class that defines the interface for several different kinds of
 *  panels that can edit a group of properties. The only subclass
 *  supplied with GEF is PropSheetCategory, but others could be
 *  defined. */

public class PropSheet extends Panel implements Observer {
  ////////////////////////////////////////////////////////////////
  // instance variables

  /** The object being edited. */
  public IProps _sel = null;

  /** The changes to that object that have not been carried out yet. */
  public Hashtable _pendingStores = new Hashtable();

  /** True iff every change shold be immeadiatly carried out. */
  public boolean _autoApply = true;

  /** Name used to identify this sheet to the user. */
  public String _tabName = "Misc";

  protected Font _propertiesFont = new Font("Dialog", Font.PLAIN, 10);

  ////////////////////////////////////////////////////////////////
  // constructors

  public PropSheet() { }

  ////////////////////////////////////////////////////////////////
  // accessors

  public void setSelection(IProps s) {
    if (_sel == s) return;
    if (_sel instanceof Observable)
      ((Observable)_sel).deleteObserver(this);
    _sel = s;
    if (_sel instanceof Observable)
      ((Observable)_sel).addObserver(this);
    updateComponents();
  }

  public void setAutoApply(boolean aa) {
    _autoApply = aa;
    if (_autoApply) apply();
  }

  public String getTabName() { return _tabName; }
  public void setTabName(String tn) { _tabName = tn; }

  public boolean canEdit(IProps item) { return true; }

  public void hide() {
    super.hide();
    setSelection(null);
  }

  public Font getPropertiesFont() { return _propertiesFont; }
  public void setPropertiesFont(Font f) {  _propertiesFont = f; }

  ////////////////////////////////////////////////////////////////
  // storing properties

  /** When the user changes a value in a widget, record that fact
   * until the next apply (which may be done immeadiatly). */
  public void store(String key, Object value) {
    if (key == null || value == null) return;
    _pendingStores.put(key, value);
    if (_autoApply) apply();
  }

  /** Take all property changes that have been stored and actually
   * change the state of the selected object. */
  public void apply() {
    if (_sel instanceof DiagramElement)
      ((DiagramElement)_sel).startTrans();
    if (_sel != null) _sel.put(_pendingStores);
    if (_sel instanceof DiagramElement)
      ((DiagramElement)_sel).endTrans();
    _pendingStores.clear();
  }

  /** Abandon any stored changes that have not been applied yet. */
  public void revert() {
    _pendingStores.clear();
    updateComponents();
  }


  ////////////////////////////////////////////////////////////////
  // notifications and updates

  public void updateComponents() { }

  public void updateComponent(String key) { updateComponents(); }

  /** The selected object may have changed one of its properties. */
  public void update(Observable obs, Object arg) {
    if (obs == _sel && arg instanceof String)
      updateComponent((String)arg);
  }

} /* end class PropSheet */
