// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: PropsGeneric.java
// Interfaces: PropsGeneric
// Original Author: jrobbins@ics.uci.edu
// $Id: PropsGeneric.java,v 1.3 1997/06/10 23:39:21 jrobbins Exp $

package uci.ui;
import java.util.*;

/** This class shows an example of how to implement interface
 *  IProps. Needs-More-Work: this class is not currently used in GEF,
 *  instead some classes basically duplicate this
 *  functionality. Needs-More-Work: it would be nice to implement a
 *  fly-weight pattern. */

public class PropsGeneric extends Observable implements IProps {
  ////////////////////////////////////////////////////////////////
  // instance variables
  private Hashtable _ht = new Hashtable();

  ////////////////////////////////////////////////////////////////
  // IProps API

  /** Get an property by name. */
  public Object get(String key) {
    return _ht.get(key);
  }
  public final Object get(String key, Object def) {
    Object res = get(key);
    if (null != res) return res;
    return def;
  }


  /** Set the value of a property. Null keys and values are ignored. */
  public boolean put(String key, Object value) {
    if (key != null && value != null) {
      _ht.put(key, value);
      changedProp(key);
      return true;
    }
    return false;
  }

  public void put(Hashtable props) {
    Enumeration keys = props.keys();
    while (keys.hasMoreElements()) {
      String key = (String) keys.nextElement();
      Object value = props.get(key);
      put(key, value);
    }
  }

  /** Return an Enumeration of Strings for property
   * names that are defined on this Props. */
  public Enumeration keysIn(String category) {
    return _ht.keys();
  }

  public boolean canPut(String key) { return true; }

  public boolean define(String key, Object value) {
    if (null == get(key)) {
      put(key, value);
      return true;
    }
    return false;
  }

  ////////////////////////////////////////////////////////////////
  // notifications and updates

  /** Notify observers that one of my properties has changed. */
  public void changedProp(String key) {
    setChanged();
    notifyObservers(key);
  }


} /* end class PropsGeneric */
