// Copyright (c) 1996-98 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation for educational, research and non-profit
// purposes, without fee, and without a written agreement is hereby granted,
// provided that the above copyright notice and this paragraph appear in all
// copies. Permission to incorporate this software into commercial products may
// be obtained by contacting the University of California. David F. Redmiles
// Department of Information and Computer Science (ICS) University of
// California Irvine, California 92697-3425 Phone: 714-824-3823. This software
// program and documentation are copyrighted by The Regents of the University
// of California. The software program and documentation are supplied "as is",
// without any accompanying services from The Regents. The Regents do not
// warrant that the operation of the program will be uninterrupted or
// error-free. The end-user understands that the program was developed for
// research purposes and is advised not to rely exclusively on the program for
// any reason. IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
// PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
// DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
// DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
// ENHANCEMENTS, OR MODIFICATIONS.

// File: EventQueue.java
// Class: EventQueue
// Original Author: jrobbins@ics.uci.edu
// $id$

package uci.util;

import java.util.*;
import java.awt.Event;

/** A <I>very</I> simple Queue implementation. The point of using this
 *  is to allow event skipping in uci.graphedit.Editor.
 *  Needs-More-Work: this might be eliminated when I switch to 100%
 *  Java 1.1. */

public class EventQueue {
  private Vector _elements = new Vector();

  /** Construct an empty event queue. */
  public EventQueue() { }

  /** Add an event to the queue. */
  public void enqueue(Event obj) {
    _elements.addElement(obj);
  }

  /* Take the first event off the queue. */
  public  Event dequeue() {
    Event e = (Event)_elements.elementAt(0);
    _elements.removeElementAt(0);
    return e;
  }

  /** See what is at the head of the queue. */
  public  Event peek() { return (Event)_elements.elementAt(0); }

  /** Return true if leading two events have the same event id. */
  public synchronized boolean nextIsSameAs(Event curEvent) {
    if (isEmpty()) return false;
    return curEvent.id == peek().id;
  }

  /** Reply the number of events in the queue now. */
  public  int size() { return _elements.size(); }

  /** Reply true iff there are no events in the queue. */
  public  boolean isEmpty() { return size() == 0; }

} /* end class EventQueue */
