
package webflow.backend;

/**
 * Class Diagnostic provides diagnostic services. It makes easy the task
 * of writing diagnostic information into a file. 
 * 
 * CHANGE : June 19, 1997
 * files are going to be under "/tmp/" directory.
 */

import java.io.*;

public class Diagnostic {
  String diagFileName;
  boolean active;

  /**
   * Constructor. record the name of the diagnostic file
   * At start diagnostic file is not active
   * @param fileName - String : name of the diagnostic file
   */
  public Diagnostic(String fileName) {
    diagFileName = "/tmp/"+ fileName;
    active = false;
  }

  /**
   * write a diagnostic message into the diagnostic file
   * @param message - String : message to write in the diagnostic file
   */
  public void write(String message) {
    // write only if diagnostics are active
    if(active) {
      try {
	// first open the file
	RandomAccessFile diagFile = new RandomAccessFile(diagFileName, "rw");

	// at the end of the file, write the message
	diagFile.seek(diagFile.length());
	diagFile.writeChars(message);
	diagFile.writeChars("\n");
	diagFile.close();
      }
      // ignore exceptions at this stage
      catch(IOException ioe) {
      }
    }
  }

  /**
   * set diagnostic to active or inactive state. In inactive state no messages
   * will be produced. In active state, all messages will be written to 
   * the diagnostic file
   * @param outputNeeded - boolean : if true - start sending messages to file
   *                                 if false - ignore messages
   */
  public void activate(boolean outputNeeded) {
    active = outputNeeded;
  }

  /* test stub */
  public static void main(String args[]) {
    Diagnostic diag = new Diagnostic("diag.test");
    diag.activate(true);
    diag.write("line 1 " + "word 4");
    diag.write("line 2 ");
    diag.activate(false);
    diag.write("this must not be written");
    diag.activate(true);
    diag.write("line 3 ");
  }
}
