package webflow.backend;

/**
* the IDFactory class will create unique IDs, given that its initial 
* string is different from any other IDFactory
*/
/* This class is made public only so that it can appear in JavaDoc. It
   should NOT be public. The user will never see it.
*/
public class IDFactory extends Object {
  String uniqueString;
  int uniqueInt;

  /** 
  * creator of the IDFactory class. Different IDFactories are created with
  * a different initial string, so that the instances will be different
  * @param unique: String with a value no other IDFactory has
  */
  IDFactory(String unique) {
    uniqueString = unique;
    uniqueInt = 0;
  }

  /**
  * return a unique ID, based on the initial unique string. 
  * @return an ID that can be used for either ports or modules (or 
  * whatever else)
  */
  public ID getUniqueID() {
    ID toReturn = new ID(uniqueString, uniqueInt++);
    return toReturn;
  }

  /**
  * return a unique ID, based on the initial unique string. 
  * @return an ID that can be used for either ports or modules (or 
  * whatever else)
  */
  public ID getUniqueID(String ServerAddress) {
    ID toReturn = new ID(ServerAddress+"_"+uniqueString, uniqueInt++);
    return toReturn;
  }

  /**
  * testing purposes only. Test stub for class IDFactory
  */
  public static void main(String [] argv)
  {
    IDFactory fac = new IDFactory("blah");
    ID one = fac.getUniqueID();
    ID two = fac.getUniqueID();
    ID three = one;
    //System.out.println(" one == two: " + one.equals(two));
//    System.out.println(" two == three: " + two.equals(three));
  //  System.out.println(" three == one: " + three.equals(one));
  }
}

