package webflow.backend;

import java.io.*;
import java.net.*;
/**
* the Message class is an abstract class from which all request and reply
* classes (i.e. all 'communication objects') are derived. It just insists
* that all derived classes will know how to write themselves onto a stream
* (i.e. to a socket), and how to read themselves on the other end. 
*/
public abstract class Message {
  /**
  * read the Message from the input stream 'in'. 
  * @param in: an InputStream from which the representation for the Message
  * 	       will be read. 
  * @return true if the Message was read, false if some error occured. 
  */
  public abstract boolean fromStream(InputStream in);

  /**
  * write the Message to the output stream 'out'. 
  * @param out: an OutputStream to which the representation for the Message
  * 		will be written. 
  * @return true if the Message was written, false if some error occured. 
  */
  public abstract boolean toStream(OutputStream out);
}























