package webflow.backend;

import java.util.*;
import java.net.*;
import java.io.*;
import uci.graphedit.*;


/** ModuleSourceList is a class that manages the list of the modules 
 *  in the Webflow system. It manages a file in wich all the modules 
 *  are saved. The public interface includes methods to create a list
 *  of modules by parsing the file, look for server on wich a certain 
 *  module exists.
*/


public class 
ModuleSourceList extends Object
{

   Vector list;

   public 
   ModuleSourceList()
   {
      list = new Vector();
   }


   /** Method parse takes a URL address of a Module Name repository file.
   *  parses it and creates a list of modules that exists in the system.
   * @param url - a URL address of the Module Name repository file
   * @return boolean - returns true on success, and false otherwise.
   */
   public  boolean 
   parse( String url)
   {
  
      URL address = null;
      DataInputStream file = null;

      //System.out.println(" I am in parse() of "+url);
    
      try
      {
         /*****************/
         /** create a URL */
         /*****************/
         address = new URL(url);

      } catch( MalformedURLException e)
        {
           //System.out.println(" Can't create URL from the given path ");
           e.printStackTrace();
           return false;
        }

      //System.out.println(" I got the address () ");

      try
      {

         if( address != null )
         {
            /****************************************/
	        /** opens a Module Name Repository file */
            /****************************************/
	        file = new DataInputStream( address.openStream() );

            //System.out.println(" I opened the file () ");

         } else   System.out.println(" I opened the file () ");
     
      } catch(IOException e)
        {
           //System.out.println(" Can't open given file ");
           e.printStackTrace();
           return false;
        }

      if( file != null )
      {
         /*********************************/
	     /** It looks like file is exist. */
         /*********************************/

         try
         {
            //System.out.println(" File is not empty ");
	        String  module = null;

            /*************************************/
            /** As long as we can read something */
            /*************************************/

	        while( ( module = file.readLine() ) != null )
            {
               /********************/
	           /** parse the file  */
               /********************/
	           StringTokenizer st = new StringTokenizer( module );
 
               /****************************************************/
	           /** We are expecting at least 3 element on the line */
               /****************************************************/

               if(st.countTokens()>2)
               { 
	               ModuleListItem m = new ModuleListItem(st.nextToken(), st.nextToken(),
						st.nextToken());

                   /*******************************/
	               /** add the module to the list */
                   /*******************************/
	               list.addElement( m );
                   //System.out.println(" A module "+m.getName()+" on "+m.getServer()+" is added !");
               } 
//               else System.out.println(" Empty line "); 

	        }// end of while
	
         } catch(IOException e)
           {
	          System.out.println(" Read error");
              e.printStackTrace();
	          return false;
           }
      }

      //System.out.println(" I am out of parse() ");

      return true;

   }// end of parse()
 
 /** Method findServer takes a module name and returns a list of all servers
  *  that have a module with that name.
  * @param moduleName - name of module
  * @return Vector - a list of servers that have that module
  */

 public Vector findServer( String moduleName ){
   
   Vector serverList = new Vector();
   int serverNum = 0;

   for( Enumeration e = list.elements(); e.hasMoreElements();)
   {
     ModuleListItem m = (ModuleListItem)e.nextElement();

     if( m.getName().equals( moduleName ) )
     {
       serverList.addElement( m.getServer() );
       serverNum ++;
       //System.out.println(moduleName+" is on "+m.getServer());
     }
   }

   //System.out.println(moduleName+" is "+(serverNum == 0?"not":"")+" found!");       
   if( serverNum == 0 )
     return null;
   else 
     return serverList;
   
 }

 public static void main(String[] argv){
   ModuleSourceList list = new ModuleSourceList();
   list.parse("file:/home/T7F/maja/src/gef/uci/graphedit/webflow/moduleList");
   Vector v = list.findServer("Adder");
   for( Enumeration e = v.elements(); e.hasMoreElements();){
     //System.out.println( ((String)e.nextElement()) );
   }


 }
  
}
