package webflow.frontend;

import java.util.*;
import java.awt.*;
import java.applet.*;
import java.net.*;
import java.io.*;
import uci.graphedit.*;

/**
* Panel
*   |
*   +------------+
*                |
*             ControlPanel
* _______________________________________________________________
*
* ControlPanel is a frame that contains the controls for the WebFlow
* Editor. The commands are: create, run, and destroy.
*
*/
public class 
ControlPanel extends Panel 
             implements UserInterfaceDebug, UserInterfaceConstants
{

  /** stores the mode of the editor */
  private int mode;            

  /** control CheckBox group */
  CheckboxGroup c;            

  /** reference to the parent editor */
  private Editor parent;       


  /**
  * Construct a Control panel with three commands: <br>
  *      - create;     <br>
  *      - run;        <br>
  *      - destroy;    <br>
  */  
  public 
  ControlPanel(Editor e)
  {
  
    /** set the reference to the parent editor */
    parent = e;                 

    setForeground(ControlPanelFgClr);
    setBackground(ControlPanelBgClr);

    c = new CheckboxGroup();
    setLayout( new GridLayout(1,3));

    add( new Checkbox("create", c, true) );
    add( new Checkbox("run", c, false) );
    add( new Checkbox("destroy", c, false));
    
  }// end of constructor

  /** 
  * handels events:
  * 
  * <table border=2>
  * <tr> <td> Selected </td> <td> Mode               </td> </tr>
  * <tr> <td> create   </td> <td> creates modules    </td> </tr>
  * <tr> <td> run      </td> <td> runs the graph     </td> </tr>
  * <tr> <td> destroy  </td> <td> destroys the graph </td> </tr>
  * </table>
  *
  */
  public boolean 
  action( Event e, Object arg)
  {
     boolean ret = false;

     if(DEBUG == true) 
       System.out.println(" I am in action() of ControlPanel "); 

     if( e.target instanceof Checkbox)
     { 
       String current = new String(c.getCurrent().getLabel());

       if( current.equals("create") )
       {  /** mode create  */
          if(DEBUG == true)
             System.out.println("  I am in create Mode");
       } 
       else if( current.equals("run") )
            {  
               /** mode run  */
               ActionExecute a = new ActionExecute( parent );

	           /** run the graph */
	           a.doIt(e);    
	           ret = true;
            }
       else if (current.equals("destroy") )
            {  
               /** mode destroy */
	           ActionDestroy a = new ActionDestroy(parent );

	           /** destroy the graph */
	           a.doIt(e);     

	           ret = true;

            }

     }// end of if

     return ret; 

  }// end of action()

}// end of class ControlPanel

