

package webflow.frontend;

import java.applet.*;
import java.awt.*;
import java.lang.Math;
import java.io.*;
import java.util.*;
import uci.graphedit.*;

/** 
   Fig
    |
    +-----------+
                |
              FigArc1
 __________________________________________________

        Primitive Figure for displaying arcs
*/

public class FigArc1 extends Fig {

  /** Used as a percentage tolerance for making it easier for the user
   * to select a hollow circle with the mouse. Needs-More-Work: This
   * is bad design that needs to be changed. Should use just
   * GRIP_FACTOR. */
  public static final double CIRCLE_ADJUST_RADIUS = 0.1;
 
 /** startng angle of the arc */
  protected int start_angle;
 
 /** size of the arc in degrees */
  protected int size_angle;

  /** Construct a new FigArc with the given position, size, and
   * line color, staring angle, and size of the arc in degrees.
   */
  public FigArc1(int x, int y, int width, int height, Color c_color,
		 int st_angle, int sz_angle) {
    super(x, y, width, height, c_color, null, 4);
    start_angle = st_angle;
    size_angle = sz_angle;
  }

  /** Construct a new FigCircle with the given position, size, and
   * attributes, starting angle, and size of the arc in degrees.
   */
  public FigArc1(int x, int y, int width, int height, Hashtable gAttrs,
		 int st_angle, int sz_angle) {
    super(x, y, width, height, Color.black, null, 4);// call ctor of Fig
    //setGraphicAttributes(gAttrs);
    put(gAttrs);
    start_angle = st_angle;             // sets the arc atributes
    size_angle = sz_angle;
  }

  /** Construct a new FigCircle with the given position, size, line
  * color, and fill color , starting angle of the arc, and size of the arc in
  * degrees.
  */

  public FigArc1(int x, int y, int width, int height,
		   Color c_color, Color f_color, int st_angle, int sz_angle) {
    super(x, y, width, height, c_color, f_color, 4); // calls the ctor of Fig
    start_angle = st_angle;      // set the arc atributes
    size_angle = sz_angle;

  }
  

  /** Draws this FigArc1 */
  public void draw(Graphics g) {

    int rx = position().x;
    int ry = position().y;
    int new_x, new_y, new_w, new_h;

    if (objectWidth > 0) { new_x = rx; new_w = objectWidth; }
    else { new_x = rx + objectWidth; new_w = -objectWidth; }
    
    if (objectHeight > 0) { new_y = ry; new_h = objectHeight; }
    else { new_y = ry + objectHeight; new_h = -objectHeight; }
    
    /* See if this is a filled object */
    if (objectFillColor!=null) {
      g.setColor(objectFillColor);
      g.fillArc(new_x,new_y,new_w,new_h, start_angle, size_angle);
    }
    g.setColor(objectLineColor);
    g.drawArc(new_x,new_y,new_w,new_h, start_angle, size_angle);
  }



  /** Position and draw the handles for when this item is selected */
  public void drawSelected(Graphics g) {
    int rx = position().x;
    int ry = position().y;
    int new_x, new_y, new_w, new_h;

    if (objectWidth > 0) { new_x = rx; new_w = objectWidth; }
    else { new_x = rx + objectWidth; new_w = -objectWidth; }
    
    if (objectHeight > 0) { new_y = ry; new_h = objectHeight; }
    else { new_y = ry + objectHeight; new_h = -objectHeight; }

    _handleRects[0].move(new_x + (objectWidth/2) - HAND_SIZE/2,
			  new_y - HAND_SIZE/2);
    _handleRects[1].move(new_x + objectWidth - HAND_SIZE/2,
			  new_y + (objectHeight/2) - HAND_SIZE/2);
    _handleRects[2].move(new_x + (objectWidth/2) - HAND_SIZE/2,
			new_y + objectHeight - HAND_SIZE/2);
    _handleRects[3].move(new_x - HAND_SIZE/2,
			  new_y + (objectHeight/2) - HAND_SIZE/2);
    drawHandles(g);
  }

  /** When this FigArc is selected, the Editor should use a
   * SelectionHandles to record that fact and process events. */
  public Selection selectionObject() { return new SelectionHandles(this); }


  /** Reply true if the given mouse coordinates are "near" the arc
   */
  public boolean inside(int x, int y) {
    boolean ret;
    Rectangle capture;
    double dbl_pntx, dbl_pnty, dbl_width, dbl_height;
    double tmpx, tmpy, centerx, centery;
    double pnt_dist;
    double arc_dist_in, arc_dist_out;
    Point p = position();
    capture = new Rectangle(p.x - GRIP_MARGIN,
			    p.y - GRIP_MARGIN,
			    objectWidth + GRIP_MARGIN * 2,
			    objectHeight + GRIP_MARGIN * 2);
    /* First see if its within the bounding rectangle */
    ret = false;
    if (capture.inside(x, y)) {
      centerx = (double)objectWidth/(double)2 + (double)p.x;
      centery = (double)objectHeight/(double)2 + (double)p.y;

      dbl_pntx = (double)(centerx - x) / (objectWidth / 2);
      dbl_pnty = (double)(centery - y) / (objectHeight / 2);

      pnt_dist = Math.sqrt(dbl_pntx * dbl_pntx + dbl_pnty * dbl_pnty);
      dbl_width = (double)objectWidth;
      dbl_height = objectHeight;
      arc_dist_out = 1 + CIRCLE_ADJUST_RADIUS;

      if (objectFillColor == null) {
	arc_dist_in = arc_dist_out - CIRCLE_ADJUST_RADIUS;
      } else
	      arc_dist_in = -1;
      ret = ( (pnt_dist < arc_dist_out) && (pnt_dist > arc_dist_in) );
    }
    return ret;
  }

  /** When the user drags one of my handles, move or resize accordingly */
  public void dragHandle(int mouse_x, int mouse_y,
                         int anchor_x, int anchor_y,
                         Handle handle) {
    Point p = position();
    int old_left = p.x;
    int old_top = p.y;
    int old_right = p.x + objectWidth;
    int old_bottom = p.y + objectHeight;

    switch (handle.index) {
    case 0:   /* The top middle hitpoint */
      p.y = Math.min(mouse_y, old_bottom - MIN_SIZE);
      objectHeight = old_bottom - p.y;
      break;
    case 1:   /* The right middle hitpoint */
      objectWidth = Math.max(mouse_x - old_left, MIN_SIZE);
      break;
    case 2:   /* The bottom middle hitpoint */
      objectHeight = Math.max(mouse_y - old_top, MIN_SIZE);
      break;
    case 3:    /* The left middle hitpoint */
      p.x = Math.min(mouse_x, old_right - 3);
      objectWidth = old_right - p.x;
      break;
    default:
      super.dragHandle(mouse_x, mouse_y, anchor_x, anchor_y, handle);
      break;
    }
    if (objectWidth < MIN_SIZE) {
      if (handle.index == 1) p.x = old_right - MIN_SIZE;
      objectWidth = MIN_SIZE;
    }
    if (objectHeight < MIN_SIZE) {
      if (handle.index == 2) p.y = old_bottom - MIN_SIZE;
      objectHeight = MIN_SIZE;
    }
  }


} /* end class FigArc */



































