
package webflow.frontend;

import java.awt.*;
import java.io.*;
import java.net.*;
import uci.graphedit.*;
import java.util.*;

/** 
*  ModuleTree contains a list of all modules that can be used 
*  in a session.
* <b>
* Reads the Module definition file from the URL address.
* The expected format is
* <p>
* <pre>
*  <ClassName> <ServerName> <ViewerClassName> [<ClassLabel>]
* </pre>
*
* If there is no <ClassLabel>, then the last in the <ClassName>
* will be the Label.
*
*/
public class 
ModuleTree extends Object
           implements UserInterfaceDebug 
{

  public static ModuleTreeItem currentItem = null;

  private String name  = null;

  /** Module Description file's URL */
  private String url   = null;

  /** Actually this is going to be a Tree */
  private Vector items = null;

  /** 
  * defines the maximum number of levels
  * on the tree.
  */
  private int    maxLevel = 0;

  /** 
  * Creates a new list of modules 
  * @param _url : URL address of the Module Description File 
  */
  public
  ModuleTree(String _name, String _url)
  {
	name  = new String( _name );
	url   = new String( _url );
	items = new Vector();
    
    if(DEBUG3==true)
        System.out.println("I created root ");

    /** Root is fixed as "modules" */
    ModuleTreeItem modules = new 
                 ModuleTreeItem(ModuleTreeItem.PACKAGE, "modules", null, 
                                null, "modules");

    if(DEBUG3==true)
       System.out.println("ModuleTreeItem is  created ");

    /** Place the first element to the Tree */
    items.insertElementAt(modules,0);
                                                 
  }// end of constructor

  /** Returns the tree */
  public Vector 
  getItems() { return items; }

  /** 
  * Parses the line read from the Module Description file.
  * 
  */
  private void
  parseInsertItem(String line)
  {
     if(line.length()==0) return;     
     if(DEBUG3==true) 
        System.out.println("\n\n line is :"+line);

     StringTokenizer st = new StringTokenizer( line );
     if(st.countTokens()<3) return;

     String moduleName = st.nextToken();
     if(DEBUG3==true) 
        System.out.println("moduleName = " + moduleName);

     String server = st.nextToken();
     if(DEBUG3==true) 
        System.out.println("server = " + server);

     String moduleDescClass = st.nextToken(); 
     if(DEBUG3==true) 
        System.out.println("moduleDescClass = " + moduleDescClass);

     /** If there is label */
     String mdLabel = null;
     if(st.hasMoreTokens()) mdLabel = st.nextToken();
     
     if(DEBUG3==true)
     {  if(mdLabel==null) System.out.println("Label is not provided");
        else System.out.println("mdLabel = "+mdLabel);
     }//end of if

     String className = new String(moduleName);
     String pckName   = className.replace('.',' ');
     StringTokenizer pckTokens = new StringTokenizer( pckName );
     
     /** Here goes one for the "webflow" of webflow.modules */
     int numEle = pckTokens.countTokens() - 1; 
     int getTokenNum = 0;

     if(DEBUG3==true)
        System.out.println("pckName   = "+pckName+" deep: "+numEle);

     /** We expect to see "webflow.modules.***" */
     String Mname = pckTokens.nextToken();

     if(!Mname.equals("webflow")) 
     { 
        if(DEBUG3==true)
          System.out.println("It does not start with *webflow* ");

        return;
     }

     if(DEBUG3==true)
        System.out.println("It does start with *webflow* ");


     Mname = pckTokens.nextToken();
 
     if(DEBUG3==true)
        System.out.println("The next one is :"+Mname);
 
     getTokenNum++;

     ModuleTreeItem current = 
        (ModuleTreeItem) items.elementAt(0);

     String CurClassName = new String("webflow.modules");

     /** Each classname should start with "modules" package */
     if(Mname.equals("modules"))
     {
        if(DEBUG3==true) System.out.println("You are modules");
        maxLevel = Math.max(maxLevel,numEle);

        Mname = pckTokens.nextToken();
        getTokenNum++;
        
        while( getTokenNum <= numEle )
        {
           CurClassName = CurClassName + "." + Mname;

           if(getTokenNum == numEle)
           {  
              if(DEBUG3==true)
                 System.out.println("ClassName is :"+CurClassName);
              
              boolean isModuleExist = false;
              
              Vector subs = current.getSubModules();
              if(subs!=null)
              {  Enumeration MList = subs.elements();
                 
                 for(;isModuleExist == false && MList.hasMoreElements();)
                 {  ModuleTreeItem ele = (ModuleTreeItem ) MList.nextElement();
                    if(ele.getClassName().equals(CurClassName)) 
                        isModuleExist = true;
                 }
              } 

              /** If module does not exist, then make one */
              if(isModuleExist==false)
              {
                 if(DEBUG3==true)
                    System.out.println("A new class");

                 ModuleTreeItem ele =
                      new ModuleTreeItem(ModuleTreeItem.MODULE,
                                         CurClassName, server, moduleDescClass,
                                         (mdLabel==null?Mname:mdLabel));
                 current.addSubItem(ele);

                 if(DEBUG3==true)
                    System.out.println("A new class is added");

              } else if(DEBUG3==true) 
                        System.out.println("A class is already exist!");

           } 
           else /** Still we did not reach the leaves of the tree */
             {  
                if(DEBUG3==true)
                   System.out.println("PackageName is :"+CurClassName);

                /** Is this module defined before? */
                boolean isModuleExist = false;
                
                /** Denotes the next level in the tree */
                ModuleTreeItem nextLevel = null;
                
                /** get the list of all defined modules-packages */ 
                Vector subs = current.getSubModules();

                /** If there are some */
                if(subs!=null)
                {  
                   /** Enumerate them */
                   Enumeration MList = subs.elements();
                   
                   /** For each element */ 
                   for(;isModuleExist == false && MList.hasMoreElements();)
                   {  ModuleTreeItem ele = (ModuleTreeItem ) MList.nextElement();
                      
                      /** 
                      * If this CurClassName is defined before
                      * this item is already exist in the ModuleTree.
                      */  
                      if(ele.getClassName().equals(CurClassName))
                      {  isModuleExist = true;
                         nextLevel     = ele;
                      }// end of if

                   }// end of for

                }// end of if

                /** If package is not defined before */
                if(isModuleExist==false)
                {
                   if(DEBUG3==true)
                      System.out.println("A new package");

                   /** Create PACKAGE tree entry */
                   ModuleTreeItem ele =
                      new ModuleTreeItem(ModuleTreeItem.PACKAGE,
                                         CurClassName, server, null,
                                         (mdLabel==null?Mname:mdLabel));

                   /** Add this to the current Level of the tree*/
                   current.addSubItem(ele);

                   /** Update the next level  */
                   nextLevel     = ele;

                   if(DEBUG3==true)
                      System.out.println("A package is added");
                    
                } else if(DEBUG3==true)
                          System.out.println("A package is already defined!");

                /** get next token from the package name */
                Mname = pckTokens.nextToken();

                /** Go one level down on the tree */
                current = nextLevel;

             }
           
           getTokenNum++;

        }// end of while
        
     } else if(DEBUG3==true) 
               System.out.println("You are *not* modules");

  }// end of parseInsertItem() 

  /**
  * Prints ou the all the stored Module-Package information
  * in the tree in deep-first fashion
  */
  private void
  printMe()
  {
     Stack myStack = new Stack();
     ModuleTreeItem current  = null;

     /** get the root node of the Module-Package Tree */
     current = (ModuleTreeItem) items.elementAt(0);

     if(current!=null) myStack.push(current);
     else System.out.println(" Root is empty ");
 
     /** If stack is not empty */
     while(myStack.empty()!=true)
     {  
        /** Get the Top of Stack */
        ModuleTreeItem top = (ModuleTreeItem) myStack.pop();

        /** If it is legal object for us */
        if(top!=null)
        {  System.out.println("\n\n Class Name :"+top.getClassName());
           System.out.println(" Label      :"+top.getClassLabel());
           System.out.println(" Server     :"+top.getServer());
           System.out.println(" DescClass  :"+top.getModuleDescClass());
 
           /** Get the sub modules */
           Vector subs = top.getSubModules();

           /** It is not a package */
           if(subs==null) 
               System.out.println(" Class itself ");
           else System.out.println(" PACKAGE "); 

           /** If it is a package */
           if(subs!=null)
           {  
              /** take all its subcomponents */
              Enumeration ele = subs.elements();
              
              /** place them on top of the stack */
              for(;ele.hasMoreElements();) 
                 myStack.push(ele.nextElement());
           }
        }  

     }// end of while

  }// end of printMe()

  /**
  * retuns the height of the tree
  */ 
  public int
  getHeight() { return maxLevel; }

  /**
  * Reads the module description file from the URL 
  */
  public boolean 
  load() 
  {
     URL address = null;
     DataInputStream file = null;
    
     try
     { address = new URL(url);
     } catch( MalformedURLException e)
       {  System.out.println(" Can't open ModulelTree URL = " + url);
          e.printStackTrace();
	      return false;
       }

     if( DEBUG == true )
         System.out.println(" URL is OK ");

     try
     {
       if( address != null )
       { file = new DataInputStream( address.openStream() ); }
     } catch(IOException e)
       {  System.out.println(" Can't open stream to ModuleTree URL = " + url);
          e.printStackTrace();
	      return false;
       }

     if( file != null )
     {
       try
       {  
          String  module = file.readLine();
          if(DEBUG3==true) System.out.println( module );

	      while( module != null )
          {  
             parseInsertItem(module);
	         module = file.readLine();

	      }//  end of while

       } catch(IOException e)
         {  System.out.println(" Read error"+e);
            e.printStackTrace();
	        return false;
         }

     }// end of if

   return true;

  }// end of load()

  /** test stub */
  public static void main(String args[]) 
  {
    ModuleTree demo = new 
       ModuleTree("timucin","http://osprey1.npac.syr.edu:9080/modules.txt");

    if(demo.load()) 
    {  System.out.println("it is loaded");
       demo.printMe();
       System.out.println("The height of the tree is "+demo.getHeight());
    }

  }// end of main()

  
}// End of ModuleTree class 

