/**
*
*/
package webflow.frontend;

import java.io.OutputStream;
import java.io.IOException;
import java.util.*;
import uci.graphedit.*;

/** 
* This class is an abstraction of module as a member of the ModuleTree 
*/
public class 
ModuleTreeItem extends Object
{
  /**
  * Type of item represented by this TreeItem.
  * <b>ModuleTreeItem.PACKAGE</b> and <b>ModuleTreeItem.MODULE</b>
  */
  public static final int PACKAGE=0, MODULE=1, UNDEFINED=-1;

  /** Keeps the type of this item (PACKAGE,MODULE,UNDEFINED) */
  private int    Type       = -1;

  /** Class Name of this object, the whole name */
  private String className  = null;
  
  /** for the viewer modules */
  private String descName   = null;
 
  /** The label of the button representing this module */
  private String classLabel = null;

  /** On which WebFlow Server we are going to run this */
  private String serverName = null; 

  /** A vector of sub module-packages */ 
  private Vector subModules = null; 
  
  /** 
  * Creates new ModuleTreeItem with given name, 
  * server and description of a module.
  */
  public 
  ModuleTreeItem(int _type, String n, String s, 
                 String desc, String _label )
  {
    
    /** Decide the type of module */
    switch(_type)
    {  case ModuleTreeItem.PACKAGE : Type = ModuleTreeItem.PACKAGE;
            break;
       case ModuleTreeItem.MODULE  : Type = ModuleTreeItem.MODULE ;
            break;
       default                     : Type = ModuleTreeItem.UNDEFINED;
            break;
    }// end of switch

    if(n!=null) className  = new String( n );
    if(s!=null) serverName = new String( s );
    if(desc!=null) descName   = new String( desc );
    if(_label!=null) classLabel = new String( _label );
    else classLabel = new String("Unknown");

    /** Package can have a sub-modules */
    if(Type==ModuleTreeItem.PACKAGE) subModules = new Vector();

  }// end of constructor

  /** returns the name of the module. */
  public String 
  getClassName() { return className; };

  /** returns the label of the module. */
  public String 
  getClassLabel() { return classLabel; };

  /** returns the server on which the module is residing*/
  public String 
  getServer(){ return serverName; }

  /** returns the class that represents the module.*/
  public String 
  getModuleDescClass(){ return descName; };

  /** Adds a subModule to the this entry if it is a PACKAGE */
  public void 
  addSubItem(ModuleTreeItem _sub)
  { if(Type==ModuleTreeItem.PACKAGE) subModules.addElement(_sub);
  }// end of addSubItem()

  /** gets the all the sub module entries if it is a PACKAGE */
  public Vector 
  getSubModules() 
  { return (Type==ModuleTreeItem.PACKAGE?subModules:null); } 

  /** gets the type code */
  public int 
  getType() 
  { return Type; };

  /** gets the package name */
  public String
  getPackageName()
  {  String ret = null;  
     String Name = className.replace('.',' ');
  
     StringTokenizer st = new StringTokenizer(Name);
     if(st.countTokens()>1)
     { 
        ret = st.nextToken();

        while(st.countTokens()>1) ret = ret+"."+st.nextToken();

     }// end of if

     return ret;

  }// end of getPackageName()

  /**
  *  Prints the module info to the  OutputStream 
  */
  public void
  printMe(OutputStream out)
  {  String line1 = "   Classname : "+getClassName()+"\n";
     String line2 = " PackageName : "+
                    (getPackageName()!=null?getPackageName():"")+"\n";
     String line3 = "        Type : ";
     line3 += (Type==PACKAGE?"PACKAGE":(Type==MODULE?"MODULE":"UNKNOWN"));
     line3 +="\n\n\n";
     byte[] bline1 = new byte[150];
     byte[] bline2 = new byte[150];
     byte[] bline3 = new byte[150];
     line1.getBytes(0,line1.length(),bline1,0);
     line2.getBytes(0,line2.length(),bline2,0);
     line3.getBytes(0,line3.length(),bline3,0);

     try 
     {  out.write(bline1); 
        out.write(bline2); 
        out.write(bline3); 
     } catch (java.io.IOException e)
       {  System.out.println("IOException is raised!"+e);
          e.printStackTrace();
       }

  }// end of printMe()

}// End of class ModuleTreeItem 
