
package webflow.frontend.WFtree;

import webflow.frontend.*;
import java.util.*;
import java.awt.*;
import uci.graphedit.*;

/**
*/
public class
ModuleTreeNode extends TreeNode
{
   /** Root node of the tree that we are going to browse */
   private ModuleTreeItem myMTI = null;

   /** Color of this component on the screen */
   private Color cmp_color = Color.blue;

   /**
   * This automatically creates the tree structure so that 
   * we can browse on it.
   *
   * @param _myMTI : The root node of the tree, ModuleTreeItem
   */
   public
   ModuleTreeNode(ModuleTreeItem _myMTI)
   {  
      super();
      myMTI = _myMTI;


      if( myMTI.getType() == ModuleTreeItem.PACKAGE)
      {  
         Enumeration ele = myMTI.getSubModules().elements();

         if(ele!=null)
         {
           Vector sorter = new Vector();
           int cnt=0, i, j;

           while( ele.hasMoreElements() ) 
           {  sorter.insertElementAt(ele.nextElement(),cnt);
              cnt++;
           }// end of while

           for(i=0;i<cnt-1;i++)
              for(j=i+1;j<cnt;j++)
              {
                 ModuleTreeItem pre = (ModuleTreeItem)sorter.elementAt(i);
                 ModuleTreeItem cur = (ModuleTreeItem)sorter.elementAt(j);

                 if(compare(pre.getClassLabel(),cur.getClassLabel())>0)
                 {  
                    sorter.setElementAt(cur,i);
                    sorter.setElementAt(pre,j);

                 }// end of if 

              }// end of for

           ele = sorter.elements();

           while( ele.hasMoreElements() )
           {  
              ModuleTreeItem mti = (ModuleTreeItem)ele.nextElement(); 

              ModuleTreeNode nw = new ModuleTreeNode(mti);

              nw.setComponentColor(Color.white);
              nw.setSelectedColor(Color.magenta);
              nw.setUnSelectedColor(Color.black);
              nw.setLabel(mti.getClassLabel());

              if(mti.getType() == ModuleTreeItem.MODULE) 
              {  nw.setComponentColor(Color.yellow);
                 nw.setSelectedColor(Color.blue.brighter());
                 nw.setUnSelectedColor(Color.red.brighter());
                 
                 Action act = new ActionCreateWebNode(mti.getModuleDescClass(),null,false,mti);
                 nw.setAction(act);
              }// end of if 
              
              addChild(nw); 
           }// end of while

         }// end of if

      }// end of if

      setComponentColor(Color.white);
      setSelectedColor(Color.magenta);
      setUnSelectedColor(Color.black);

   }// end of constructor

   /** 
   * @param _newClr : Sets the color of the component
   */
   public void
   setComponentColor(Color _newClr)
   { cmp_color = _newClr; }


   /** 
   * Overwrite the <b>draw()</b> method od TreeNode object. 
   *
   * @param g : Graphics object.
   */
   public void
   draw(Graphics g)
   {

      Point p = getPosition();

      String label = null;

      /** Define the label of this object first */
      if( (isOpen() == false) && ( isLeaf() == false )) 
           label = new String("[+] "+myMTI.getClassLabel());
      else if(isLeaf() == false) label = new String("[-] "+myMTI.getClassLabel());
           else label = new String(myMTI.getClassLabel());

      int labelwidth = 10 + g.getFontMetrics().charsWidth(label.toCharArray(),0,label.length());

      setWidth( Math.max(getWidth(),labelwidth) );

      /** draw the button */
      g.setColor(cmp_color);

      int xi = p.x;
      int yi = p.y;
      int wid = getWidth();
      int hei = getHeight();
      boolean val = (isSelected()==false);
      
      g.draw3DRect(xi,yi,wid,hei,val);
      g.draw3DRect(xi+1,yi+1,wid-2,hei-2,val);
      g.draw3DRect(xi+2,yi+2,wid-4,hei-4,val);

      g.fillRect(xi+2,yi+2,wid-4,hei-4);

      //g.fill3DRect(p.x,p.y,getWidth(),getHeight(),isSelected()==false);

      /** draw the button label */
      g.setColor( (isSelected()==true?getSelectedColor():getUnSelectedColor()) );
      if(isSelected()==true)
           g.drawString(label,p.x+6,p.y+1+getHeight()-g.getFontMetrics().getHeight()/2);
      else g.drawString(label,p.x+5,p.y+getHeight()-g.getFontMetrics().getHeight()/2);

   }// end of draw()
  

   /** 
   * A :65 Z :90, a:97 z:122
   * Because of this, we need to rewrite this string compare function.
   */
   private static final int
   compare(String firstSTR, String secondSTR)
   {
      int res = 0;
      int i = 0;
      char[] first  = firstSTR.toCharArray();
      char[] second = secondSTR.toCharArray();

      int limit = Math.min(first.length,second.length);
      
      while(i<limit && first[i]==second[i]) i++;

      if(i==limit)
      {  
         /** first string is smaller */
         if(limit<second.length) res = -1;

         /** first string is bigger  */  
         if(limit<first.length) res =  1; 
      }
      else 
       {  
          /** If both capital or small letters */
          if ( ( first[i]>64 && first[i]<91 && second[i]>64 && second[i]<91 ) ||
               (first[i]>96 && first[i]<123 && second[i]>96 && second[i]<123) ) 
             { if(first[i]>second[i]) res =  1;
               else res = -1;
             }
          else
            {  /** If one is capital and the other is small letter */
               if(first[i]>second[i]) res = -1;
               else res = 1; 
            }
       }
     
      return res;

   }// end of compare()

}// end of class ModuleTreeNode




