package webflow.frontend.WFtree;

import java.lang.*;
import java.util.*;
import java.awt.*;
import webflow.frontend.UserInterfaceDebug;

/**
*
* @date August , 1997 ( Updated at : November 3, 1997 )
* @author H. Timucin Ozdemir ( timucin@npac.syr.edu )
*/
public class 
TreeViewPanel extends Panel 
              implements UserInterfaceDebug
{
  ScrollerInterface scroller = null; 
  TreeNode selected = null;
  Dimension psize = null; // preferred size 
  Dimension tsize = null; // total (tree) size
  // draw to offscreen graphics from example GraphLayout (SUN) 
  Image offscreen = null;
  Dimension offscreensize = null;
  Graphics offgraphics = null;
  // some layout constants
  public int horizontal_offset = 24;
  public int vertical_offset = 16;
  public Color line_color = Color.black;
  public Color background_color =  Color.lightGray;
  private TreeNode root;
  
  Frame myFrame = null;

 /** without scrollbars : */
 public 
 TreeViewPanel(TreeNode _root, int wid,int hei, Frame _myFrame) 
 {
   this.scroller = null;
   selected = root = _root;
   //root.select();
   root.unOpen();
   psize = new Dimension(wid,hei);
   tsize = new Dimension(0,0);
   setFont( new Font("Courier",Font.BOLD,12));
   myFrame = _myFrame;
 }

  /** with scrollbars : */
  public 
  TreeViewPanel(TreeNode _root, ScrollerInterface scroller,
                int wid, int hei, Frame _myFrame) 
  {
    this.scroller = scroller;
    root = _root;
    psize = new Dimension(wid,hei);
    tsize = new Dimension(0,0);
    setFont( new Font("Courier",Font.BOLD,12));
    myFrame = _myFrame;
  }// end of constructor

  /** */
  public void
  setRoot(TreeNode _root)
  {
     selected = root = _root;

     root.unOpen();

     offscreen = null;

     if(scroller!=null)
     {
        int h_val = 0;
        int v_val = 0; 
        int h_vis = size().width;
        int v_vis = size().height;
        int h_max = tsize.width;
        int v_max = tsize.height;
        if (h_val>h_max) h_val= h_max;
        if (v_val>v_max) v_val= v_max;
	
        scroller.setHorizontalScrollerValues(h_val,h_vis,0,h_max);
        scroller.setVerticalScrollerValues(v_val,v_vis,0,v_max);

     }// end of if

     repaint();

  }// end of setRoot() 
  

  /** */
  public void
  setFrame(Frame _myFrame)
  {  if(_myFrame!=null) myFrame = _myFrame; }

  /** */ 
  public Dimension 
  preferredSize() 
  { 
    return minimumSize();

  }// end of preferredSize()

  /** */ 
  public Dimension 
  updateSize() 
  { 
    recomputeTree();

    Dimension frameSize = myFrame.size();
    Dimension ret = psize;

    if(frameSize.width < psize.width || tsize.width < psize.width ) 
                      ret.width = psize.width;
    else ret.width = Math.min(frameSize.width-20,tsize.width);

    if(frameSize.height < psize.height || tsize.height < psize.height ) 
                 ret.height = psize.height;
    else ret.height = Math.min(frameSize.height-20,tsize.height);

    if(DEBUG3==true)
    {  System.out.println("TreeViewPanel::preferredSize() frameSize "+frameSize);
       System.out.println("TreeViewPanel::preferredSize() psize "+psize);
       System.out.println("TreeViewPanel::preferredSize() tsize "+tsize);
       System.out.println("TreeViewPanel::preferredSize() is "+ret);
    }

    ret = psize;

    return ret;

  }// end of preferredSize()

  /** */
  public Dimension 
  minimumSize() 
  { 
    if(DEBUG3==true)
      System.out.println("TreeViewPanel:minimumSize() is "+psize);
    return psize;
  }// end of minimumSize()

  /** */
  public void 
  fullUpdate() 
  {
    // System.out.println("TreeViewPanel::fullUpdate()");
    recomputeTree(); 
    repaint();
  }// end of fullUpdate()
  
  /** */
  public Dimension 
  computeTree(TreeNode node,int x,int y) 
  {
    Dimension child_extent = null;
    node.setPosition(x,y);

    int hei = (node.getLabel().length()>0?getFontMetrics(getFont()).getHeight()+10:node.getHeight());
        hei = Math.min(hei,node.getHeight());

    int wid = (node.getLabel().length()>0?getFontMetrics(getFont()).charWidth('W')*(node.getLabel().length()+5):
                                 node.getWidth());
        wid= Math.min(wid,node.getWidth());

    int xc = x+horizontal_offset;
    int yc = y+hei+vertical_offset;

    if(node.isOpen() == true )
    {  Enumeration ele = node.getChilds().elements();
      
       while(ele.hasMoreElements() )
       {  
          TreeNode child = (TreeNode) ele.nextElement();

          child_extent = computeTree(child,xc,yc);

          if (child_extent.width+horizontal_offset>wid)
                wid = child_extent.width+horizontal_offset;

          yc += child_extent.height+vertical_offset;

       }// end of while

    }// end of if

    hei = yc - y - vertical_offset;

    return new Dimension(wid,hei);

  }// end of computeTree()
  

  /** */
  public void 
  recomputeTree() 
  {
    // System.out.println("TreeViewPanel::recomputeTree()");
    int x = horizontal_offset;
    int y = vertical_offset;
    Dimension extent = computeTree(root,x,y);
    tsize.width = x+extent.width+horizontal_offset;
    tsize.height= y+extent.height+vertical_offset;
    // System.out.println("tree computed, extent is "+extent);
  }// end of recomputeTree()


  /** */
  public void 
  drawTree(Graphics g,TreeNode node) 
  {
    node.draw(g);
    Point p = node.getPosition();
    int lx = p.x+(horizontal_offset/2);
    int ly = p.y+node.getHeight();
    int yp = -1;

    if ( node.isOpen() == true )
    {  
       Enumeration ele = node.getChilds().elements();
       
       while( ele.hasMoreElements() )
       {
          TreeNode nt = (TreeNode) ele.nextElement();
          Point pt = nt.getPosition();
          yp = pt.y + (nt.getHeight()/2);
          g.setColor(line_color);
          g.fillRect(lx-1,yp-1,pt.x-lx+1,3);
          drawTree(g,nt);

       }// end of while

    }// end of if

    if (yp>0) 
    { /** draw vertical line */
      g.setColor(line_color);
      g.fillRect(lx-1,ly-1,3,yp-ly+1);
    }// end of if

  }// end of drawTree()

  

  /** */
  public void 
  update(Graphics g) 
  {
    /**  force redraw : */
    offscreen = null;
    paint(g);
  }


  /** */
  public void 
  paint(Graphics g) 
  {
    /** draws into offscreen Graphics  */
    Dimension d = size();
    if ((offscreen == null) || (d.width != offscreensize.width) || 
        (d.height != offscreensize.height)) 
    {
      offscreen = createImage(d.width, d.height);
      offscreensize = d;
      offgraphics = offscreen.getGraphics();
      offgraphics.setFont(getFont());
      offgraphics.setColor(background_color);
      offgraphics.fillRect(0, 0, d.width, d.height);
      
      /**  scroller stuff  */                   
      if (scroller!=null) 
      {
         Point p = scroller.getScrollPosition();
         int h_val = p.x;
         int v_val = p.y; 
	     int h_vis = d.width;
	     int v_vis = d.height;
	     int h_max = tsize.width;
	     int v_max = tsize.height;
	// System.out.println("Vertical : scroller has value "+v_val);
 	// System.out.println("Vertical : visible size is "+v_vis+" - maximal size is "+v_max);
	// System.out.println("Horizontal : scroller has value "+h_val);
	// System.out.println("Horizontal : visible size is "+h_vis+" - maximal size is "+h_max);
	     if (h_val>h_max) h_val= h_max;
	     if (v_val>v_max) v_val= v_max;
	
	     scroller.setHorizontalScrollerValues(h_val,h_vis,0,h_max);
	     scroller.setVerticalScrollerValues(v_val,v_vis,0,v_max);
	     offgraphics.translate(-h_val,-v_val);

      }// end of if    

      recomputeTree();
      drawTree(offgraphics,root);

    }// end of if

    g.drawImage(offscreen, 0, 0, null);

  }// end of paint()


  /** */
  public TreeNode
  findNode(TreeNode current, int x, int y)
  {
     TreeNode found = null;
     
     if(current.inside(x,y)==true) found = current;
     else if( current.isOpen()==true)
          {  Enumeration ele = current.getChilds().elements();
              
             while( found==null && ele.hasMoreElements() )
             {  found = findNode((TreeNode) ele.nextElement(),x,y); }
 
          }
     
     return found; 

  }// end of findNode()


  /** */
  public boolean
  mouseDown(Event evt, int x, int y)
  {
     boolean ret = false;
     
     /** If there is a scroller, we need to change position */
     if(scroller != null)
     {
        Point p = scroller.getScrollPosition();
        x += p.x; y+=p.y;

     }// end of if

     /** look for the hit */
     TreeNode found = findNode(root,x,y);

     /** If it hits somebody */
     if( found != null )
     {
       /** update the current item */
       selected = found;

       /** make this visible */
       selected.select();

       /** we served the action */
       ret = true;

     }// end of if

     /** If we did something, we need update */
     if( ret == true ) repaint();

     return ret;

  }// end of mouseDown()

  /** */
  public boolean
  mouseUp(Event evt, int x, int y)
  {
     boolean ret = false;
     
     /** If there is selected sombody with */
     /** the revious mouseDown event       */

     if( selected != null )
     {
        /** If there is a scroller, we need to change position */
        if(scroller != null)
        {
           Point p = scroller.getScrollPosition();
           x += p.x; y+=p.y;

        }// end of if
   
        /** If it hits the same object */
        if( selected.inside(x,y) == true  && 
            selected.getAction()!=null )
               selected.getAction().doIt(evt);

        /** make it normal */
        selected.unSelect();

        /** we served the action */ 
        ret = true;

     }// end of if

     /** If we did something, we need update */
     if( ret == true ) repaint();

     return ret;

  }// end of mouseUp()

}// end of class TreeViewPanel

