package webflow.modules.QuantumSimulations;

import webflow.backend.*;
import java.net.*;
import java.io.*;
import webflow.modules.Templates.*;

public class CERIUS2 extends Template {
  
  InputPort inport;
  OutputPort outport;
  WFFileDescriptor InputFile;
  WFFileDescriptor OutputFile = new WFFileDescriptor();

  //any GRAM script has the following attributes
  String contact;   //now read as GATE message; (fixed)
                    //do not block; should come from MDS
  String directory; //this depends on application and contact (fixed)
                    //do not block;
                    //now read as DIRE message
  String executable;//as directory (fixed); do not block;
                    //now read as EXEC message
  String arguments; //for CERIUS2 it is the input filename
                    //derived from input port (block)
  String stdin;     //read as STDI message; not used now
  String stdout;    //read as STDO message; do not block; fixed
  String stderr;    //read as STDE message; do not block; fixed

  String gatekeeper;//used in converting GATE message to full name of contact


  // GRAM CLIENT CODE osprey2.npac.syr.edu specific
  // GRAM client script

  String gramfile="/tmp/gramScript";  //name of the gram client script
                                      //to be generated on osprey2

  //  String GCD="/home/B7D/globus/haupt"; //Gram Client Directory on osprey2
  String GCD="/usr/people/globus/GRAM"; //Gram Client Directory on lglobus
                                 //when /tmp files deleted import from here

  String clientcodename="gram_client_test_irix.0.6.4"; 
  String clientcode="/tmp/"+clientcodename;


  //CERIUS2 SPECIFIC
  String inFile;   //input port
  String outFile;  //output port 
  

  String host;  //derived from contact
  String message,newmessage;
  String command;
  String disp, dummy;
  int idx; //auxilliary

  public MetaModule initialize() {
    
    CreateServerSocket();
    System.out.println("CERIUS: start initialization"); 
    MetaModule mm = new MetaModule("mm");
    outport = new OutputPort("String");
    mm.putPortID(outport.getPortID());
    inport = new InputPort("WFFileDescriptor");
    mm.putPortID(inport.getPortID());
    newmessage=null;
    disp="dummy";
    gatekeeper=null;
    System.out.println("CERIUS: initialization completed"); 
    return(mm);
    }
  
  public void run() {
    
    //wait till you know the ip address of the display
    System.out.println("CERIUS: start"); 
    
    while(newmessage==null) {;}
    System.out.println("set display to: "+newmessage);
    disp=newmessage.substring(4);
    System.out.println("CERIUS: ready to go..."); 

    //defaults:
    contact="modi4.ncsa.uiuc.edu:4001:modi4.ncsa.uiuc.edu-fork@globus.org";
    idx=contact.indexOf(":");
    host=contact.substring(idx);
    directory="/scratch-modi4/haupt/CERIUS2";
    executable="cer";
    arguments=disp;
    stdin=null;
    stdout=directory+"/gauss_stdout";
    stderr=directory+"/gauss_stderr";

    while(true)
      {
       System.out.println("CERIUS: waiting for input"); 

       InputFile  = (WFFileDescriptor)inport.receive();
       inFile = InputFile.getFileName();
       idx = inFile.lastIndexOf("/");
       arguments = inFile.substring(idx+1);
       idx = inFile.indexOf(".");
       outFile = inFile.substring(0,idx)+".out";
       OutputFile.setFileName(outFile);
       OutputFile.setFileDirectory(directory+"/");
       OutputFile.setFileHost("modi4.ncsa.uiuc.edu");
       OutputFile.setFileAttribute(InputFile.getFileAttribute());

     // make sure that the client code exists!

     File client=new File(clientcode);
     if(!client.exists()) {
         System.out.println("GRAM-client code does not exist");
         String copycmd="cp "+GCD+"/"+clientcodename+" "+clientcode;
         System.out.println(copycmd);
         try {
               Process p0 = Runtime.getRuntime().exec(copycmd);
               p0.waitFor();
	 } catch (Exception e) {
             System.out.println("Error: " + e.toString());
         }
     System.out.println("file copied"); 
     }

        //generate script on the fly

        String FileContents = new String("#!/bin/sh -f\n");
        FileContents += "gatekeeper_contact="+contact+"\n";
        FileContents += "spec=\"&(directory="+directory+")";
        FileContents += "(executable="+executable+")";

        if(arguments != null) { FileContents += "(arguments="+arguments+")";}
        if(stdin != null) { FileContents += "(stdin="+stdin+")";}

        FileContents += "(stdout="+stdout+")(stderr="+stderr+")(count=1)\"\n";
        FileContents += "job_state_mask=31\n";
        FileContents += clientcode+" $gatekeeper_contact \"$spec\" $job_state_mask\n";

	// write it to a file

        int sl = FileContents.length();
        byte[] ftpbuf = new byte[sl];
        FileContents.getBytes(0,sl,ftpbuf,0);

        File to_file = new File(gramfile);
        if(to_file.exists()) {
          if(!to_file.canWrite())
             System.out.println("write failed: file "+gramfile+" is unwritable");
        }


        FileOutputStream to = null;
        try {
          to = new FileOutputStream(to_file);
          to.write(ftpbuf,0,sl);
        } catch (IOException e) {;}

        finally {
          if(to !=null) try {to.close();} catch (IOException e) {;}
        }

        // give it executable permissions and run it

 
    try {
         System.out.println("change file permission");
         Process p1 = Runtime.getRuntime().exec("chmod 777 "+gramfile);
         p1.waitFor();
         System.out.println("execute the script");
         System.out.println("/usr/bin/X11/xterm -d "+disp+":0 -e "+gramfile);
         Process p = Runtime.getRuntime().exec("/usr/bin/X11/xterm -d "+disp+":0 -e "+gramfile);
         p.waitFor();
    }
        catch (Exception e) {
        System.out.println("Error: " + e.toString());
    }

       if(outport.isConnected()) {
        System.out.println("GRAM: forwarding "+outFile);
       	outport.send(OutputFile);
      }
      else {
        System.out.println("CERIUS2 Output Port is not connected!");
      }
       }
  }
  
    
    public void destroy() {

    outport.destroyPort();
    inport.destroyPort();
    }
    
    public void stop() {}
    
    public void update_value(String s) {
    System.out.println("CERIUS2: new message: "+s);
    newmessage = s;
    dummy = disp;
    disp = s;
    if(newmessage.startsWith("GATE")) {
      gatekeeper = newmessage.substring(4);
      if(gatekeeper.equals("NPAC osprey2")) {
       contact="osprey2.npac.syr.edu:754:osprey2.npac.syr.edu-fork@globus.org";
      }
      if(gatekeeper.equals("NPAC osprey4")) {
       contact="osprey4.npac.syr.edu:754:osprey2.npac.syr.edu-fork@globus.org";
      }
      if(gatekeeper.equals("NCSA Origin2000 (f)")) {
        contact="modi4.ncsa.uiuc.edu:4001:modi4.ncsa.uiuc.edu-fork@globus.org";
      }
      if(gatekeeper.equals("NCSA Origin2000 (lsf)")) {
        contact="modi4.ncsa.uiuc.edu:4000:modi4.ncsa.uiuc.edu-lsf@globus.org";
      }
      idx=contact.indexOf(":");
      host=contact.substring(idx);
      disp=dummy;
    }  
    if(newmessage.startsWith("EXEC")) {
      executable = newmessage.substring(4);
      disp=dummy;
    }  
    if(newmessage.startsWith("ARGS")) {
      arguments = newmessage.substring(4);
      disp=dummy;
    }  


  }   

}	
    
    
    
