package webflow.modules.QuantumSimulations;

import webflow.backend.*;
import java.net.*;
import java.io.*;
import webflow.modules.Templates.*;

public class GAUS2PDB extends Template {
  
  InputPort inport;
  OutputPort outport;
  WFFileDescriptor inFile;
  WFFileDescriptor outFile = new WFFileDescriptor();

  //ftp stuff
  
  String localfile, remotefile, rhost;


  //any GRAM script has the following attributes
  String contact;   //now read as GATE message; (fixed)
                    //do not block; should come from MDS
  String directory; //this depends on application and contact (fixed)
                    //do not block;
                    //now read as DIRE message
  String executable;//as directory (fixed); do not block;
                    //now read as EXEC message
  String arguments; //for GAUS2PDB it is the input filename
                    //derived from input port (block)
  String stdin;     //read as STDI message; not used now
  String stdout;    //read as STDO message; do not block; fixed
  String stderr;    //read as STDE message; do not block; fixed

  String gatekeeper;//used in converting GATE message to full name of contact


  // GRAM CLIENT CODE aga.npac.syr.edu specific
  // GRAM client script

  String gramfile="/tmp/gramScript";  //name of the gram client script
                                      //to be generated on osprey2
  String GCD;
  String clientcodename, clientcode;
  String jobmask;
  String tmpdirectory;

  //GAUS2PDB SPECIFIC

  String host;  //derived from contact
  String message,newmessage;
  String command;
  String disp, dummy;
  int idx; //auxilliary
  boolean flag;

  public MetaModule initialize() {
    
    CreateServerSocket();
    MetaModule mm = new MetaModule("mm");
    outport = new OutputPort("String");
    mm.putPortID(outport.getPortID());
    inport = new InputPort("WFFileDescriptor");
    mm.putPortID(inport.getPortID());
    newmessage=null;
    disp="dummy";
    gatekeeper=null;
    contact=null;
    return(mm);
    }
  
  public void run() {
    
    System.err.println("GAUS2PDB: set display to: "+disp);

    while(true)
      {

	// note: fortran script gau2pdb takes gaussian.out as input
	//       and produces gaussian.pdb

     inFile  = (WFFileDescriptor)inport.receive();

     //-------------------------------------------------------------------
     // upload data

    localfile = inFile.getFileDirectory()+inFile.getFileName();

    //defuault outFile

    outFile.setFileName(inFile.getFileName());
    outFile.setFileDirectory(inFile.getFileDirectory());
    outFile.setFileAttribute(inFile.getFileAttribute());
    //outFile.FileHost set by the frontend

    //default remotefile

    remotefile = outFile.getFileDirectory()+outFile.getFileName();

     if(contact==null) { 
       contact="osprey4.npac.syr.edu:754:osprey4.npac.syr.edu-fork@globus.org";
       gatekeeper="NPAC osprey4";
     }
     idx=contact.indexOf(":");
     host=contact.substring(0,idx);


//String GCD="/home/B7D/globus/haupt"; //Gram Client Directory on osprey2
  String GCD="/usr/people/globus/GRAM"; //Gram Client Directory on knothead...
                   //when /tmp files deleted import from here
  //GCD Gram Client Directory (in case /tmp files are deleted)
  //depends on WebFlow server used
      
  //  String clientcodename="gram_client_test";  //solaris, globus <0.6.4


  if(gatekeeper.indexOf("NPAC")>-1) {
    // use on knothead (SGI); use Globus 0.6.3
      clientcodename="gram_client_test_irix"; 
      directory="/home/B8D/haupt"; // seen by osprey4
      tmpdirectory="/tmp/";      
      jobmask="3";
      executable="gau2pdb-solaris";
  }
  if(gatekeeper.indexOf("Origin")>-1) {
    // use on knothead (SGI); use Globus 0.6.4
      clientcodename="gram_client_test_irix.0.6.4";
      directory = "/scratch-modi4/haupt/GAUS2PDB";
      tmpdirectory = "/scratch-modi4/haupt/GAUS2PDB/";
      jobmask="31";
      executable="gau2pdb-irix";
  }

     clientcode="/tmp/"+clientcodename;  //on knothead
      // contact comes from the applet

    rhost = host;
    remotefile = tmpdirectory + "gaussian.out";

    String ftpfile="/tmp/ftpScript";   //ftp commands get,put, etc)
    String ftp0file="/tmp/ftp0";       //UNIX ftp command

        ftpscript(localfile, remotefile,ftpfile);
        ftp0script(rhost, ftpfile, ftp0file);

        //do ftp

 
	String ftpcmd = "/usr/bin/X11/xterm -d "+disp+":0 -e "+ftp0file;

        try {
            Process p = Runtime.getRuntime().exec(ftpcmd);
            p.waitFor();
        }
            catch (Exception e) {
            System.err.println("Error: " + e.toString());
        }

     // execute GAUS2PDB


    arguments=null;
    stdin=null;
    stdout=tmpdirectory+"gau2pdb_stdout";
    stderr=tmpdirectory+"gau2pdb_stderr";


    System.err.println("GAUS2PDB: writeGramScript");

     writeGramScript(GCD,clientcodename,directory,jobmask);

        // give it executable permissions and run it

     try {
         System.err.println("change file permission");
         Process p1 = Runtime.getRuntime().exec("chmod 777 "+gramfile);
         p1.waitFor();
         System.err.println("execute the script");
         System.err.println("/usr/bin/X11/xterm -d "+disp+":0 -e "+gramfile);
         Process p = Runtime.getRuntime().exec("/usr/bin/X11/xterm -d "+disp+":0 -e "+gramfile);
         p.waitFor();
    }
        catch (Exception e) {
        System.err.println("Error: " + e.toString());
    }

       if(outport.isConnected()) {
        outFile.setFileName("gaussian.pdb");
        outFile.setFileDirectory(directory+"/");
        outFile.setFileAttribute("CERIUS2");
        outFile.setFileHost(host);
        System.err.println("GRAM: forwarding "+outFile.getFileName());
       	outport.send(outFile);
      }
      else {
        System.err.println("GAUS2PDB Output Port is not connected!");
      }
       }
  }
  

  void ftpscript(String localfile, String remotefile, String ftpfile) {

        //generate ftpscript on the fly

        String ftpFileContents = new String("put "+localfile+" "+remotefile);
        int sl = ftpFileContents.length();
        byte[] ftpbuf = new byte[sl];
        ftpFileContents.getBytes(0,sl,ftpbuf,0);

        File to_file = new File(ftpfile);
        if(to_file.exists()) {
          if(!to_file.canWrite()) {
             System.err.println("FTP failed: file "+ftpfile+"is unwritable");
	  }
	}

        FileOutputStream to = null;
        try {
          to = new FileOutputStream(to_file);
          to.write(ftpbuf,0,sl);
        } catch (IOException e) {;}

        finally {
          if(to !=null) try {to.close();} catch (IOException e) {;}
        }
 
  } 


  void ftp0script(String rhost, String ftpfile, String ftp0file) {

        //generate ftp0 file on the fly

        String ftp0FileContents = "ftp "+rhost+ "< "+ftpfile;
        int sl0 = ftp0FileContents.length();
        byte[] ftp0buf = new byte[sl0];
        ftp0FileContents.getBytes(0,sl0,ftp0buf,0);

        File to_file0 = new File(ftp0file);
        if(to_file0.exists()) {
          if(!to_file0.canWrite()) {
             System.err.println("FTP failed: file "+ftp0file+"is unwritable");
	  }
	}

        FileOutputStream to0 = null;
        try {
          to0 = new FileOutputStream(to_file0);
          to0.write(ftp0buf,0,sl0);
        } catch (IOException e) {;}

        finally {
          if(to0 !=null) try {to0.close();} catch (IOException e) {;}
        }


        // give it executable permissions and run it

 
    try {
         System.err.println("change file permission");
         Process p1 = Runtime.getRuntime().exec("chmod 777 "+ftp0file);
         p1.waitFor();
    }
        catch (Exception e) {
        System.err.println("FTPin Error: " + e.toString());
    }



  }



  void writeGramScript(String GCD, String clientcodename, String directory,
                       String jobmask) {

     // make sure that the client code exists!

     File client=new File(clientcode);
     if(!client.exists()) {
         System.err.println("GRAM-client code does not exist");
         String copycmd="cp "+GCD+"/"+clientcodename+" "+clientcode;
         System.err.println(copycmd);
         try {
               Process p0 = Runtime.getRuntime().exec(copycmd);
               p0.waitFor();
	 } catch (Exception e) {
             System.err.println("Error: " + e.toString());
         }
     System.err.println("file copied"); 
     }


        //generate script on the fly

        String FileContents = new String("#!/bin/sh -f\n");
        FileContents += "gatekeeper_contact="+contact+"\n";
        FileContents += "spec=\"&(directory="+directory+")";
        FileContents += "(executable="+executable+")";

        if(arguments != null) { FileContents += "(arguments="+arguments+")";}
        if(stdin != null) { FileContents += "(stdin="+stdin+")";}

        FileContents += "(stdout="+stdout+")(stderr="+stderr+")(count=1)\"\n";
        FileContents += "job_state_mask="+jobmask+"\n";
        FileContents += clientcode+" $gatekeeper_contact \"$spec\" $job_state_mask\n";

	// write it to a file

        int sl = FileContents.length();
        byte[] ftpbuf = new byte[sl];
        FileContents.getBytes(0,sl,ftpbuf,0);

        File to_file = new File(gramfile);
        if(to_file.exists()) {
          if(!to_file.canWrite())
             System.err.println("write failed: file "+gramfile+" is unwritable");
        }


        FileOutputStream to = null;
        try {
          to = new FileOutputStream(to_file);
          to.write(ftpbuf,0,sl);
        } catch (IOException e) {;}

        finally {
          if(to !=null) try {to.close();} catch (IOException e) {;}
        }

    }
    
    public void destroy() {

    outport.destroyPort();
    inport.destroyPort();
    }
    
    public void stop() {}
    
    public void update_value(String newmessage) {
    System.err.println("GAUS2PDB: new message: "+newmessage);

    if(newmessage.startsWith("DISP")) {
      disp = newmessage.substring(4);
    } 
    if(newmessage.startsWith("GATE")) {
      gatekeeper = newmessage.substring(4);
      if(gatekeeper.equals("NPAC osprey2")) {
       contact="osprey2.npac.syr.edu:754:osprey2.npac.syr.edu-fork@globus.org";
      }
      if(gatekeeper.equals("NPAC osprey4")) {
       contact="osprey4.npac.syr.edu:754:osprey4.npac.syr.edu-fork@globus.org";
      }
      if(gatekeeper.equals("NCSA Origin2000 (f)")) {
        contact="modi4.ncsa.uiuc.edu:4001:modi4.ncsa.uiuc.edu-fork@globus.org";
      }
      if(gatekeeper.equals("NCSA Origin2000 (lsf)")) {
        contact="modi4.ncsa.uiuc.edu:4000:modi4.ncsa.uiuc.edu-lsf@globus.org";
      }
      idx=contact.indexOf(":");
      host=contact.substring(0,idx);
      outFile.setFileHost(host);
    }  
    if(newmessage.startsWith("EXEC")) {
      executable = newmessage.substring(4);
    }  
    if(newmessage.startsWith("ARGS")) {
      arguments = newmessage.substring(4);
    }  


  }   

}	
    
    
    
