package webflow.util;

//import java.awt.EventQueue;

/**
 * WFNEventQueue maintains a queue of events. Other components may post or
 * remove events from it. It is thread-safe, i.e. multiple threads may
 * access the queue simultaneously with no danger of race conditions. <BR>
 * Threads that call getEvent() when there are no events available, block, 
 * and are awakened by either threads that post an event, or threads that
 * have just been awakened (after being blocked waiting for an event) which
 * have noticed that there are more events available on the queue. 
 *
 * @version 0.1.0 first draft, April 25, 1997
 */
public class List extends Object 
{
  /** number of threads waiting for an event */
  int numberWaiting;

  int size = 0;

  /** head of the List */
  ListItem head = null;

  /** tail of the List */
  ListItem tail = null;

  /**
  * Creates a WFNEventQueue. The queue will be unusable until properly 
  * initialized. 
  */
  public List() 
  {
    head = null;
    tail = null;
    numberWaiting = 0;
    size = 0;

  }// end of constructor 

  /**
  * Returns the name of the component. 
  * @return "Queue" - the name of Queue
  */
  public String 
  getComponentName() 
  {
    return("List");
  }// end of getComponentName()

  /**
  */
  public String 
  toString()
  { return getComponentName(); }

  /**
  */
  public void 
  addItem(ListItem _newItem) throws NotAllowableListItemException
  {
     if( ( head == null ) || ( _newItem.getClass() == head.getClass() ) )
     {
        if(tail!=null) 
        {  tail.setSuccessor(_newItem);
           tail = _newItem;
        }
        else head = tail = _newItem;
  
        size++;

     } else throw new NotAllowableListItemException();

  }// end of addItem()

  /**
  */
  public void
  removeItem(ListItem _Item) throws IllegalArgumentException
  {
//     System.out.println(" in removeItem() ");
     if( head == null ) throw new IllegalArgumentException();
     else
       {
          ListItem curr = head;
          ListItem prev = head;
          boolean  found = false;

          while( curr != null && found == false )
          {
             prev = curr;

             if(curr.getObject() == _Item.getObject() ) found = true;
             else curr = curr.getSuccessor();

          }// end of while

          if( found == true )
          {
             if( curr == head ) 
             {    head = curr.getSuccessor();
                  if(curr == tail ) tail = curr.getSuccessor(); 
             }
             else
               { 
                 prev.setSuccessor(curr.getSuccessor());

                 if( curr == tail ) tail = prev; 

               }
             
             size--;

          } else throw new IllegalArgumentException();

       }

//       System.out.println(" end of removeItem() ");
      
  }// end of removeItem()

  /**
  */
  public ListItem
  getItem() throws ListEmptyException
  {
    if( head != null )
    {
      ListItem ret = head;

      if( head == tail ) 
           head = tail = null; 
      else 
           head = head.getSuccessor();

      size--;

      return ret;

    } else throw new ListEmptyException();

  }// end of getItem()

  /**
  */
  public ListItem
  peekItem() throws ListEmptyException
  {
    if( head != null ) 
         return head;
    else 
         throw new ListEmptyException();

  }// end of peekItem()

  /**
  */
  public boolean
  isEmpty()
  { return ( head == null ) ; }

  /**
  */
  public int
  getSize()
  { return size; }


}// end of class Queue
