package webflow.util;

/**
 * WFNEventQueue maintains a queue of events. Other components may post or
 * remove events from it. It is thread-safe, i.e. multiple threads may
 * access the queue simultaneously with no danger of race conditions. <BR>
 * Threads that call getEvent() when there are no events available, block, 
 * and are awakened by either threads that post an event, or threads that
 * have just been awakened (after being blocked waiting for an event) which
 * have noticed that there are more events available on the queue. 
 *
 * @version 0.1.0 first draft, April 25, 1997
 */
public class Queue extends Object 
{
  /** number of threads waiting for an event */
  int numberWaiting;

  /** head of the queue */
  QueueItem head = null;

  /** tail of the queue */
  QueueItem tail = null;

  /** */
  int Size = 0;

  /**
  * Creates a WFNEventQueue. The queue will be unusable until properly 
  * initialized. 
  */
  public Queue() 
  {
    head = null;
    tail = null;
    numberWaiting = 0;
    Size = 0;

  }// end of constructor 

  /**
  * Returns the name of the component. 
  * @return "Queue" - the name of Queue
  */
  public String 
  getComponentName() 
  {
    return("Queue");
  }// end of getComponentName()

  public String 
  toString()
  { return getComponentName(); }

  /**
  * Adds the new element to the queue.
  * @param _newItem New element
  */
  public synchronized void 
  addItem(QueueItem _newItem) throws NotAllowableQueueItemException
  {
     if( ( head == null ) || ( _newItem.getClass() == head.getClass() ) )
     {
        if(tail!=null) 
        {  tail.setSuccessor(_newItem);
           tail = _newItem;
        }
        else head = tail = _newItem;

        Size++;

     } else throw new NotAllowableQueueItemException();

  }// end of addItem()

  /**
  * Extracts the element at the header of the queue.
  * @return  Element at the header of the queue.
  */
  public synchronized QueueItem
  getItem() throws QueueEmptyException
  {
    if( head != null )
    {
      QueueItem ret = head;

      if( head == tail ) 
           head = tail = null; 
      else 
           head = head.getSuccessor();
      
      Size--;

      return ret;

    } else 
      { 
         System.out.println(" QueueEmptyException ");
         throw new QueueEmptyException();
      }

  }// end of getItem()

  /**
  * Brings the first element of the queue.
  * @return Element at the header of the queue.
  */
  public synchronized QueueItem
  peekItem() throws QueueEmptyException
  {
    if( head != null ) 
         return head;
    else 
         throw new QueueEmptyException();

  }// end of peekItem()

  /**
  * Whether the queue is empty or not.
  * @return true - if the queue is empty. <br>
  *         false - if the queue is NOT empty.
  */ 
  public synchronized boolean
  isEmpty()
  { return ( head == null ) ; }

  /** 
  * Returns the size of the queue.
  * @return The number of elements in the queue.
  */
  public synchronized int
  getSize()
  {  return Size; }

}// end of class Queue
