package webflow.util.Workers;

import java.util.Vector;

import webflow.util.List;

import webflow.util.ListItem;

/**
* WorkerPool stores and/or produces worker threads. 
* Worker threads are recycled when they are no longer used <BR>
*
* @version 0.1.0 first draft, April 21, 1997
* @version 0.1   completed , August 11, 1997 (Hasan Timucin OZDEMIR)
*/
public class 
WorkerPool extends Thread 
{

  /** Number of Maximum Worker objects */
  int NumberOfWorkers = 50;
 
  /** Keeps the Worker Object in one List */
  Vector   WorkerList      = null;

  /** Keeps the available Worker Objects  */
  Vector   AvailableIndex  = null;
 
  public static final String AVAILABLE = "Available";

  public static final String     BUSSY = "Bussy";

  webflow.util.List BussyQueue  = null;

  private int timeSlice = 100;

  private static boolean DEBUG = false;

  /**
  * Creates a minimum WorkerPool. 
  */
  public 
  WorkerPool() 
  {
     createObject();

  }// end of constructor

  /*
  * Creates a WorkerPool with given size. Note that if the 
  * given size is less then the minimum size of the Pool 
  * then it uses the minimum size.
  * @param _size The size of the WorkerPool 
  */
  public 
  WorkerPool(int _size)
  {
     NumberOfWorkers = Math.max(5, Math.min(NumberOfWorkers, _size) );

     createObject();

  }// end of constructor

  /*
  * Creates the necessary Worker Tables.
  */
  private void
  createObject()
  {
     WorkerList = new Vector(NumberOfWorkers);

     AvailableIndex = new Vector(NumberOfWorkers);
     
     /***********************************/
     /** Create an empty Worker objects */
     /***********************************/

     for(int i=0;i <NumberOfWorkers; i++)
         WorkerList.insertElementAt(new Worker(this), i);

     /*************************************/
     /** Create the Available List Object */
     /*************************************/

     for(int i=0;i <NumberOfWorkers; i++) 
         AvailableIndex.insertElementAt(AVAILABLE,i);  

     BussyQueue = new webflow.util.List();

  }// end of createObject()

  /**
  * Supplies a worker thread. A worker thread is picked from the pool, and
  * supplied to the caller. If no workers are available in the pool, 
  * then WorkerPoolFullException is going to be raised.
  * @return the activated worker thread, or null if none could be created
  */
  public synchronized Worker 
  getWorker() throws WorkerPoolFullException 
  {
    int WorkerIndex = AvailableIndex.indexOf(AVAILABLE);
 
    if(WorkerIndex>=0 && WorkerIndex<NumberOfWorkers)
    {

      AvailableIndex.setElementAt(BUSSY,WorkerIndex);

      try
      { 
        BussyQueue.addItem(new ListItem(WorkerList.elementAt(WorkerIndex)));

      } catch(Exception e) 
        {
          System.out.println(" Exception at freeWorker : "+e.getMessage());
        };

      return (Worker) WorkerList.elementAt(WorkerIndex);
      
    } else throw new WorkerPoolFullException();

  }// end of getWorker()

  /**
  * Returns a worker thread to the pool of available workers. 
  * If the provided info about the Worker is not valid, then
  * it raises UnknownWorkerException.
  * @param worker the worker to be made available
  */
  public synchronized void 
  freeWorker(Worker worker) throws UnknownWorkerException 
  {
    int WorkerIndex = WorkerList.indexOf(worker);
   
    if(WorkerIndex>=0 && WorkerIndex<NumberOfWorkers)
    {

      try
      {  
         BussyQueue.removeItem(new ListItem(WorkerList.elementAt(WorkerIndex)));

      } catch(Exception e) 
        {
          System.out.println(" Exception at freeWorker : "+e.getMessage());
        };

      AvailableIndex.setElementAt(AVAILABLE,WorkerIndex);

    } else throw new UnknownWorkerException();

  }// end of freeWorker()

  /**
  * Prints the current status of the Worker objects in the pool.
  */
  public synchronized void
  printStatusOfWorkers()
  {
     int i;
     for(i=0;i<NumberOfWorkers;i++)
        System.out.println(i+" status : "+(String)AvailableIndex.elementAt(i));

  }// end of while

  /**
  * Returns the name of the component. 
  * @return "WorkerPool" - the name of WorkerPool
  */
  public String 
  getComponentName() 
  {
    return("WorkerPool");
  }// end of getComponentName()

  /**
  * Notifies the WorkerPool object.
  */
  public synchronized void 
  wakeup()
  { notify(); 
  }// end of wakeup()

  /**
  * Thread interface.
  */
  public synchronized void
  run()
  {
     ListItem currentListItem = null;
     Worker   current; 
     boolean there_is_no_thread = true;

     while( true )
     {
        there_is_no_thread = true;

        while( there_is_no_thread )
        {
           if( BussyQueue.getSize() > 0 )
           {  
              try
              { 
                 currentListItem = BussyQueue.peekItem();
             
                 there_is_no_thread = ( ((Worker) currentListItem.getObject()).isActionSet() == false );

              } catch (Exception e) {}

           }// end of if
 
           
           if( there_is_no_thread )
           {
               try
               { wait(2*timeSlice);
               } catch(Exception e) 
                 {  System.out.println(" WorkerPool: I cant' wait() "+e.getMessage()); 
                 };

               if(DEBUG) System.out.println(" WorkerPool: I woke up");
               if(DEBUG) printStatusOfWorkers();

           }// end of if

        }// end of while

        if(DEBUG) System.out.println(" WorkerPool: BussyQueue size :"+BussyQueue.getSize());

        boolean SetPriorityOK = true;

        while( currentListItem != null )
        {
           /**************************/
           /** Get the Worker object */
           /**************************/
           current = (Worker) currentListItem.getObject();
           SetPriorityOK = true;
        
           try 
           { 
              if(current.isActionSet()) current.setPriority(4);
              else SetPriorityOK = false;

           } catch(Exception e) 
             {
                try { freeWorker(current); }
                catch(Exception e2) {};

                SetPriorityOK = false;
             }  
  
           /******************************/
           /** If this guy has an action */
           /******************************/
           if(SetPriorityOK == true)
           {

             /**************************************/
             /** Give sometime for the new process */
             /**************************************/
             try
             {
                wait(timeSlice);

             } catch(InterruptedException ie) 
               {}


             /*****************************************/
             /** Decrease the priority of this thread */
             /*****************************************/
             try 
             { 
                current.setPriority(2);

             } catch(Exception e) 
               { 
                  try { freeWorker(current); }
                  catch(Exception e2) {};
               } 

             if(DEBUG) printStatusOfWorkers();

           }// end of if

           /*********************************************/
           /** Proceed to the next element on the queue */
           /*********************************************/

           currentListItem = currentListItem.getSuccessor();

        }// end of while

     }// end of while

  }// end of run()

}// end of WorkerPool



