package webflow.util.Workers;

import  webflow.util.*;

/**
*/
public class
WorkerPoolManager extends Thread
{
  /** WorkerPool object for Workers */
  WorkerPool WPool  = null;

  /** A List of actions to be sent to WorkerPool */
  Queue  ActionWaitingQueue = null;

  /** Default WorkerPool Size */
  int Size = 50;

  private static boolean DEBUG = false;

  /**
  * Defines a WorkerPoolManager with 50 workers.
  */
  public
  WorkerPoolManager()
  {
     createObject();

  }// end of constructor()

  /**
  * Defines a WorkerPoolManager with _size workers.
  * @param _size Number of workers. Should be between 5 and 50.
  */
  public
  WorkerPoolManager(int _size)
  { 
    Size = Math.max(5, Math.min(50,_size) );

    createObject();

  }// end of constructor()

  /**
  * Actual definition is done here.
  * Allocate WorkerPool and define the Waiting List.
  */
  void
  createObject()
  {
     WPool = new WorkerPool(Size);

     ActionWaitingQueue = new Queue();
     
     start();

     WPool.start();

  }// end of createObject()

  /**
  * Action to be taken care of. 
  * @param newAction It is going to be done here.
  */
  public synchronized void
  put(Action newAction)
  {
    try 
    {   ActionWaitingQueue.addItem( new QueueItem(newAction));
    } catch(Exception e) {}

    if(DEBUG) System.out.println(" I put the action !");

    /****************************************************/
    /** Notify that we put something to the ActionQueue */
    /****************************************************/
    notify();

  }// end of put()
  

  /** 
  * In this thight loop, Manager looks waiting list.
  * If there are some actions in this waiting list, then
  * it will take care of it.
  * If there is nothing in the waiting list, then it just
  * waits for a while.
  */
  public synchronized void
  run()
  {
    
     while( true )
     {
        if(DEBUG) System.out.println(" Manager : Queue size =" + ActionWaitingQueue.getSize());

        if( ActionWaitingQueue.getSize() > 0)
        {
           if(DEBUG) System.out.println(" Manager : There is an action ");
           Action act  =  null;
           Worker curr = null;

           try
           {   

               /*****************************************/
               /** Asks a Worker object for this action */
               /*****************************************/

               curr = WPool.getWorker();
               if(DEBUG) System.out.println(" Manager : I got the Worker ");

               /***************************************/
               /** Gets the first Action of the Queue */
               /***************************************/

               act = (Action) (ActionWaitingQueue.getItem()).getObject();
               if(DEBUG) System.out.println(" Manager : I got the action ");

               /*******************************************/
               /** This might couse a problem since WPool */
               /** runnning in different Thread           */
               /*******************************************/

               curr.service(act);
               if(DEBUG)
               {
                  System.out.println(" Manager : I set the service ");
                  System.out.print(" Manager : After getItem() Queue size :");
                  System.out.println(ActionWaitingQueue.getSize());
               }// end of if 

           } 
             catch(WorkerPoolFullException wpfe)
             {   
                System.out.println(" Exception at WorkerPoolManager : WorkerPoolFullException ");
             }
             catch(QueueEmptyException lee)
             {  
                System.out.println(" Exception at WorkerPoolManager : QueueEmptyException ");
                try 
                {  WPool.freeWorker(curr); 
                } catch(UnknownWorkerException uwe) {}

             }
             catch(Exception e) 
             {
                System.out.println(" Exception at WorkerPoolManager : "+e.getMessage());
                e.printStackTrace();
             }
           
        }
        else
         {
            /**********************/
            /** Sleep for a while */
            /**********************/

            if(DEBUG) System.out.println(" WorkerPoolManager : I am goin to sleep ");

            try
            {  wait(1000); 
            } catch ( Exception e ) 
              { 
                 System.out.println(" Exception at WorkerPoolManager : "+e.getMessage());
                 e.printStackTrace();
              };

            if(DEBUG) System.out.println(" WorkerPoolManager : I woke up ");
         }
 
     }// end of while

  }// end of run

}// end of class WorkerPoolManager
