
/*
 * %W% %E%
 *
 * Copyright (c) 1993-1997 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package webflow.modules.demo.collab;

import SharedData.*;
import java.awt.*;
import java.applet.Applet;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to make sure that the Shared Data classes have
 * all the required API in them.
 *
 * It's based on the chat demonstration which is part of the
 * west-coast rmi distribution.
 *
 * @version     0.92 27 February 1997
 * @author      Rich Burridge
 */

public class
ChatUserApp extends Panel implements ChatDebugFlags {
    Button    setNameButton;
    Button    sayButton;
    Button    signOffButton;
    TextField nameField;
    TextField typeField;
    TextArea  messageArea;

    /** The name of this Client. */
    protected String name;

    /** The chat session that this client application will join. */
    private Session session;

    /** The client that will be joining the session and channel. */
    private ChatClient client;

    /** The channel that this client application will use to send data. */
    private Channel channel;
 
    /** The consumer of all data sent over the channel. */
    private ChatConsumer chatConsumer;

    /** The data that will be sent over the channel. */
    private Data data;

    /** Indicates if the client is successfully connected to the server. */
    boolean connected = false;

/* Variables set from command line options. */

    int     width       = 0;
    int     height      = 0;
    String  hostname    = null;
    int     hostport    = 0;
    String  sessionType = null;


    public
    ChatUserApp() {
        if (ChatUserApp_Debug) {
            System.out.println("ChatUserApp: constructor.");
        }
    }


    public synchronized void
    initialize() {
        Panel              p;
        Font               lbl;
        GridBagLayout      g;
        GridBagConstraints c;

        if (ChatUserApp_Debug) {
            System.out.println("ChatUserApp: initialize.");
        }

        lbl = new Font("Helvetica", Font.BOLD, 14);
        g   = new GridBagLayout();
        c   = new GridBagConstraints();

	c.fill      = GridBagConstraints.NONE;
	c.weightx   = 1.0;
	c.gridwidth = GridBagConstraints.REMAINDER;
 
	setLayout(g);
 
	p = new Panel();
	g.setConstraints(p, c);
	add(p);
	p.add(setNameButton = new Button("Set Name"));
	p.add(nameField     = new TextField(20)).setFont(lbl);
	p.add(signOffButton = new Button("SignOff"));
	signOffButton.disable();
 
	p = new Panel();
	g.setConstraints(p, c);
	add(p);
	p.add(new Label("Messages: "));
	p.add(messageArea = new TextArea(6, 30));
 
	p = new Panel();
	g.setConstraints(p, c);
	add(p);
	p.add(sayButton = new Button("Say"));
	sayButton.disable();
	p.add(typeField = new TextField(20));
    }


    private boolean
    isConnected() {
        if (ChatUserApp_Debug) {
            System.out.println("ChatUserApp: isConnected: " + connected);
        }

        return(connected);
    }


    private boolean
    connect() {
	if (ChatUserApp_Debug) {
            System.out.println("ChatUserApp: connect.");
        }

	if (name == null) {
	    throw new Error("need a non-null name to connect");
	}  

	try {
	    String sessionName = "ChatSession";

/* Create a chat client. */

            System.out.println("Creating a ChatMember...");
	    client = new ChatClient(name);

/* Resolve the chat session. */

	    try {
                String url = "coll://" + hostname + ":" + hostport +
			     "/" + sessionType + "/Session/" + sessionName;

		session = SessionFactory.createSession(url);

/*  Join the session and a named channel and set the channels data consumer. */

		session.join(client);
		channel = Channel.join(session, "ChatChannel", client);
		chatConsumer = new ChatConsumer(client.getName(), messageArea);
		channel.setConsumer(client, chatConsumer);

	        repaint();
            } catch (Exception e) {
                System.out.print("Caught exception in ");
		System.out.println("ChatUserApp.connect: " + e);
		if (ChatUserApp_Debug) {
		    e.printStackTrace();
		}
		return(false);
            }
        } catch (Throwable th) {
            System.err.println("ChatUserApp: connect caught: " + th);
            if (ChatUserApp_Debug) {
		th.printStackTrace();
            }
	    throw new Error("SetClient.start failed : " + th);
        }
	return(true);
    }


/*  Convert the users message to bytes and send it over the channel. */

    private void
    writeLine(String message) {
        if (ChatUserApp_Debug) {
            System.out.println("ChatUserApp: writeLine: " + message);
        }

	if (isConnected()) {
            try {
		data = new Data(message);
		data.setPriority(Channel.HIGH_PRIORITY);
                channel.sendToAll(client, data);
            } catch (Exception e) {
                System.out.print("Caught exception in ");
                System.out.println("ChatUserApp.writeLine(): " + e);
                if (ChatUserApp_Debug) {
		    e.printStackTrace();
		}
            }
	}
    }


    private void
    disconnect() {
        try {
            if (ChatUserApp_Debug) {
                System.out.println("ChatUserApp: disconnect.");
            }
/* Leave the channel and leave the session. */

            channel.leave(client);
            session.leave(client);
        } catch (Exception e) {
            System.err.println("Caught exception while trying to " +
			       "disconnect from chat server: " + e);
            if (ChatUserApp_Debug) {
		e.printStackTrace();
	    }
        }
    }


    public boolean
    action(Event event, Object obj) {
        if (ChatUserApp_Debug) {
            System.out.println("ChatUserApp: action.");
        }

	if ("SignOff".equals(obj)) {
	    sayButton.disable();
	    signOffButton.disable();
	    setNameButton.disable();
	    disconnect();
	    return(true);
	} else if ("Say".equals(obj)) {

// Say the message field info to all the players

	    if (session == null) {
		return(true);
	    } 
	    try {
		writeLine(typeField.getText());
	    } catch (Exception e) {
		System.err.println("Caught exception while invoking Say: " + e);
                if (ChatUserApp_Debug) {
		    e.printStackTrace();
		}
	    } 
	    return(true);
	} else if ("Set Name".equals(obj)) {

/* Set the name */

	    name = nameField.getText();
	    if (name == null) {
		nameField.setText("Need to give a name before \"Set Name\"");
		return(true);
	    } else {
		connected = connect();
		nameField.setEditable(false);
		setNameButton.disable();
		signOffButton.enable();
		sayButton.enable();
	    }
	    return(true);
	} 
        return(false);
    }


    private void usage() {
        System.out.println("usage: ChatUserApp [options]");
        System.out.println("-width  : application width.");
        System.out.println("-height : application height.");
        System.out.println("-server : server hostname or multicast address.");
        System.out.println("-port   : port number.");
        System.out.println("-type   : session type.");
    }


    void
    getArgs(String args[]) {
        if (ChatUserApp_Debug) {
            System.out.println("ChatUserApp: getArgs:" +
				" args: " + args);
        }

        for (int i = 0 ; i < args.length ; i++) {
            if (args[i].equals("-width")) {
                try {
                    width = Integer.parseInt(args[++i]);
                } catch (NumberFormatException e) {
                    System.out.println("invalid width [" + args[i] + "]");
                    width = 500;
                }
            } else if (args[i].equals("-height")) {
                try {
                    height = Integer.parseInt(args[++i]);
                } catch (NumberFormatException e) {
                    System.out.println("invalid height [" + args[i] + "]");
                    height = 500;
                }
            } else if (args[i].equals("-server")) {
                if (++i < args.length) {
		    hostname = args[i];
		} else {
		    System.out.println("no server name given.");
		    hostname = "stard";
		}
            } else if (args[i].equals("-port")) {
                try {
                    hostport = Integer.parseInt(args[++i]);
                } catch (NumberFormatException e) {
                    System.out.println("invalid port [" + args[i] + "]");
                    hostport = 4461;
                }
            } else if (args[i].equals("-type")) {
                if (++i < args.length) {
                    sessionType = args[i];
                } else {
                    System.out.println("no session type given.");
                    sessionType = "socket";
                }
            } else if (args[i].startsWith("-")) {
                System.out.println("unknown option: [" + args[i] + "]");
                usage();
                System.exit(1);
            }
	}
    }


    void
    makeFrame(ChatUserApp chatUser) {
        Frame frame = new Frame();
 
        if (ChatUserApp_Debug) {
            System.out.println("ChatUserApp: makeFrame.");
        }
 
        frame.setTitle("Chat User");
        frame.add(chatUser);
        frame.pack();
        frame.setSize(width, height);
        frame.show();
    }


    public static void 
    main(String args[]) {
        ChatUserApp chatUser = new ChatUserApp();

        if (ChatUserApp_Debug) {
            System.out.println("ChatUserApp: main.");
        }

	chatUser.getArgs(args);
	chatUser.initialize();
	chatUser.makeFrame(chatUser);
    }
}
